@@ -48,6 +48,7 @@
 #include <list>
 #include <cmath>
 
+#include <boost/utility.hpp>
 
 #include "pbd/xml++.h"
 #include <gtkmm/box.h>
@@ -82,6 +83,7 @@
 #include "video_timeline.h"
 
 #include "widgets/ardour_button.h"
+#include "widgets/ardour_dropdown.h"
 #include "widgets/ardour_spacer.h"
 
 #include "add_route_dialog.h"
@@ -103,14 +105,18 @@
 #include "big_clock_window.h"
 #include "big_transport_window.h"
 #include "bundle_manager.h"
+#include "dsp_stats_window.h"
 #include "engine_dialog.h"
 #include "export_video_dialog.h"
 #include "global_port_matrix.h"
 #include "idleometer.h"
+#include "io_plugin_window.h"
 #include "keyeditor.h"
 #include "location_ui.h"
 #include "lua_script_manager.h"
+#include "luawindow.h"
 #include "plugin_dspload_window.h"
+#include "plugin_manager_ui.h"
 #include "rc_option_editor.h"
 #include "route_dialogs.h"
 #include "route_params_ui.h"
@@ -118,6 +124,7 @@
 #include "speaker_dialog.h"
 #include "transport_masters_dialog.h"
 #include "virtual_keyboard_window.h"
+#include "library_download_dialog.h"
 #else
 class About;
 class AddRouteDialog;
@@ -130,15 +137,20 @@ class ExportVideoDialog;
 class KeyEditor;
 class LocationUIWindow;
 class LuaScriptManager;
+class LuaWindow;
 class RCOptionEditor;
 class RouteParams_UI;
 class SessionOptionEditor;
 class SpeakerDialog;
 class GlobalPortMatrixWindow;
 class IdleOMeter;
+class IOPluginWindow;
 class PluginDSPLoadWindow;
+class PluginManagerUI;
+class DspStatisticsWindow;
 class TransportMastersWindow;
 class VirtualKeyboardWindow;
+class LibraryDownloadDialog;
 #endif
 
 class VideoTimeLine;
@@ -149,6 +161,8 @@ class DuplicateRouteDialog;
 class MainClock;
 class Mixer_UI;
 class PublicEditor;
+class RecorderUI;
+class TriggerPage;
 class SaveAsDialog;
 class SaveTemplateDialog;
 class SessionDialog;
@@ -210,6 +224,7 @@ public:
 	void load_from_application_api (const std::string& path);
 	void finish();
 
+	int copy_demo_sessions ();
 	int load_session (const std::string& path, const std::string& snapshot, std::string mix_template = std::string());
 	bool session_load_in_progress;
 	int build_session (std::string const& path, std::string const& snapshot, std::string const& session_template, ARDOUR::BusProfile const&, bool from_startup_fsm, bool unnamed);
@@ -229,7 +244,7 @@ public:
 	int load_session_from_startup_fsm ();
 
 	/// @return true if session was successfully unloaded.
-	int unload_session (bool hide_stuff = false);
+	int unload_session (bool hide_stuff = false, bool force_unload = false);
 	void close_session();
 
 	int  save_state_canfail (std::string state_name = "", bool switch_to_it = false);
@@ -252,6 +267,7 @@ public:
 	void toggle_editing_space();
 	void toggle_mixer_space();
 	void toggle_keep_tearoffs();
+	void show_plugin_manager();
 
 	void reset_focus (Gtk::Widget*);
 
@@ -262,7 +278,7 @@ public:
 	 *
 	 *  (either RapidScreenUpdate || SuperRapidScreenUpdate - user-config)
 	 */
-	static sigc::signal<void, samplepos_t> Clock;
+	static sigc::signal<void, Temporal::timepos_t> Clock;
 
 	static void close_all_dialogs () { CloseAllDialogs(); }
 	static sigc::signal<void> CloseAllDialogs;
@@ -271,7 +287,10 @@ public:
 	XMLNode* editor_settings() const;
 	XMLNode* preferences_settings() const;
 	XMLNode* mixer_settings () const;
+	XMLNode* trigger_page_settings () const;
+	XMLNode* recorder_settings () const;
 	XMLNode* keyboard_settings () const;
+	XMLNode* clock_mode_settings () const;
 	XMLNode* tearoff_settings (const char*) const;
 
 	void save_ardour_state ();
@@ -290,8 +309,6 @@ public:
 
 	VideoTimeLine *video_timeline;
 
-	void store_clock_modes ();
-	void restore_clock_modes ();
 	void reset_main_clocks ();
 
 	void synchronize_sync_source_and_video_pullup ();
@@ -313,11 +330,11 @@ public:
 	void export_video (bool range = false);
 
 	void session_add_audio_route (bool, int32_t, int32_t, ARDOUR::TrackMode, ARDOUR::RouteGroup *,
-	                              uint32_t, std::string const &, bool, ARDOUR::PresentationInfo::order_t order);
+	                              uint32_t, std::string const &, bool, ARDOUR::PresentationInfo::order_t order, bool trigger_visibility);
 
 	void session_add_midi_route (bool, ARDOUR::RouteGroup *, uint32_t, std::string const &, bool,
 	                             ARDOUR::PluginInfoPtr, ARDOUR::Plugin::PresetRecord*,
-	                             ARDOUR::PresentationInfo::order_t order);
+	                             ARDOUR::PresentationInfo::order_t order, bool trigger_visibility);
 
 	void session_add_foldback_bus (int32_t, uint32_t, std::string const &);
 
@@ -333,6 +350,7 @@ public:
 
 	void show_ui_prefs ();
 	void show_mixer_prefs ();
+	void show_plugin_prefs ();
 
 	bool check_audioengine(Gtk::Window&);
 
@@ -348,6 +366,8 @@ public:
 	void reset_route_peak_display (ARDOUR::Route*);
 	void reset_group_peak_display (ARDOUR::RouteGroup*);
 
+	void show_library_download_window();
+
 	const std::string& announce_string() const { return _announce_string; }
 
 	void hide_application ();
@@ -408,6 +428,8 @@ private:
 	Gtk::Notebook _tabs;
 	PublicEditor*  editor;
 	Mixer_UI*      mixer;
+	RecorderUI*    recorder;
+	TriggerPage*   trigger_page;
 	Gtk::Tooltips _tooltips;
 	NSM_Client*    nsm;
 	bool          _was_dirty;
@@ -425,9 +447,9 @@ private:
 	void tabbable_state_change (ArdourWidgets::Tabbable&);
 
 	void toggle_meterbridge ();
-	void toggle_luawindow ();
 
 	int  setup_windows ();
+	void apply_window_settings (bool);
 	void setup_transport ();
 	void setup_clock ();
 
@@ -458,6 +480,10 @@ private:
 	void session_dirty_changed ();
 	void update_title ();
 
+	void cue_rec_state_changed ();
+	void cue_rec_state_clicked ();
+	void cue_ffwd_state_clicked ();
+
 	void map_transport_state ();
 	int32_t do_engine_start ();
 
@@ -497,18 +523,15 @@ private:
 
 	TransportControlUI transport_ctrl;
 
-	ArdourWidgets::ArdourButton punch_in_button;
-	ArdourWidgets::ArdourButton punch_out_button;
-	ArdourWidgets::ArdourButton layered_button;
+	ArdourWidgets::ArdourButton   punch_in_button;
+	ArdourWidgets::ArdourButton   punch_out_button;
+	ArdourWidgets::ArdourDropdown record_mode_selector;
 
 	ArdourWidgets::ArdourVSpacer recpunch_spacer;
-	ArdourWidgets::ArdourVSpacer monitoring_spacer;
 	ArdourWidgets::ArdourVSpacer latency_spacer;
 	ArdourWidgets::ArdourVSpacer monitor_spacer;
-
-	ArdourWidgets::ArdourButton monitor_in_button;
-	ArdourWidgets::ArdourButton monitor_disk_button;
-	ArdourWidgets::ArdourButton auto_input_button;
+	ArdourWidgets::ArdourVSpacer scripts_spacer;
+	ArdourWidgets::ArdourVSpacer cuectrl_spacer;
 
 	ArdourWidgets::ArdourButton monitor_dim_button;
 	ArdourWidgets::ArdourButton monitor_mono_button;
@@ -518,7 +541,6 @@ private:
 	Gtk::Label   layered_label;
 
 	Gtk::Label   punch_space;
-	Gtk::Label   mon_space;
 
 	void toggle_external_sync ();
 	void toggle_time_master ();
@@ -527,6 +549,9 @@ private:
 
 	ArdourWidgets::ArdourButton latency_disable_button;
 
+	ArdourWidgets::ArdourButton  _cue_rec_enable;
+	ArdourWidgets::ArdourButton  _cue_play_enable;
+
 	Gtk::Label route_latency_value;
 	Gtk::Label io_latency_label;
 	Gtk::Label io_latency_value;
@@ -580,7 +605,7 @@ private:
 	void audition_alert_clicked ();
 	bool error_alert_press (GdkEventButton *);
 
-	void layered_button_clicked ();
+	void set_record_mode (ARDOUR::RecordMode);
 
 	void big_clock_value_changed ();
 	void primary_clock_value_changed ();
@@ -672,9 +697,9 @@ private:
 	void transport_play_preroll();
 	void transport_rec_preroll();
 	void transport_rec_count_in();
-	void transport_forward (int option);
-	void transport_rewind (int option);
-	void transport_ffwd_rewind (int option, int dir);
+	void transport_forward ();
+	void transport_rewind ();
+	void transport_ffwd_rewind (bool fwd);
 	void transport_loop ();
 	void toggle_roll (bool with_abort, bool roll_out_of_bounded_mode);
 	bool trx_record_enable_all_tracks ();
@@ -703,11 +728,11 @@ private:
 	int         create_mixer ();
 	int         create_editor ();
 	int         create_meterbridge ();
-	int         create_luawindow ();
 	int         create_masters ();
+	int         create_recorder ();
+	int         create_trigger_page ();
 
 	Meterbridge  *meterbridge;
-	LuaWindow    *luawindow;
 
 	/* Dialogs that can be created via new<T> */
 
@@ -723,7 +748,10 @@ private:
 	WM::Proxy<ExportVideoDialog> export_video_dialog;
 	WM::Proxy<LuaScriptManager> lua_script_window;
 	WM::Proxy<IdleOMeter> idleometer;
+	WM::Proxy<IOPluginWindow> io_plugin_window;
+	WM::Proxy<PluginManagerUI> plugin_manager_ui;
 	WM::Proxy<PluginDSPLoadWindow> plugin_dsp_load_window;
+	WM::Proxy<DspStatisticsWindow> dsp_statistics_window;
 	WM::Proxy<TransportMastersWindow> transport_masters_window;
 
 	/* Windows/Dialogs that require a creator method */
@@ -734,9 +762,11 @@ private:
 	WM::ProxyWithConstructor<BigClockWindow> big_clock_window;
 	WM::ProxyWithConstructor<BigTransportWindow> big_transport_window;
 	WM::ProxyWithConstructor<VirtualKeyboardWindow> virtual_keyboard_window;
+	WM::ProxyWithConstructor<LibraryDownloadDialog> library_download_window;
 	WM::ProxyWithConstructor<GlobalPortMatrixWindow> audio_port_matrix;
 	WM::ProxyWithConstructor<GlobalPortMatrixWindow> midi_port_matrix;
 	WM::ProxyWithConstructor<KeyEditor> key_editor;
+	WM::ProxyWithConstructor<LuaWindow> luawindow;
 
 	/* creator methods */
 
@@ -746,8 +776,10 @@ private:
 	BigClockWindow*         create_big_clock_window();
 	BigTransportWindow*     create_big_transport_window();
 	VirtualKeyboardWindow*  create_virtual_keyboard_window();
+	LibraryDownloadDialog*  create_library_download_window();
 	GlobalPortMatrixWindow* create_global_port_matrix (ARDOUR::DataType);
 	KeyEditor*              create_key_editor ();
+	LuaWindow*              create_luawindow ();
 
 	ARDOUR::SystemExec *video_server_process;
 
@@ -777,11 +809,11 @@ private:
 	void flush_trash ();
 
 	bool have_configure_timeout;
-	ARDOUR::microseconds_t last_configure_time;
+	PBD::microseconds_t last_configure_time;
 	gint configure_timeout ();
 
-	ARDOUR::microseconds_t last_peak_grab;
-	ARDOUR::microseconds_t last_shuttle_request;
+	PBD::microseconds_t last_peak_grab;
+	PBD::microseconds_t last_shuttle_request;
 
 	bool have_disk_speed_dialog_displayed;
 	void disk_speed_dialog_gone (int ignored_response, Gtk::MessageDialog*);
@@ -803,7 +835,7 @@ private:
 
 	void editor_realized ();
 
-	std::vector<std::string> positional_sync_strings;
+	std::vector<std::string> record_mode_strings;
 
 	void toggle_use_mmc ();
 	void toggle_send_mmc ();
@@ -858,6 +890,7 @@ private:
 
 	void successful_graph_sort ();
 	bool _feedback_exists;
+	bool _ambiguous_latency;
 
 	enum ArdourLogLevel {
 		LogLevelNone = 0,
@@ -892,6 +925,8 @@ private:
 	ArdourWidgets::ArdourButton editor_visibility_button;
 	ArdourWidgets::ArdourButton mixer_visibility_button;
 	ArdourWidgets::ArdourButton prefs_visibility_button;
+	ArdourWidgets::ArdourButton recorder_visibility_button;
+	ArdourWidgets::ArdourButton trigger_page_visibility_button;
 
 	bool key_press_focus_accelerator_handler (Gtk::Window& window, GdkEventKey* ev, Gtkmm2ext::Bindings*);
 	bool try_gtk_accel_binding (GtkWindow* win, GdkEventKey* ev, bool translate, GdkModifierType modifier);
@@ -911,6 +946,7 @@ private:
 	void action_script_changed (int i, const std::string&);
 
 	void ask_about_scratch_deletion ();
+	bool nsm_first_session_opened;
 };
 
 #endif /* __ardour_gui_h__ */
