@@ -80,7 +80,7 @@ ARDOUR_UI::reset_focus (Gtk::Widget* w)
 
 	Gtk::Widget* top = w->get_toplevel();
 
-	if (!top || !top->is_toplevel()) {
+	if (!top || !top->get_is_toplevel()) {
 		return;
 	}
 
@@ -88,7 +88,7 @@ ARDOUR_UI::reset_focus (Gtk::Widget* w)
 
 	while (w) {
 
-		if (w->is_toplevel()) {
+		if (w->get_is_toplevel()) {
 			/* Setting the focus widget to a Gtk::Window causes all
 			 * subsequent calls to ::has_focus() on the nominal
 			 * focus widget in that window to return
@@ -162,7 +162,7 @@ ARDOUR_UI::shared_popup_menu ()
 {
 	ENSURE_GUI_THREAD (*this, &ARDOUR_UI::shared_popup_menu, ignored);
 
-	assert (!_shared_popup_menu || !_shared_popup_menu->is_visible());
+	assert (!_shared_popup_menu || !_shared_popup_menu->get_visible());
 	delete _shared_popup_menu;
 	_shared_popup_menu = new Gtk::Menu;
 	return _shared_popup_menu;
@@ -176,19 +176,21 @@ ARDOUR_UI::set_flat_buttons ()
 
 
 void
-ARDOUR_UI::update_transport_clocks (samplepos_t pos)
+ARDOUR_UI::update_transport_clocks (samplepos_t p)
 {
+	timepos_t pos (p);
+
 	switch (UIConfiguration::instance().get_primary_clock_delta_mode()) {
 		case NoDelta:
 			primary_clock->set (pos);
 			break;
 		case DeltaEditPoint:
-			primary_clock->set (pos, false, editor->get_preferred_edit_position (EDIT_IGNORE_PHEAD));
+			primary_clock->set_duration (pos.distance (editor->get_preferred_edit_position (EDIT_IGNORE_PHEAD)), false);
 			break;
 		case DeltaOriginMarker:
 			{
 				Location* loc = _session->locations()->clock_origin_location ();
-				primary_clock->set (pos, false, loc ? loc->start() : 0);
+				primary_clock->set_duration (pos.distance (loc->start()));
 			}
 			break;
 	}
@@ -198,12 +200,12 @@ ARDOUR_UI::update_transport_clocks (samplepos_t pos)
 			secondary_clock->set (pos);
 			break;
 		case DeltaEditPoint:
-			secondary_clock->set (pos, false, editor->get_preferred_edit_position (EDIT_IGNORE_PHEAD));
+			secondary_clock->set_duration (pos.distance (editor->get_preferred_edit_position (EDIT_IGNORE_PHEAD)), false);
 			break;
 		case DeltaOriginMarker:
 			{
 				Location* loc = _session->locations()->clock_origin_location ();
-				secondary_clock->set (pos, false, loc ? loc->start() : 0);
+				secondary_clock->set_duration (pos.distance (loc->start()));
 			}
 			break;
 	}
@@ -213,7 +215,7 @@ ARDOUR_UI::update_transport_clocks (samplepos_t pos)
 	}
 
 	if (!editor->preview_video_drag_active ()) {
-		ARDOUR_UI::instance()->video_timeline->manual_seek_video_monitor(pos);
+		ARDOUR_UI::instance()->video_timeline->manual_seek_video_monitor(p);
 	}
 }
 
