@@ -45,6 +45,7 @@
 #include "pbd/fpu.h"
 #include "pbd/convert.h"
 #include "pbd/openuri.h"
+#include "pbd/types_convert.h"
 
 #include "gtkmm2ext/cairo_packer.h"
 #include "gtkmm2ext/utils.h"
@@ -61,11 +62,13 @@
 #include "engine_dialog.h"
 #include "editor.h"
 #include "editing.h"
+#include "enums_convert.h"
 #include "actions.h"
 #include "meterbridge.h"
 #include "luawindow.h"
 #include "mixer_ui.h"
 #include "recorder_ui.h"
+#include "trigger_page.h"
 #include "window_manager.h"
 #include "global_port_matrix.h"
 #include "location_ui.h"
@@ -108,20 +111,6 @@ ARDOUR_UI::create_editor ()
 	return 0;
 }
 
-int
-ARDOUR_UI::create_luawindow ()
-
-{
-	try {
-		luawindow = LuaWindow::instance ();
-	}
-	catch (failed_constructor& err) {
-		return -1;
-	}
-
-	return 0;
-}
-
 int
 ARDOUR_UI::create_recorder ()
 {
@@ -134,6 +123,18 @@ ARDOUR_UI::create_recorder ()
 	return 0;
 }
 
+int
+ARDOUR_UI::create_trigger_page ()
+{
+	try {
+		trigger_page = new TriggerPage ();
+		trigger_page->StateChange.connect (sigc::mem_fun (*this, &ARDOUR_UI::tabbable_state_change));
+	} catch (failed_constructor& err) {
+		return -1;
+	}
+	return 0;
+}
+
 void
 ARDOUR_UI::escape ()
 {
@@ -172,6 +173,7 @@ ARDOUR_UI::install_actions ()
 	ActionManager::register_action (main_menu_actions, X_("EditorMenu"), _("Editor"));
 	ActionManager::register_action (main_menu_actions, X_("PrefsMenu"), _("Preferences"));
 	ActionManager::register_action (main_menu_actions, X_("RecorderMenu"), _("Recorder"));
+	ActionManager::register_action (main_menu_actions, X_("TriggerMenu"), _("Cue Grid"));
 	ActionManager::register_action (main_menu_actions, X_("DetachMenu"), _("Detach"));
 	ActionManager::register_action (main_menu_actions, X_("Help"), _("Help"));
 	ActionManager::register_action (main_menu_actions, X_("KeyMouseActions"), _("Misc. Shortcuts"));
@@ -278,7 +280,6 @@ ARDOUR_UI::install_actions ()
 		ActionManager::register_action (common_actions, X_("show-ui-prefs"), _("Show more UI preferences"), sigc::mem_fun (*this, &ARDOUR_UI::show_ui_prefs));
 	}
 
-	ActionManager::register_action (common_actions, X_("toggle-luawindow"), S_("Window|Scripting"),  sigc::mem_fun(*this, &ARDOUR_UI::toggle_luawindow));
 	ActionManager::register_action (common_actions, X_("toggle-meterbridge"), S_("Window|Meterbridge"),  sigc::mem_fun(*this, &ARDOUR_UI::toggle_meterbridge));
 
 	act = ActionManager::register_action (common_actions, X_("NewMIDITracer"), _("MIDI Tracer"), sigc::mem_fun(*this, &ARDOUR_UI::new_midi_tracer_window));
@@ -324,7 +325,6 @@ ARDOUR_UI::install_actions ()
 
 	act = ActionManager::register_action (transport_actions, X_("ToggleRoll"), _("Start/Stop"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::toggle_roll), false, false));
 	ActionManager::session_sensitive_actions.push_back (act);
-	ActionManager::transport_sensitive_actions.push_back (act);
 	act = ActionManager::register_action (transport_actions, X_("alternate-ToggleRoll"), _("Start/Stop"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::toggle_roll), false, false));
 	ActionManager::session_sensitive_actions.push_back (act);
 	ActionManager::transport_sensitive_actions.push_back (act);
@@ -584,6 +584,11 @@ ARDOUR_UI::install_dependent_actions ()
 		ActionManager::session_sensitive_actions.push_back (act);
 	}
 
+	act = ActionManager::register_action (common_actions, "jump-to-loop-start", _("Jump to Loop Start"), sigc::bind(sigc::mem_fun(*editor, &PublicEditor::jump_to_loop_marker), true));
+	ActionManager::session_sensitive_actions.push_back (act);
+	act = ActionManager::register_action (common_actions, "jump-to-loop-end", _("Jump to Loop End"), sigc::bind(sigc::mem_fun(*editor, &PublicEditor::jump_to_loop_marker), false));
+	ActionManager::session_sensitive_actions.push_back (act);
+
 	act = ActionManager::register_action (common_actions, X_("addExistingAudioFiles"), _("Import"), sigc::mem_fun (*editor, &PublicEditor::external_audio_dialog));
 	ActionManager::session_sensitive_actions.push_back (act);
 	ActionManager::write_sensitive_actions.push_back (act);
@@ -688,21 +693,25 @@ ARDOUR_UI::install_dependent_actions ()
 	ActionManager::register_action (common_actions, X_("hide-mixer"), _("Hide"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::hide_tabbable), mixer));
 	ActionManager::register_action (common_actions, X_("hide-preferences"), _("Hide"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::hide_tabbable), rc_option_editor));
 	ActionManager::register_action (common_actions, X_("hide-recorder"), _("Hide"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::hide_tabbable), recorder));
+	ActionManager::register_action (common_actions, X_("hide-trigger"), _("Hide"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::hide_tabbable), trigger_page));
 
 	ActionManager::register_action (common_actions, X_("attach-editor"), _("Attach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::attach_tabbable), editor));
 	ActionManager::register_action (common_actions, X_("attach-mixer"), _("Attach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::attach_tabbable), mixer));
 	ActionManager::register_action (common_actions, X_("attach-preferences"), _("Attach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::attach_tabbable), rc_option_editor));
 	ActionManager::register_action (common_actions, X_("attach-recorder"), _("Attach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::attach_tabbable), recorder));
+	ActionManager::register_action (common_actions, X_("attach-trigger"), _("Attach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::attach_tabbable), trigger_page));
 
 	ActionManager::register_action (common_actions, X_("detach-editor"), _("Detach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::detach_tabbable), editor));
 	ActionManager::register_action (common_actions, X_("detach-mixer"), _("Detach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::detach_tabbable), mixer));
 	ActionManager::register_action (common_actions, X_("detach-preferences"), _("Detach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::detach_tabbable), rc_option_editor));
 	ActionManager::register_action (common_actions, X_("detach-recorder"), _("Detach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::detach_tabbable), recorder));
+	ActionManager::register_action (common_actions, X_("detach-trigger"), _("Detach"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::detach_tabbable), trigger_page));
 
 	ActionManager::register_action (common_actions, X_("show-editor"), _("Show Editor"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::show_tabbable), editor));
 	ActionManager::register_action (common_actions, X_("show-mixer"), _("Show Mixer"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::show_tabbable), mixer));
 	ActionManager::register_action (common_actions, X_("show-preferences"), _("Show"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::show_tabbable), rc_option_editor));
 	ActionManager::register_action (common_actions, X_("show-recorder"), _("Show Recorder"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::show_tabbable), recorder));
+	ActionManager::register_action (common_actions, X_("show-trigger"), _("Show Cues"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::show_tabbable), trigger_page));
 
 	/* These "change" actions are not intended to be used inside menus, but
 	   are for the tab/window control buttons, which have somewhat odd
@@ -712,6 +721,7 @@ ARDOUR_UI::install_dependent_actions ()
 	ActionManager::register_action (common_actions, X_("change-mixer-visibility"), _("Change"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::button_change_tabbable_visibility), mixer));
 	ActionManager::register_action (common_actions, X_("change-preferences-visibility"), _("Change"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::button_change_tabbable_visibility), rc_option_editor));
 	ActionManager::register_action (common_actions, X_("change-recorder-visibility"), _("Change"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::button_change_tabbable_visibility), recorder));
+	ActionManager::register_action (common_actions, X_("change-trigger-visibility"), _("Change"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::button_change_tabbable_visibility), trigger_page));
 
 	/* These "change" actions are not intended to be used inside menus, but
 	   are for the tab/window control key bindings, which have somewhat odd
@@ -721,6 +731,7 @@ ARDOUR_UI::install_dependent_actions ()
 	ActionManager::register_action (common_actions, X_("key-change-mixer-visibility"), _("Change"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::key_change_tabbable_visibility), mixer));
 	ActionManager::register_action (common_actions, X_("key-change-preferences-visibility"), _("Change"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::key_change_tabbable_visibility), rc_option_editor));
 	ActionManager::register_action (common_actions, X_("key-change-recorder-visibility"), _("Change"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::key_change_tabbable_visibility), recorder));
+	ActionManager::register_action (common_actions, X_("key-change-trigger-visibility"), _("Change"), sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::key_change_tabbable_visibility), trigger_page));
 
 	ActionManager::register_action (common_actions, X_("previous-tab"), _("Previous Tab"), sigc::mem_fun (*this, &ARDOUR_UI::step_up_through_tabs));
 	ActionManager::register_action (common_actions, X_("next-tab"), _("Next Tab"), sigc::mem_fun (*this, &ARDOUR_UI::step_down_through_tabs));
@@ -731,7 +742,6 @@ ARDOUR_UI::install_dependent_actions ()
 
 	ActionManager::register_toggle_action (common_actions, X_("ToggleMaximalEditor"), _("Maximise Editor Space"), sigc::mem_fun (*this, &ARDOUR_UI::toggle_editing_space));
 	ActionManager::register_toggle_action (common_actions, X_("ToggleMaximalMixer"), _("Maximise Mixer Space"), sigc::mem_fun (*this, &ARDOUR_UI::toggle_mixer_space));
-	ActionManager::session_sensitive_actions.push_back (act);
 }
 
 void
@@ -910,6 +920,8 @@ ARDOUR_UI::save_ardour_state ()
 		current_tab = "mixer";
 	} else if (current_page_number == _tabs.page_num (recorder->contents())) {
 		current_tab = "recorder";
+	} else if (current_page_number == _tabs.page_num (trigger_page->contents())) {
+		current_tab = "trigger";
 	} else if (current_page_number == _tabs.page_num (rc_option_editor->contents())) {
 		current_tab = "preferences";
 	}
@@ -933,6 +945,17 @@ ARDOUR_UI::save_ardour_state ()
 	XMLNode& bnode (meterbridge->get_state());
 	XMLNode& pnode (rc_option_editor->get_state());
 	XMLNode& rnode (recorder->get_state());
+	XMLNode& tnode (trigger_page->get_state());
+
+	/* store clock modes */
+	XMLNode* cnode = new XMLNode(X_("ClockModes"));
+	for (auto const& i: AudioClock::clocks) {
+		XMLNode* child = new XMLNode (X_("Clock"));
+		child->set_property (X_("name"), i->name());
+		child->set_property (X_("mode"), i->mode());
+		child->set_property (X_("on"), i->on());
+		cnode->add_child_nocopy (*child);
+	}
 
 	Config->add_extra_xml (*window_node);
 	Config->add_extra_xml (audio_midi_setup->get_state());
@@ -950,6 +973,8 @@ ARDOUR_UI::save_ardour_state ()
 		_session->add_instant_xml (pnode);
 		_session->add_instant_xml (bnode);
 		_session->add_instant_xml (rnode);
+		_session->add_instant_xml (tnode);
+		_session->add_instant_xml (*cnode);
 		if (location_ui) {
 			_session->add_instant_xml (location_ui->ui().get_state ());
 		}
@@ -958,20 +983,25 @@ ARDOUR_UI::save_ardour_state ()
 			vkstate.add_child_nocopy (virtual_keyboard_window.get_state ());
 			_session->add_instant_xml (vkstate);
 		}
-	} else {
+	}
+
+	/* save current Window settings and sizes for new sessions */
+	{
 		Config->add_instant_xml (main_window_node);
 		Config->add_instant_xml (enode);
 		Config->add_instant_xml (mnode);
 		Config->add_instant_xml (pnode);
 		Config->add_instant_xml (bnode);
 		Config->add_instant_xml (rnode);
+		Config->add_instant_xml (tnode);
+		Config->add_instant_xml (*cnode);
 		if (location_ui) {
 			Config->add_instant_xml (location_ui->ui().get_state ());
 		}
 		if (virtual_keyboard_window) {
 			XMLNode& vkstate (virtual_keyboard_window->get_state());
 			vkstate.add_child_nocopy (virtual_keyboard_window.get_state ());
-			_session->add_instant_xml (vkstate);
+			Config->add_instant_xml (vkstate);
 		}
 	}
 
@@ -980,6 +1010,8 @@ ARDOUR_UI::save_ardour_state ()
 	delete &bnode;
 	delete &pnode;
 	delete &rnode;
+	delete &tnode;
+	delete cnode;
 
 	Keyboard::save_keybindings ();
 }
@@ -987,10 +1019,10 @@ ARDOUR_UI::save_ardour_state ()
 void
 ARDOUR_UI::on_theme_changed ()
 {
+	ARDOUR::set_global_ui_scale_factor (UIConfiguration::instance().get_ui_scale());
 	LV2Plugin::set_global_ui_background_color (UIConfiguration::instance().color ("gtk_background"));
 	LV2Plugin::set_global_ui_foreground_color (UIConfiguration::instance().color ("gtk_foreground"));
 	LV2Plugin::set_global_ui_contrasting_color (UIConfiguration::instance().color ("theme:contrasting clock")); // more contrast that "theme:contrasting"
-	LV2Plugin::set_global_ui_scale_factor (UIConfiguration::instance().get_ui_scale());
 	LV2Plugin::set_global_ui_style_boxy (UIConfiguration::instance().get_boxy_buttons());
 	LV2Plugin::set_global_ui_style_flat (UIConfiguration::instance().get_flat_buttons());
 }
