@@ -57,6 +57,7 @@
 #include "ardour/filename_extensions.h"
 #include "ardour/filesystem_paths.h"
 #include "ardour/profile.h"
+#include "ardour/recent_sessions.h"
 
 #include "gtkmm2ext/application.h"
 
@@ -188,10 +189,14 @@ This session was created with a sample rate of %1 Hz, but\n\
 %2 is currently running at %3 Hz.  If you load this session,\n\
 audio may be played at the wrong sample rate.\n"), desired, PROGRAM_NAME, actual));
 
-	image->set_alignment(ALIGN_CENTER, ALIGN_TOP);
+	image->set_alignment(ALIGN_CENTER, ALIGN_START);
 	hbox->pack_start (*image, PACK_EXPAND_WIDGET, 12);
 	hbox->pack_end (message, PACK_EXPAND_PADDING, 12);
 	dialog.get_vbox()->pack_start(*hbox, PACK_EXPAND_PADDING, 6);
+
+	if (ARDOUR::AudioEngine::instance ()->setup_required ()) {
+		dialog.add_button (_("Reconfigure Engine"), RESPONSE_YES);
+	}
 	dialog.add_button (_("Do not load session"), RESPONSE_REJECT);
 	dialog.add_button (_("Load session anyway"), RESPONSE_ACCEPT);
 	dialog.set_default_response (RESPONSE_ACCEPT);
@@ -201,6 +206,11 @@ audio may be played at the wrong sample rate.\n"), desired, PROGRAM_NAME, actual
 	hbox->show();
 
 	switch (dialog.run()) {
+	case RESPONSE_YES:
+		ARDOUR::AudioEngine::instance ()->stop ();
+		(dynamic_cast<EngineControl*> (audio_midi_setup.get (true)))->run ();
+		(dynamic_cast<EngineControl*> (audio_midi_setup.get (true)))->hide ();
+		return AudioEngine::instance()->running () ? -1 : 1;
 	case RESPONSE_ACCEPT:
 		return 0;
 	default:
@@ -328,6 +338,41 @@ ARDOUR_UI::recorder_settings () const
 	return node;
 }
 
+XMLNode*
+ARDOUR_UI::trigger_page_settings () const
+{
+	XMLNode* node = 0;
+
+	if (_session) {
+		node = _session->instant_xml(X_("TriggerPage"));
+	} else {
+		node = Config->instant_xml(X_("TriggerPage"));
+	}
+
+	if (!node) {
+		node = new XMLNode (X_("TriggerPage"));
+	}
+
+	return node;
+}
+
+XMLNode*
+ARDOUR_UI::clock_mode_settings () const
+{
+	XMLNode* node = 0;
+
+	if (_session) {
+		node = _session->instant_xml(X_("ClockModes"));
+	}
+	if (!node) {
+		node = Config->instant_xml(X_("ClockModes"));
+	}
+	if (!node) {
+		node = new XMLNode (X_("ClockModes"));
+	}
+	return node;
+}
+
 XMLNode*
 ARDOUR_UI::keyboard_settings () const
 {
@@ -410,7 +455,7 @@ ARDOUR_UI::nsm_init ()
 	 * The wrapper startup script should set the environment variable 'ARDOUR_SELF'
 	 */
 	const char *process_name = g_getenv ("ARDOUR_SELF");
-	nsm->announce (PROGRAM_NAME, ":dirty:", process_name ? process_name : "ardour6");
+	nsm->announce (PROGRAM_NAME, ":dirty:switch:", process_name ? process_name : "ardour7");
 
 	unsigned int i = 0;
 	// wait for announce reply from nsm server
@@ -563,6 +608,53 @@ ARDOUR_UI::starting ()
 	return 0;
 }
 
+int
+ARDOUR_UI::copy_demo_sessions ()
+{
+	int copied = 0;
+	if (ARDOUR::Profile->get_mixbus () && Config->get_copy_demo_sessions ()) {
+		std::string dspd (Config->get_default_session_parent_dir());
+		Searchpath ds (ARDOUR::ardour_data_search_path());
+		ds.add_subdirectory_to_paths ("sessions");
+		vector<string> demos;
+		find_files_matching_pattern (demos, ds, string_compose ("*%1", ARDOUR::session_archive_suffix));
+
+		ARDOUR::RecentSessions rs;
+		ARDOUR::read_recent_sessions (rs);
+
+		for (vector<string>::iterator i = demos.begin(); i != demos.end (); ++i) {
+			/* "demo-session" must be inside "demo-session.<session_archive_suffix>" */
+			std::string name = basename_nosuffix (basename_nosuffix (*i));
+			std::string path = Glib::build_filename (dspd, name);
+			/* skip if session-dir already exists */
+			if (Glib::file_test(path.c_str(), Glib::FILE_TEST_IS_DIR)) {
+				/* ..but add it to recent-list */
+				store_recent_sessions (name, path);
+				continue;
+			}
+			/* skip sessions that are already in 'recent'.
+			 * eg. a new user changed <session-default-dir> shortly after installation
+			 */
+			for (ARDOUR::RecentSessions::iterator r = rs.begin(); r != rs.end(); ++r) {
+				if ((*r).first == name) {
+					continue;
+				}
+			}
+			try {
+				PBD::FileArchive ar (*i);
+				if (0 == ar.inflate (dspd)) {
+					store_recent_sessions (name, path);
+					info << string_compose (_("Copied demo session `%1'."), name) << endmsg;
+					++copied;
+				}
+			} catch (...) {
+					info << string_compose (_("Failed to extract demo session `%1'."), name) << endmsg;
+			}
+		}
+	}
+	return copied;
+}
+
 int
 ARDOUR_UI::load_session_from_startup_fsm ()
 {
@@ -733,7 +825,9 @@ ARDOUR_UI::load_from_application_api (const std::string& path)
 	}
 
 	if (nsm) {
-		if (!AudioEngine::instance()->set_backend("JACK", "", "")) {
+		unload_session(false, true);
+
+		if (!AudioEngine::instance()->set_backend("JACK", ARDOUR_COMMAND_LINE::backend_client_name, "")) {
 			error << _("NSM: The JACK backend is mandatory and can not be loaded.") << endmsg;
 			return;
 		}
@@ -757,10 +851,13 @@ ARDOUR_UI::load_from_application_api (const std::string& path)
 			}
 		}
 
-		PluginScanDialog psd (true, false);
-		psd.start ();
+		if (!nsm_first_session_opened) {
+			PluginScanDialog psd (true, false);
+			psd.start ();
 
-		post_engine ();
+			post_engine ();
+			nsm_first_session_opened = true;
+		}
 	}
 
 	/* the mechanisms that can result is this being called are only
