@@ -157,7 +157,7 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	peak_display.set_events (peak_display.get_events() & ~(Gdk::EventMask (Gdk::LEAVE_NOTIFY_MASK|Gdk::ENTER_NOTIFY_MASK|Gdk::POINTER_MOTION_MASK)));
 	peak_display.signal_map().connect (sigc::bind (sigc::ptr_fun (reset_cursor_to_default), &peak_display));
 	peak_display.signal_state_changed().connect (sigc::bind (sigc::ptr_fun (reset_cursor_to_default_state), &peak_display));
-	peak_display.unset_flags (Gtk::CAN_FOCUS);
+	peak_display.set_can_focus (false);
 	peak_display.set_editable (false);
 
 	gain_automation_state_button.set_name ("mixer strip button");
@@ -165,7 +165,7 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 	set_tooltip (gain_automation_state_button, _("Fader automation mode"));
 	set_tooltip (peak_display, _("dBFS - Digital Peak Hold. Click to reset."));
 
-	gain_automation_state_button.unset_flags (Gtk::CAN_FOCUS);
+	gain_automation_state_button.set_can_focus (false);
 
 	gain_automation_state_button.set_size_request(15, 15);
 
@@ -176,7 +176,7 @@ GainMeterBase::GainMeterBase (Session* s, bool horizontal, int fader_length, int
 
 	set_tooltip (&meter_point_button, _("Metering point"));
 
-	meter_point_button.unset_flags (Gtk::CAN_FOCUS);
+	meter_point_button.set_can_focus (false);
 
 	meter_point_button.set_size_request(15, 15);
 
@@ -583,11 +583,12 @@ GainMeterBase::fader_moved ()
 			value = gain_adjustment.get_value();
 		}
 
-		// XXX hack allow to override group
-		// (this breaks group'ed  shift+click reset)
-		if (Keyboard::the_keyboard().key_is_down (GDK_Shift_R)
-				|| Keyboard::the_keyboard().key_is_down (GDK_Shift_L)) {
-			_control->set_value (value, Controllable::InverseGroup);
+		if (Keyboard::the_keyboard().modifier_state() == Keyboard::group_override_modifier ()) {
+			if (Config->get_group_override_inverts ()) {
+				_control->set_value (value, Controllable::InverseGroup);
+			} else {
+				_control->set_value (value, Controllable::NoGroup);
+			}
 		} else {
 			_control->set_value (value, Controllable::UseGroup);
 		}
@@ -644,6 +645,30 @@ GainMeterBase::set_fader_name (const char * name)
 	gain_slider->set_name (name);
 }
 
+void
+GainMeterBase::set_fader_fg (uint32_t c)
+{
+	gain_slider->set_fg (c);
+}
+
+void
+GainMeterBase::set_fader_bg (uint32_t c)
+{
+	gain_slider->set_bg (c);
+}
+
+void
+GainMeterBase::unset_fader_fg ()
+{
+	gain_slider->unset_fg ();
+}
+
+void
+GainMeterBase::unset_fader_bg ()
+{
+	gain_slider->unset_bg ();
+}
+
 void
 GainMeterBase::update_gain_sensitive ()
 {
@@ -729,13 +754,13 @@ GainMeterBase::meter_point_clicked (MeterPoint mp)
 void
 GainMeterBase::amp_start_touch ()
 {
-	_control->start_touch (_control->session().transport_sample());
+	_control->start_touch (timepos_t (_control->session().transport_sample()));
 }
 
 void
 GainMeterBase::amp_stop_touch ()
 {
-	_control->stop_touch (_control->session().transport_sample());
+	_control->stop_touch (timepos_t (_control->session().transport_sample()));
 	effective_gain_display ();
 }
 
@@ -924,7 +949,7 @@ GainMeter::GainMeter (Session* s, int fader_length)
 
 	set_tooltip (gain_automation_state_button, _("Fader automation mode"));
 
-	gain_automation_state_button.unset_flags (Gtk::CAN_FOCUS);
+	gain_automation_state_button.set_can_focus (false);
 
 	gain_automation_state_button.set_size_request (PX_SCALE(12, 15), PX_SCALE(12, 15));
 
@@ -996,10 +1021,11 @@ GainMeter::set_controls (boost::shared_ptr<Route> r,
 
 	if (_route) {
 		_route->active_changed.connect (model_connections, invalidator (*this), boost::bind (&GainMeter::route_active_changed, this), gui_context ());
-		hbox.pack_start (meter_hbox, true, true);
-		meter_hbox.show ();
 	}
 
+	hbox.pack_start (meter_hbox, true, true);
+	meter_hbox.show ();
+
 //	if (r && !r->is_auditioner()) {
 //		fader_vbox->pack_start (gain_automation_state_button, false, false, 0);
 //	}
@@ -1019,12 +1045,12 @@ GainMeter::get_gm_width ()
 	Gtk::Requisition sz;
 	int min_w = 0;
 	sz.width = 0;
-	meter_metric_area.size_request (sz);
+	sz = meter_metric_area.size_request ();
 	min_w += sz.width;
-	level_meter->size_request (sz);
+	sz = level_meter->size_request ();
 	min_w += sz.width;
 
-	fader_alignment.size_request (sz);
+	sz = fader_alignment.size_request ();
 	if (_width == Wide)
 		return max(sz.width * 2, min_w * 2) + 6;
 	else
