@@ -103,7 +103,7 @@ public:
 	void  set_strip_width (Width, bool save = false);
 	Width get_strip_width () const { return _strip_width; }
 
-	XMLNode& get_state ();
+	XMLNode& get_state () const;
 	int set_state (const XMLNode&, int /* version */);
 
 	void save_plugin_order_file ();
@@ -123,6 +123,11 @@ public:
 	void select_next_strip ();
 	void select_prev_strip ();
 
+	void store_mixer_scene (size_t);
+	void recall_mixer_scene (size_t, bool interactive = true);
+	void clear_mixer_scene (size_t, bool interactive = true);
+	void rename_mixer_scene (size_t n);
+
 	void do_vca_assign (boost::shared_ptr<ARDOUR::VCA>);
 	void do_vca_unassign (boost::shared_ptr<ARDOUR::VCA>);
 	void show_spill (boost::shared_ptr<ARDOUR::Stripable>);
@@ -203,6 +208,20 @@ private:
 	Gtk::HBox             out_packer;
 	ArdourWidgets::HPane  list_hpane;
 
+	Gtk::EventBox         _mixer_scene_spacer;
+	Gtk::Frame            _mixer_scene_frame;
+	Gtk::Table            _mixer_scene_table;
+	Gtk::VBox             _mixer_scene_vbox;
+
+	std::vector<ArdourWidgets::ArdourButton*> _mixer_scene_buttons;
+	std::vector<Gtk::Label*>                  _mixer_scene_labels;
+	ARDOUR::MixerScene*                       _mixer_scene_release;
+
+	void popup_scene_menu (GdkEventButton* ev, int);
+	bool scene_button_press (GdkEventButton*, int);
+	bool scene_button_release (GdkEventButton*, int);
+	bool scene_label_press (GdkEventButton* ev, int);
+
 	MixerGroupTabs* _group_tabs;
 
 	bool on_scroll_event (GdkEventScroll*);
@@ -224,6 +243,8 @@ private:
 
 	void add_stripables (ARDOUR::StripableList&);
 
+	void update_scene_buttons ();
+
 	void add_routes (ARDOUR::RouteList&);
 	void remove_strip (MixerStrip *);
 	void remove_foldback (FoldbackStrip *);
