@@ -26,6 +26,8 @@
 
 #include "midi++/mmc.h"
 
+#include "pbd/natsort.h"
+
 #include "ardour/audio_port.h"
 #include "ardour/async_midi_port.h"
 #include "ardour/audioengine.h"
@@ -457,7 +459,9 @@ PortGroupList::gather (ARDOUR::Session* session, ARDOUR::DataType type, bool inp
 		if (!inputs) {
 
 			program->add_bundle (session->the_auditioner()->output()->bundle());
-			program->add_bundle (session->click_io()->bundle());
+			if (session->click_io()) {
+				program->add_bundle (session->click_io()->bundle());
+			}
 
 			boost::shared_ptr<Bundle> ltc (new Bundle (_("LTC Out"), inputs));
 			ltc->add_channel (_("LTC Out"), DataType::AUDIO, session->engine().make_port_name_non_relative (session->ltc_output_port()->name()));
@@ -513,8 +517,8 @@ PortGroupList::gather (ARDOUR::Session* session, ARDOUR::DataType type, bool inp
 	if (!inputs && (type == DataType::MIDI || type == DataType::NIL)) {
 		boost::shared_ptr<ARDOUR::Port> ap = boost::dynamic_pointer_cast<ARDOUR::Port> (session->vkbd_output_port());
 		AudioEngine* ae = AudioEngine::instance();
-		boost::shared_ptr<Bundle> vm (new Bundle (_("Virtual MIDI"), inputs));
-		vm->add_channel (_("Virtual Keyboard"), DataType::MIDI, ae->make_port_name_non_relative (ap->name()));
+		boost::shared_ptr<Bundle> vm (new Bundle (ap->pretty_name (), inputs));
+		vm->add_channel (ap->pretty_name (), DataType::MIDI, ae->make_port_name_non_relative (ap->name()));
 		program->add_bundle (vm);
 	}
 
@@ -583,16 +587,27 @@ PortGroupList::gather (ARDOUR::Session* session, ARDOUR::DataType type, bool inp
 
 	if (ports.size () > 0) {
 
-		for (vector<string>::const_iterator s = ports.begin(); s != ports.end(); ) {
+		struct SortByPortName {
+			bool operator() (std::string const& lhs, std::string const& rhs) const {
+				return PBD::naturally_less (lhs.c_str (), rhs.c_str ());
+			}
+		} port_sorter;
+
+		std::sort (ports.begin (), ports.end (), port_sorter);
+
+		for (vector<string>::const_iterator s = ports.begin(); s != ports.end(); ++s) {
 
 			std::string const p = *s;
 
-			if (!system->has_port(p) &&
-			    !bus->has_port(p) &&
-			    !track->has_port(p) &&
-			    !sidechain->has_port(p) &&
-			    !program->has_port(p) &&
-			    !other->has_port(p)) {
+			if (allow_dups || (
+			        !system->has_port(p)
+			     && !bus->has_port(p)
+			     && !track->has_port(p)
+			     && !sidechain->has_port(p)
+			     && !program->has_port(p)
+			     && !other->has_port(p)
+			    )
+			   ) {
 
 				/* special hack: ignore MIDI ports labelled Midi-Through. these
 				   are basically useless and mess things up for default
@@ -600,7 +615,6 @@ PortGroupList::gather (ARDOUR::Session* session, ARDOUR::DataType type, bool inp
 				*/
 
 				if (p.find ("Midi-Through") != string::npos || p.find ("Midi Through") != string::npos) {
-					++s;
 					continue;
 				}
 
@@ -616,7 +630,6 @@ PortGroupList::gather (ARDOUR::Session* session, ARDOUR::DataType type, bool inp
 
 				if ((lp.find (monitor) != string::npos) &&
 				    (lp.find (lpn) != string::npos)) {
-					++s;
 					continue;
 				}
 
@@ -638,13 +651,14 @@ PortGroupList::gather (ARDOUR::Session* session, ARDOUR::DataType type, bool inp
 
 					PortFlags flags (AudioEngine::instance()->port_engine().get_port_flags (ph));
 
-					if (port_has_prefix (p, lpnc)) {
+					if (flags & Hidden ) {
+						continue;
+					} else if (port_has_prefix (p, lpnc)) {
 
 						/* we own this port (named after the program) */
 
 						/* Hide scene ports for now */
 						if (p.find (_("Scene ")) != string::npos) {
-							++s;
 							continue;
 						}
 
@@ -659,15 +673,13 @@ PortGroupList::gather (ARDOUR::Session* session, ARDOUR::DataType type, bool inp
 					}
 				}
 			}
-
-			++s;
 		}
 	}
 
 	for (DataType::iterator i = DataType::begin(); i != DataType::end(); ++i) {
 		if (!extra_system[*i].empty()) {
 			boost::shared_ptr<Bundle> b = make_bundle_from_ports (extra_system[*i], *i, inputs);
-			system->add_bundle (b);
+			system->add_bundle (b, allow_dups);
 		}
 	}
 
