@@ -57,6 +57,9 @@
 #include "widgets/ardour_fader.h"
 #include "widgets/slider_controller.h"
 
+#ifdef HAVE_BEATBOX
+#include "beatbox_gui.h"
+#endif
 #include "plugin_interest.h"
 #include "plugin_display.h"
 #include "io_selector.h"
@@ -100,7 +103,7 @@ public:
 	void set_custom_ui_mode(bool use_custom) { want_custom = use_custom; }
 
 	int set_state (const XMLNode&, int);
-	XMLNode& get_state ();
+	XMLNode& get_state () const;
 
 private:
 	ProcessorBox* _processor_box;
@@ -109,6 +112,7 @@ private:
 	bool want_custom;
 
 	void processor_going_away ();
+	sigc::connection _unmap_connection;
 	PBD::ScopedConnection going_away_connection;
 	PBD::ScopedConnectionList gui_connections;
 };
@@ -149,13 +153,13 @@ public:
 	bool drag_data_get (Glib::RefPtr<Gdk::DragContext> const, Gtk::SelectionData &);
 	bool can_copy_state (Gtkmm2ext::DnDVBoxChild*) const;
 
-	enum Position {
+	enum ProcessorPosition {
 		PreFader,
 		Fader,
 		PostFader
 	};
 
-	void set_position (Position, uint32_t);
+	void set_position (ProcessorPosition, uint32_t);
 	bool unknown_processor () const { return _unknown_processor; } ;
 	boost::shared_ptr<ARDOUR::Processor> processor () const;
 	void set_enum_width (Width);
@@ -171,11 +175,12 @@ public:
 	std::string state_id () const;
 	Gtk::Menu* build_controls_menu ();
 	Gtk::Menu* build_send_options_menu ();
+	Gtk::Menu* build_presets_menu ();
 
 protected:
 	ArdourWidgets::ArdourButton _button;
 	Gtk::VBox _vbox;
-	Position _position;
+	ProcessorPosition _position;
 	uint32_t _position_num;
 	ProcessorBox* _parent;
 
@@ -184,6 +189,7 @@ protected:
 private:
 	bool _selectable;
 	bool _unknown_processor;
+	bool _ignore_preset_select;
 	void led_clicked(GdkEventButton *);
 	void processor_active_changed ();
 	void processor_property_changed (const PBD::PropertyChange&);
@@ -253,6 +259,9 @@ private:
 
 	void toggle_panner_link ();
 	void toggle_allow_feedback ();
+	void plugin_preset_selected (ARDOUR::Plugin::PresetRecord);
+	void plugin_preset_add ();
+	void plugin_preset_delete ();
 
 	class PluginInlineDisplay : public PluginDisplay {
 	public:
@@ -435,7 +444,8 @@ public:
 
 	void hide_things ();
 
-	bool edit_aux_send(boost::shared_ptr<ARDOUR::Processor>);
+	bool edit_aux_send (boost::shared_ptr<ARDOUR::Processor>);
+	bool edit_triggerbox (boost::shared_ptr<ARDOUR::Processor>);
 
 	/* Everything except a WindowProxy object should use this to get the window */
 	Gtk::Window* get_processor_ui (boost::shared_ptr<ARDOUR::Processor>) const;
@@ -568,6 +578,7 @@ private:
 
 	bool can_cut() const;
 	bool stub_processor_selected() const;
+	bool channelstrip_selected() const;
 
 	static Glib::RefPtr<Gtk::Action> cut_action;
 	static Glib::RefPtr<Gtk::Action> copy_action;
