@@ -72,6 +72,7 @@ namespace ARDOUR {
 	class Trimmable;
 	class Movable;
 	class Stripable;
+	class MidiOperator;
 }
 
 namespace Gtk {
@@ -123,13 +124,18 @@ public:
 	virtual ~PublicEditor ();
 
 	/** @return Singleton PublicEditor instance */
-	static PublicEditor& instance () { return *_instance; }
+	static PublicEditor& instance () { assert (_instance); return *_instance; }
 
 	virtual bool have_idled() const = 0;
 	virtual void first_idle() = 0;
 
 	virtual void setup_tooltips() = 0;
 
+	/* returns the time domain to be used when there's no other overriding
+	 * reason to choose one.
+	 */
+	virtual Temporal::TimeDomain default_time_domain() const = 0;
+
 	/** Attach this editor to a Session.
 	 * @param s Session to connect to.
 	 */
@@ -152,8 +158,8 @@ public:
 	 * Snap a value according to the current snap setting.
 	 * ensure_snap overrides SnapOff and magnetic snap
 	 */
-	virtual void snap_to (ARDOUR::MusicSample& first,
-	                      ARDOUR::RoundMode   direction = ARDOUR::RoundNearest,
+	virtual void snap_to (Temporal::timepos_t & first,
+	                      Temporal::RoundMode   direction = Temporal::RoundNearest,
 	                      ARDOUR::SnapPref    gpref = ARDOUR::SnapToAny_Visual,
 	                      bool                ensure_snap = false) = 0;
 
@@ -205,14 +211,26 @@ public:
 	virtual void new_region_from_selection () = 0;
 	virtual void separate_region_from_selection () = 0;
 
+	virtual void reverse_region () = 0;
+	virtual void normalize_region () = 0;
+	virtual void quantize_region () = 0;
+	virtual void legatize_region (bool shrink_only) = 0;
+	virtual void transform_region () = 0;
+	virtual void transpose_region () = 0;
+	virtual void pitch_shift_region () = 0;
+
 	virtual void transition_to_rolling (bool fwd) = 0;
 	virtual samplepos_t pixel_to_sample (double pixel) const = 0;
 	virtual samplepos_t playhead_cursor_sample () const = 0;
 	virtual double sample_to_pixel (samplepos_t sample) const = 0;
 	virtual double sample_to_pixel_unrounded (samplepos_t sample) const = 0;
+	virtual double time_to_pixel (Temporal::timepos_t const &) const = 0;
+	virtual double time_to_pixel_unrounded (Temporal::timepos_t const &) const = 0;
+	virtual double duration_to_pixels (Temporal::timecnt_t const &) const = 0;
+	virtual double duration_to_pixels_unrounded (Temporal::timecnt_t const &) const = 0;
 
 	virtual Selection& get_selection () const = 0;
-	virtual bool get_selection_extents (samplepos_t &start, samplepos_t &end) const = 0;
+	virtual bool get_selection_extents (Temporal::timepos_t &start, Temporal::timepos_t &end) const = 0;
 	virtual Selection& get_cut_buffer () const = 0;
 
 	virtual void set_selection (std::list<Selectable*>, Selection::Operation) = 0;
@@ -226,14 +244,15 @@ public:
 	virtual void rec_with_count_in () = 0;
 	virtual void maybe_locate_with_edit_preroll (samplepos_t location) = 0;
 	virtual void goto_nth_marker (int nth) = 0;
+	virtual void jump_to_loop_marker (bool start) = 0;
 	virtual void trigger_script (int nth) = 0;
 	virtual void add_location_from_playhead_cursor () = 0;
 	virtual void remove_location_at_playhead_cursor () = 0;
-	virtual void add_location_mark (samplepos_t where) = 0;
+	virtual void add_location_mark (Temporal::timepos_t const & where) = 0;
 	virtual void update_grid () = 0;
 	virtual void remove_tracks () = 0;
-	virtual void set_loop_range (samplepos_t start, samplepos_t end, std::string cmd) = 0;
-	virtual void set_punch_range (samplepos_t start, samplepos_t end, std::string cmd) = 0;
+	virtual void set_loop_range (Temporal::timepos_t const & start, Temporal::timepos_t const & end, std::string cmd) = 0;
+	virtual void set_punch_range (Temporal::timepos_t const & start, Temporal::timepos_t const & end, std::string cmd) = 0;
 
 	virtual void jump_forward_to_mark () = 0;
 	virtual void jump_backward_to_mark () = 0;
@@ -261,11 +280,13 @@ public:
 
 	/** Import existing media */
 	virtual void do_import (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode, ARDOUR::SrcQuality,
-	                        ARDOUR::MidiTrackNameSource, ARDOUR::MidiTempoMapDisposition, samplepos_t&,
+	                        ARDOUR::MidiTrackNameSource, ARDOUR::MidiTempoMapDisposition, Temporal::timepos_t&,
 	                        boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>(),
+	                        boost::shared_ptr<ARDOUR::Track> track = boost::shared_ptr<ARDOUR::Track>(),
 	                        bool with_markers = false) = 0;
-	virtual void do_embed (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode, samplepos_t&,
-	                       boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>()) = 0;
+	virtual void do_embed (std::vector<std::string> paths, Editing::ImportDisposition, Editing::ImportMode mode, Temporal::timepos_t&,
+	                       boost::shared_ptr<ARDOUR::PluginInfo> instrument = boost::shared_ptr<ARDOUR::PluginInfo>(),
+	                       boost::shared_ptr<ARDOUR::Track> track = boost::shared_ptr<ARDOUR::Track>()) = 0;
 
 	/** Open main export dialog */
 	virtual void export_audio () = 0;
@@ -314,6 +335,8 @@ public:
 	virtual void toggle_stationary_playhead () = 0;
 	virtual bool stationary_playhead() const = 0;
 
+	virtual void toggle_cue_behavior () = 0;
+
 	/** Set whether the editor should follow the playhead.
 	 * @param yn true to follow playhead, otherwise false.
 	 * @param catch_up true to reset the editor view to show the playhead (if yn == true), otherwise false.
@@ -350,18 +373,25 @@ public:
 	virtual void remove_last_capture () = 0;
 	virtual void maximise_editing_space () = 0;
 	virtual void restore_editing_space () = 0;
-	virtual samplepos_t get_preferred_edit_position (Editing::EditIgnoreOption = Editing::EDIT_IGNORE_NONE, bool from_context_menu = false, bool from_outside_canvas = false) = 0;
+	virtual Temporal::timepos_t get_preferred_edit_position (Editing::EditIgnoreOption = Editing::EDIT_IGNORE_NONE, bool from_context_menu = false, bool from_outside_canvas = false) = 0;
 	virtual void toggle_meter_updating() = 0;
-	virtual void split_regions_at (ARDOUR::MusicSample, RegionSelection&) = 0;
+	virtual void split_regions_at (Temporal::timepos_t const &, RegionSelection&) = 0;
 	virtual void split_region_at_points (boost::shared_ptr<ARDOUR::Region>, ARDOUR::AnalysisFeatureList&, bool can_ferret, bool select_new = false) = 0;
-	virtual void mouse_add_new_marker (samplepos_t where, bool is_cd=false) = 0;
+	virtual void mouse_add_new_marker (Temporal::timepos_t where, ARDOUR::Location::Flags extra_flags = ARDOUR::Location::Flags (0), int32_t cue_id = 0) = 0;
 	virtual void foreach_time_axis_view (sigc::slot<void,TimeAxisView&>) = 0;
 	virtual void add_to_idle_resize (TimeAxisView*, int32_t) = 0;
-	virtual samplecnt_t get_nudge_distance (samplepos_t pos, samplecnt_t& next) = 0;
-	virtual samplecnt_t get_paste_offset (samplepos_t pos, unsigned paste_count, samplecnt_t duration) = 0;
-	virtual unsigned get_grid_beat_divisions(samplepos_t position) = 0;
-	virtual Temporal::Beats get_grid_type_as_beats (bool& success, samplepos_t position) = 0;
-	virtual int32_t get_grid_music_divisions (uint32_t event_state) = 0;
+	virtual Temporal::timecnt_t get_nudge_distance (Temporal::timepos_t const & pos, Temporal::timecnt_t& next) = 0;
+	virtual Temporal::timecnt_t get_paste_offset (Temporal::timepos_t const & pos, unsigned paste_count, Temporal::timecnt_t const & duration) = 0;
+
+	virtual Temporal::Beats get_grid_type_as_beats (bool& success, Temporal::timepos_t const & position) = 0;
+	virtual Temporal::Beats get_draw_length_as_beats (bool& success, Temporal::timepos_t const & position) = 0;
+
+	virtual int draw_velocity () const = 0;
+	virtual int draw_channel () const = 0;
+
+	virtual int32_t get_grid_beat_divisions (Editing::GridType gt) = 0;
+	virtual int32_t get_grid_music_divisions (Editing::GridType gt, uint32_t event_state) = 0;
+
 	virtual void edit_notes (MidiRegionView*) = 0;
 
 	virtual void queue_visual_videotimeline_update () = 0;
@@ -374,6 +404,8 @@ public:
 	virtual void set_video_timeline_height (const int h) = 0;
 	virtual void embed_audio_from_video (std::string, samplepos_t n = 0, bool lock_position_to_video = true) = 0;
 
+	virtual void trigger_script_by_name (const std::string script_name, const std::string args = "") = 0;
+
 	virtual bool track_selection_change_without_scroll () const = 0;
 	virtual bool show_touched_automation () const = 0;
 
@@ -401,6 +433,9 @@ public:
 			: action (a), target (tgt) {}
 	};
 
+	/* data-type of [region] object currently dragged with x-ardour/region.pbdid */
+	static ARDOUR::DataType pbdid_dragged_dt;
+
 	std::map<std::string,RegionAction> region_action_map;
 
 	Glib::RefPtr<Gtk::ActionGroup> editor_actions;
@@ -432,6 +467,7 @@ public:
 	virtual bool canvas_tempo_marker_event (GdkEvent* event, ArdourCanvas::Item*, TempoMarker*) = 0;
 	virtual bool canvas_tempo_curve_event (GdkEvent* event, ArdourCanvas::Item*, TempoCurve*) = 0;
 	virtual bool canvas_meter_marker_event (GdkEvent* event, ArdourCanvas::Item*, MeterMarker*) = 0;
+	virtual bool canvas_bbt_marker_event (GdkEvent* event, ArdourCanvas::Item*, BBTMarker*) = 0;
 	virtual bool canvas_automation_track_event(GdkEvent* event, ArdourCanvas::Item*, AutomationTimeAxisView*) = 0;
 
 	virtual bool canvas_tempo_bar_event (GdkEvent* event, ArdourCanvas::Item*) = 0;
@@ -491,31 +527,39 @@ public:
 
 	virtual void get_pointer_position (double &, double &) const = 0;
 
-	virtual std::pair <samplepos_t, samplepos_t> session_gui_extents (bool use_extra = true) const = 0;
+	virtual std::pair <Temporal::timepos_t, Temporal::timepos_t> session_gui_extents (bool use_extra = true) const = 0;
 
 	virtual ARDOUR::Location* find_location_from_marker (ArdourMarker*, bool&) const = 0;
 	virtual ArdourMarker* find_marker_from_location_id (PBD::ID const&, bool) const = 0;
 
-	virtual void snap_to_with_modifier (ARDOUR::MusicSample& first,
+	virtual void snap_to_with_modifier (Temporal::timepos_t & first,
 	                                    GdkEvent const*      ev,
-	                                    ARDOUR::RoundMode    direction = ARDOUR::RoundNearest,
+	                                    Temporal::RoundMode    direction = Temporal::RoundNearest,
 	                                    ARDOUR::SnapPref     gpref = ARDOUR::SnapToAny_Visual) = 0;
+	virtual Temporal::timepos_t snap_to_bbt (Temporal::timepos_t const & pos, Temporal::RoundMode, ARDOUR::SnapPref) = 0;
 
-	virtual void set_snapped_cursor_position (samplepos_t pos) = 0;
+	virtual void set_snapped_cursor_position (Temporal::timepos_t const & pos) = 0;
 
-	virtual void get_regions_at (RegionSelection &, samplepos_t where, TrackViewList const &) const = 0;
-	virtual void get_regions_after (RegionSelection&, samplepos_t where, const TrackViewList& ts) const = 0;
-	virtual RegionSelection get_regions_from_selection_and_mouse (samplepos_t) = 0;
+	virtual void get_regions_at (RegionSelection &, Temporal::timepos_t const & where, TrackViewList const &) const = 0;
+	virtual void get_regions_after (RegionSelection&, Temporal::timepos_t const & where, const TrackViewList& ts) const = 0;
+	virtual RegionSelection get_regions_from_selection_and_mouse (Temporal::timepos_t const &) = 0;
 	virtual void get_regionviews_by_id (PBD::ID const id, RegionSelection & regions) const = 0;
 	virtual void get_per_region_note_selection (std::list<std::pair<PBD::ID, std::set<boost::shared_ptr<Evoral::Note<Temporal::Beats> > > > >&) const = 0;
 
 	virtual void build_region_boundary_cache () = 0;
 	virtual void mark_region_boundary_cache_dirty () = 0;
 
-	virtual void mouse_add_new_tempo_event (samplepos_t where) = 0;
-	virtual void mouse_add_new_meter_event (samplepos_t where) = 0;
-	virtual void edit_tempo_section (ARDOUR::TempoSection*) = 0;
-	virtual void edit_meter_section (ARDOUR::MeterSection*) = 0;
+	virtual void mouse_add_new_tempo_event (Temporal::timepos_t where) = 0;
+	virtual void mouse_add_new_meter_event (Temporal::timepos_t where) = 0;
+	virtual void edit_tempo_section (Temporal::TempoPoint&) = 0;
+	virtual void edit_meter_section (Temporal::MeterPoint&) = 0;
+
+	virtual bool should_ripple () const = 0;
+
+	virtual void queue_redisplay_track_views () = 0;
+
+	virtual ARDOUR::Quantize get_quantize_op (bool force_dialog, bool& did_show_dialog) = 0;
+	virtual void apply_midi_note_edit_op (ARDOUR::MidiOperator& op, const RegionSelection& rs) = 0;
 
 	/// Singleton instance, set up by Editor::Editor()
 
