@@ -30,9 +30,9 @@
 
 #include <gtkmm2ext/gtk_ui.h>
 
+#include "temporal/tempo.h"
+
 #include "ardour/playlist.h"
-#include "ardour/profile.h"
-#include "ardour/session.h"
 #include "ardour/source.h"
 
 #include "gtkmm2ext/colors.h"
@@ -47,11 +47,9 @@
 #include "streamview.h"
 #include "region_view.h"
 #include "automation_region_view.h"
-#include "route_time_axis.h"
 #include "public_editor.h"
 #include "region_editor.h"
 #include "ghostregion.h"
-#include "route_time_axis.h"
 #include "ui_config.h"
 #include "utils.h"
 #include "rgb_macros.h"
@@ -71,41 +69,43 @@ static const int32_t sync_mark_width = 9;
 
 PBD::Signal1<void,RegionView*> RegionView::RegionViewGoingAway;
 
-RegionView::RegionView (ArdourCanvas::Container*          parent,
-                        TimeAxisView&                     tv,
-                        boost::shared_ptr<ARDOUR::Region> r,
-                        double                            spu,
-                        uint32_t                          basic_color,
-                        bool                              automation)
-	: TimeAxisViewItem (r->name(), *parent, tv, spu, basic_color, r->position(), r->length(), false, automation,
+RegionView::RegionView (ArdourCanvas::Container*                 parent,
+                        TimeAxisView&                            tv,
+                        const boost::shared_ptr<ARDOUR::Region>& r,
+                        double                                   spu,
+                        uint32_t                                 basic_color,
+                        bool                                     automation)
+        : TimeAxisViewItem (r->name(), *parent, tv, spu, basic_color, r->position(), r->length(), false, automation,
 			    (automation ? TimeAxisViewItem::ShowFrame :
 			     TimeAxisViewItem::Visibility ((UIConfiguration::instance().get_show_region_name() ? TimeAxisViewItem::ShowNameText : 0) |
 							   TimeAxisViewItem::ShowNameHighlight| TimeAxisViewItem::ShowFrame)))
-	, _region (r)
-	, sync_mark(0)
-	, sync_line(0)
-	, editor(0)
-	, current_visible_sync_position(0.0)
-	, valid(false)
-	, _enable_display(false)
-	, _pixel_width(1.0)
-	, in_destructor(false)
-	, wait_for_data(false)
-	, _silence_text (0)
-	, _xrun_markers_visible (false)
-	, _cue_markers_visible (false)
+        , _region (r)
+        , sync_mark (nullptr)
+        , sync_line (nullptr)
+        , editor (nullptr)
+        , current_visible_sync_position (0.0)
+        , valid (false)
+        , _disable_display (0)
+        , _pixel_width (1.0)
+        , in_destructor (false)
+        , wait_for_data (false)
+        , _silence_text (nullptr)
+        , _xrun_markers_visible (false)
+        , _cue_markers_visible (false)
 {
 	UIConfiguration::instance().ParameterChanged.connect (sigc::mem_fun (*this, &RegionView::parameter_changed));
 
-	for (SourceList::const_iterator s = _region->sources().begin(); s != _region->sources().end(); ++s) {
-		(*s)->CueMarkersChanged.connect (*this, invalidator (*this), boost::bind (&RegionView::update_cue_markers, this), gui_context());
+	for (const auto& s : _region->sources ()) {
+		s->CueMarkersChanged.connect (*this, invalidator (*this), [this] { update_cue_markers (); }, gui_context ());
 	}
 }
 
 RegionView::RegionView (const RegionView& other)
 	: sigc::trackable(other)
 	, TimeAxisViewItem (other)
-	, _silence_text (0)
+	, sync_mark (nullptr)
+	, sync_line (nullptr)
+	, _silence_text (nullptr)
 	, _xrun_markers_visible (false)
 	, _cue_markers_visible (false)
 {
@@ -118,15 +118,17 @@ RegionView::RegionView (const RegionView& other)
 	valid = false;
 	_pixel_width = other._pixel_width;
 
-	for (SourceList::const_iterator s = _region->sources().begin(); s != _region->sources().end(); ++s) {
-		(*s)->CueMarkersChanged.connect (*this, invalidator (*this), boost::bind (&RegionView::update_cue_markers, this), gui_context());
+	for (const auto& s : _region->sources ()) {
+		s->CueMarkersChanged.connect (*this, invalidator (*this), [this] { update_cue_markers (); }, gui_context ());
 	}
 }
 
-RegionView::RegionView (const RegionView& other, boost::shared_ptr<Region> other_region)
+RegionView::RegionView (const RegionView& other, const boost::shared_ptr<Region>& other_region)
 	: sigc::trackable(other)
 	, TimeAxisViewItem (other)
-	, _silence_text (0)
+	, sync_mark (nullptr)
+	, sync_line (nullptr)
+	, _silence_text (nullptr)
 	, _xrun_markers_visible (false)
 	, _cue_markers_visible (false)
 {
@@ -144,50 +146,45 @@ RegionView::RegionView (const RegionView& other, boost::shared_ptr<Region> other
 	valid = false;
 	_pixel_width = other._pixel_width;
 
-	for (SourceList::const_iterator s = _region->sources().begin(); s != _region->sources().end(); ++s) {
-		(*s)->CueMarkersChanged.connect (*this, invalidator (*this), boost::bind (&RegionView::update_cue_markers, this), gui_context());
+	for (const auto& s : _region->sources ()) {
+		s->CueMarkersChanged.connect (*this, invalidator (*this), [this] { update_cue_markers (); }, gui_context ());
 	}
 }
 
-
-RegionView::RegionView (ArdourCanvas::Container*          parent,
-                        TimeAxisView&                     tv,
-                        boost::shared_ptr<ARDOUR::Region> r,
-                        double                            spu,
-                        uint32_t                          basic_color,
-                        bool                              recording,
-                        TimeAxisViewItem::Visibility      visibility)
-	: TimeAxisViewItem (r->name(), *parent, tv, spu, basic_color, r->position(), r->length(), recording, false, visibility)
-	, _region (r)
-	, sync_mark(0)
-	, sync_line(0)
-	, editor(0)
-	, current_visible_sync_position(0.0)
-	, valid(false)
-	, _enable_display(false)
-	, _pixel_width(1.0)
-	, in_destructor(false)
-	, wait_for_data(false)
-	, _silence_text (0)
+RegionView::RegionView (ArdourCanvas::Container*                 parent,
+                        TimeAxisView&                            tv,
+                        const boost::shared_ptr<ARDOUR::Region>& r,
+                        double                                   spu,
+                        uint32_t                                 basic_color,
+                        bool                                     recording,
+                        TimeAxisViewItem::Visibility             visibility)
+        : TimeAxisViewItem (r->name(), *parent, tv, spu, basic_color, r->position(), r->length(), recording, false, visibility)
+        , _region (r)
+        , sync_mark (nullptr)
+        , sync_line (nullptr)
+        , editor (nullptr)
+        , current_visible_sync_position (0.0)
+        , valid (false)
+        , _disable_display (0)
+        , _pixel_width (1.0)
+        , in_destructor (false)
+        , wait_for_data (false)
+        , _silence_text (nullptr)
 {
 	UIConfiguration::instance().ParameterChanged.connect (sigc::mem_fun (*this, &RegionView::parameter_changed));
 
-	for (SourceList::const_iterator s = _region->sources().begin(); s != _region->sources().end(); ++s) {
-		(*s)->CueMarkersChanged.connect (*this, invalidator (*this), boost::bind (&RegionView::update_cue_markers, this), gui_context());
+	for (const auto& s : _region->sources ()) {
+		s->CueMarkersChanged.connect (*this, invalidator (*this), [this] { update_cue_markers (); }, gui_context ());
 	}
 }
 
 void
 RegionView::init (bool wfd)
 {
-	editor        = 0;
+	editor        = nullptr;
 	valid         = true;
 	in_destructor = false;
 	wait_for_data = wfd;
-	sync_mark     = 0;
-	sync_line     = 0;
-	sync_mark     = 0;
-	sync_line     = 0;
 
 	if (name_highlight) {
 		name_highlight->set_data ("regionview", this);
@@ -216,8 +213,8 @@ RegionView::init (bool wfd)
 	XrunPositions xrp;
 	_region->captured_xruns (xrp, true);
 	int arrow_size = (int)(7.0 * UIConfiguration::instance ().get_ui_scale ()) & ~1;
-	for (XrunPositions::const_iterator x = xrp.begin (); x != xrp.end (); ++x) {
-		ArdourCanvas::Arrow* canvas_item = new ArdourCanvas::Arrow(group);
+	for (long x : xrp) {
+		auto* canvas_item = new ArdourCanvas::Arrow(group);
 		canvas_item->set_color (UIConfiguration::instance().color ("neutral:background"));
 		canvas_item->set_show_head (1, true);
 		canvas_item->set_show_head (0, false);
@@ -227,7 +224,7 @@ RegionView::init (bool wfd)
 		canvas_item->set_y1 (arrow_size);
 		canvas_item->raise_to_top ();
 		canvas_item->hide ();
-		_xrun_markers.push_back (make_pair(*x, canvas_item));
+		_xrun_markers.emplace_back(x, canvas_item);
 	}
 
 	_xrun_markers_visible = false;
@@ -236,10 +233,6 @@ RegionView::init (bool wfd)
 	_cue_markers_visible = false;
 	update_cue_markers ();
 
-	if (wfd) {
-		_enable_display = true;
-	}
-
 	/* derived class calls set_height () including RegionView::set_height() in ::init() */
 	//set_height (trackview.current_height());
 
@@ -256,20 +249,20 @@ RegionView::~RegionView ()
 {
 	in_destructor = true;
 
-	for (vector<GhostRegion*>::iterator g = ghosts.begin(); g != ghosts.end(); ++g) {
-		delete *g;
+	for (auto& ghost : ghosts) {
+		delete ghost;
 	}
 
-	for (list<ArdourCanvas::Rectangle*>::iterator i = _coverage_frame.begin (); i != _coverage_frame.end (); ++i) {
-		delete *i;
+	for (auto& i : _coverage_frame) {
+		delete i;
 	}
 
-	for (list<std::pair<samplepos_t, ArdourCanvas::Arrow*> >::iterator i = _xrun_markers.begin(); i != _xrun_markers.end(); ++i) {
-		delete ((*i).second);
+	for (auto& _xrun_marker : _xrun_markers) {
+		delete (_xrun_marker.second);
 	}
 
-	for (ViewCueMarkers::iterator c = _cue_markers.begin(); c != _cue_markers.end(); ++c) {
-		delete (*c);
+	for (auto& _cue_marker : _cue_markers) {
+		delete _cue_marker;
 	}
 
 	drop_silent_frames ();
@@ -300,38 +293,37 @@ RegionView::set_silent_frames (const AudioIntervalResult& silences, double /*thr
 
 	uint32_t const color = UIConfiguration::instance().color_mod ("silence", "silence");
 
-	for (AudioIntervalResult::const_iterator i = silences.begin(); i != silences.end(); ++i) {
-
-		ArdourCanvas::Rectangle* cr = new ArdourCanvas::Rectangle (group);
+	for (const auto& silence : silences) {
+		auto* cr = new ArdourCanvas::Rectangle (group);
 		cr->set_ignore_events (true);
 		_silent_frames.push_back (cr);
 
 		/* coordinates for the rect are relative to the regionview origin */
 
-		cr->set_x0 (trackview.editor().sample_to_pixel (i->first - _region->start()));
-		cr->set_x1 (trackview.editor().sample_to_pixel (i->second - _region->start()));
+		cr->set_x0 (trackview.editor ().sample_to_pixel (silence.first - _region->start_sample ()));
+		cr->set_x1 (trackview.editor ().sample_to_pixel (silence.second - _region->start_sample ()));
 		cr->set_y0 (1);
 		cr->set_y1 (_height - 2);
 		cr->set_outline (false);
 		cr->set_fill_color (color);
 
-		shortest = min (shortest, i->second - i->first);
+		shortest = min (shortest, silence.second - silence.first);
 	}
 
 	/* Find shortest audible segment */
 	samplecnt_t shortest_audible = max_samplecnt;
 
-	samplecnt_t s = _region->start();
-	for (AudioIntervalResult::const_iterator i = silences.begin(); i != silences.end(); ++i) {
-		samplecnt_t const dur = i->first - s;
+	samplecnt_t s = _region->start_sample();
+	for (const auto& silence : silences) {
+		samplecnt_t const dur = silence.first - s;
 		if (dur > 0) {
 			shortest_audible = min (shortest_audible, dur);
 		}
 
-		s = i->second;
+		s = silence.second;
 	}
 
-	samplecnt_t const dur = _region->start() + _region->length() - 1 - s;
+	samplecnt_t const dur = _region->start_sample() + _region->length_samples() - 1 - s;
 	if (dur > 0) {
 		shortest_audible = min (shortest_audible, dur);
 	}
@@ -343,7 +335,7 @@ RegionView::set_silent_frames (const AudioIntervalResult& silences, double /*thr
 
 	/* both positions are relative to the region start offset in source */
 
-	_silence_text->set_x_position (trackview.editor().sample_to_pixel (silences.front().first - _region->start()) + 10.0);
+	_silence_text->set_x_position (trackview.editor().sample_to_pixel (silences.front().first - _region->start_sample()) + 10.0);
 	_silence_text->set_y_position (20.0);
 
 	double ms = (float) shortest/_region->session().sample_rate();
@@ -390,22 +382,22 @@ RegionView::set_silent_frames (const AudioIntervalResult& silences, double /*thr
 void
 RegionView::hide_silent_frames ()
 {
-	for (list<ArdourCanvas::Rectangle*>::iterator i = _silent_frames.begin (); i != _silent_frames.end (); ++i) {
-		(*i)->hide ();
+	for (auto& _silent_frame : _silent_frames) {
+		_silent_frame->hide ();
 	}
-	_silence_text->hide();
+	_silence_text->hide ();
 }
 
 void
 RegionView::drop_silent_frames ()
 {
-	for (list<ArdourCanvas::Rectangle*>::iterator i = _silent_frames.begin (); i != _silent_frames.end (); ++i) {
-		delete *i;
+	for (auto& _silent_frame : _silent_frames) {
+		delete _silent_frame;
 	}
 	_silent_frames.clear ();
 
 	delete _silence_text;
-	_silence_text = 0;
+	_silence_text = nullptr;
 }
 
 gint
@@ -431,6 +423,8 @@ RegionView::lock_toggle ()
 void
 RegionView::region_changed (const PropertyChange& what_changed)
 {
+	DisplaySuspender ds (*this, false);
+
 	ENSURE_GUI_THREAD (*this, &RegionView::region_changed, what_changed);
 
 	if (what_changed.contains (ARDOUR::bounds_change)) {
@@ -447,7 +441,7 @@ RegionView::region_changed (const PropertyChange& what_changed)
 	if (what_changed.contains (ARDOUR::Properties::name)) {
 		region_renamed ();
 	}
-	if (what_changed.contains (ARDOUR::Properties::position_lock_style)) {
+	if (what_changed.contains (ARDOUR::Properties::time_domain)) {
 		region_renamed ();
 	}
 	if (what_changed.contains (ARDOUR::Properties::sync_position)) {
@@ -470,7 +464,9 @@ RegionView::region_resized (const PropertyChange& what_changed)
 {
 	double unit_length;
 
-	if (what_changed.contains (ARDOUR::Properties::position)) {
+	/* position is stored in length */
+
+	if (what_changed.contains (ARDOUR::Properties::length)) {
 		set_position (_region->position(), 0);
 	}
 
@@ -482,10 +478,10 @@ RegionView::region_resized (const PropertyChange& what_changed)
 
 		set_duration (_region->length(), 0);
 
-		unit_length = _region->length() / samples_per_pixel;
+		unit_length = _region->length_samples() / samples_per_pixel;
 
-		for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
-			(*i)->set_duration (unit_length);
+		for (auto& ghost : ghosts) {
+			ghost->set_duration (unit_length);
 		}
 
 		update_xrun_markers ();
@@ -499,10 +495,10 @@ RegionView::reset_width_dependent_items (double pixel_width)
 	_pixel_width = pixel_width;
 
 	if (_xrun_markers_visible) {
-		const samplepos_t start = _region->start();
-		for (list<std::pair<samplepos_t, ArdourCanvas::Arrow*> >::iterator i = _xrun_markers.begin(); i != _xrun_markers.end(); ++i) {
-			float x_pos = trackview.editor().sample_to_pixel (i->first - start);
-			i->second->set_x (x_pos);
+		const samplepos_t start = _region->start_sample();
+		for (auto& _xrun_marker : _xrun_markers) {
+			float x_pos = trackview.editor ().sample_to_pixel (_xrun_marker.first - start);
+			_xrun_marker.second->set_x (x_pos);
 		}
 	}
 }
@@ -515,15 +511,15 @@ RegionView::update_xrun_markers ()
 		return;
 	}
 
-	const samplepos_t start = _region->start();
-	const samplepos_t length = _region->length();
-	for (list<std::pair<samplepos_t, ArdourCanvas::Arrow*> >::iterator i = _xrun_markers.begin(); i != _xrun_markers.end(); ++i) {
-		float x_pos = trackview.editor().sample_to_pixel (i->first - start);
-		i->second->set_x (x_pos);
-		if (show_xruns_markers && (i->first >= start && i->first < start + length)) {
-			i->second->show ();
-		} else  {
-			i->second->hide ();
+	const samplepos_t start = _region->start_sample();
+	const samplepos_t length = _region->length_samples();
+	for (auto& _xrun_marker : _xrun_markers) {
+		float x_pos = trackview.editor ().sample_to_pixel (_xrun_marker.first - start);
+		_xrun_marker.second->set_x (x_pos);
+		if (show_xruns_markers && (_xrun_marker.first >= start && _xrun_marker.first < start + length)) {
+			_xrun_marker.second->show ();
+		} else {
+			_xrun_marker.second->hide ();
 		}
 	}
 	_xrun_markers_visible = show_xruns_markers;
@@ -543,8 +539,8 @@ RegionView::update_cue_markers ()
 		return;
 	}
 
-	const samplepos_t start = region()->start();
-	const samplepos_t end = region()->start() + region()->length();
+	const timepos_t start = region()->start();
+	const timepos_t end = region()->start() + region()->length();
 	const Gtkmm2ext::SVAModifier alpha = UIConfiguration::instance().modifier (X_("region mark"));
 	const uint32_t color = Gtkmm2ext::HSV (UIConfiguration::instance().color ("region mark")).mod (alpha).color();
 
@@ -559,8 +555,8 @@ RegionView::update_cue_markers ()
 	 * marker set
 	 */
 
-	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ) {
-		if (model_markers.find ((*v)->model_marker) == model_markers.end()) {
+	for (auto v = _cue_markers.begin (); v != _cue_markers.end ();) {
+		if (model_markers.find ((*v)->model_marker) == model_markers.end ()) {
 			delete *v;
 			v = _cue_markers.erase (v);
 		} else {
@@ -574,27 +570,25 @@ RegionView::update_cue_markers ()
 	 * timestamps and some of them may be outside the Region.
 	 */
 
-	for (CueMarkers::const_iterator c = model_markers.begin(); c != model_markers.end(); c++) {
+	for (const auto& model_marker : model_markers) {
+		auto existing = _cue_markers.end ();
 
-		ViewCueMarkers::iterator existing = _cue_markers.end();
-
-		for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
-			if ((*v)->model_marker == *c) {
+		for (auto v = _cue_markers.begin (); v != _cue_markers.end (); ++v) {
+			if ((*v)->model_marker == model_marker) {
 				existing = v;
 				break;
 			}
 		}
 
-		if (existing == _cue_markers.end()) {
-
-			if (c->position() < start || c->position() >= end) {
-				/* not withing this region */
+		if (existing == _cue_markers.end ()) {
+			if (model_marker.position () < start || model_marker.position () >= end) {
+				/* not within this region */
 				continue;
 			}
 
 			/* Create a new ViewCueMarker */
 
-			ArdourMarker* mark = new ArdourMarker (trackview.editor(), *group, color , c->text(), ArdourMarker::RegionCue, c->position() - start, true, this);
+			auto* mark = new ArdourMarker (trackview.editor(), *group, color , model_marker.text(), ArdourMarker::RegionCue, timepos_t (start.distance (model_marker.position())), true, this);
 			mark->set_points_color (color);
 			mark->set_show_line (true);
 			/* make sure the line has a clean end, before the frame
@@ -609,12 +603,12 @@ RegionView::update_cue_markers ()
 				mark->hide ();
 			}
 
-			_cue_markers.push_back (new ViewCueMarker (mark, *c));
+			_cue_markers.push_back (new ViewCueMarker (mark, model_marker));
 
 		} else {
 
-			if (c->position() < start || c->position() >= end) {
-				/* not withing this region */
+			if (model_marker.position() < start || model_marker.position() >= end) {
+				/* not within this region */
 				delete (*existing);
 				_cue_markers.erase (existing);
 				continue;
@@ -628,7 +622,7 @@ RegionView::update_cue_markers ()
 				(*existing)->view_marker->hide ();
 			}
 
-			(*existing)->view_marker->set_position (c->position() - start);
+			(*existing)->view_marker->set_position (timepos_t (start.distance (model_marker.position ())));
 			(*existing)->view_marker->the_item().raise_to_top ();
 		}
 	}
@@ -662,7 +656,7 @@ RegionView::lower_to_bottom ()
 }
 
 bool
-RegionView::set_position (samplepos_t pos, void* /*src*/, double* ignored)
+RegionView::set_position (timepos_t const & pos, void* /*src*/, double* ignored)
 {
 	double delta;
 	bool ret;
@@ -676,8 +670,8 @@ RegionView::set_position (samplepos_t pos, void* /*src*/, double* ignored)
 	}
 
 	if (delta) {
-		for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
-			(*i)->group->move (ArdourCanvas::Duple (delta, 0.0));
+		for (auto& ghost : ghosts) {
+			ghost->group->move (ArdourCanvas::Duple (delta, 0.0));
 		}
 	}
 
@@ -689,23 +683,23 @@ RegionView::set_samples_per_pixel (double fpp)
 {
 	TimeAxisViewItem::set_samples_per_pixel (fpp);
 
-	for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
-		(*i)->set_samples_per_pixel (fpp);
-		(*i)->set_duration (_region->length() / fpp);
+	for (auto& ghost : ghosts) {
+		ghost->set_samples_per_pixel (fpp);
+		ghost->set_duration (_region->length_samples () / fpp);
 	}
 
 	region_sync_changed ();
 }
 
 bool
-RegionView::set_duration (samplecnt_t samples, void *src)
+RegionView::set_duration (timecnt_t const & dur, void *src)
 {
-	if (!TimeAxisViewItem::set_duration (samples, src)) {
+	if (!TimeAxisViewItem::set_duration (dur, src)) {
 		return false;
 	}
 
-	for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
-		(*i)->set_duration (_region->length() / samples_per_pixel);
+	for (auto& ghost : ghosts) {
+		ghost->set_duration (_region->length_samples () / samples_per_pixel);
 	}
 
 	return true;
@@ -720,8 +714,8 @@ RegionView::set_colors ()
 	const Gtkmm2ext::SVAModifier alpha = UIConfiguration::instance().modifier (X_("region mark"));
 	const uint32_t color = Gtkmm2ext::HSV (UIConfiguration::instance().color ("region mark")).mod (alpha).color();
 
-	for (ViewCueMarkers::iterator c = _cue_markers.begin(); c != _cue_markers.end(); ++c) {
-		(*c)->view_marker->set_color_rgba (color);
+	for (auto& _cue_marker : _cue_markers) {
+		_cue_marker->view_marker->set_color_rgba (color);
 	}
 }
 
@@ -764,7 +758,7 @@ RegionView::get_fill_color () const
 void
 RegionView::show_region_editor ()
 {
-	if (editor == 0) {
+	if (!editor) {
 		editor = new RegionEditor (trackview.session(), region());
 	}
 
@@ -783,10 +777,10 @@ RegionView::hide_region_editor()
 std::string
 RegionView::make_name () const
 {
-	std::string str;
+	std::string str{};
 
 	// XXX nice to have some good icons for this
-	if (_region->position_lock_style() == MusicTime) {
+	if (_region->position_time_domain() == Temporal::BeatTime) {
 		str += "\u266B"; // BEAMED EIGHTH NOTES
 	}
 
@@ -805,7 +799,11 @@ RegionView::make_name () const
 	}
 
 	if (_region->muted()) {
+#ifdef __APPLE__
 		str = string ("!") + str;
+#else
+		str = std::string("\U0001F507") + str; // SPEAKER WITH CANCELLATION STROKE
+#endif
 	}
 
 	return str;
@@ -824,11 +822,11 @@ void
 RegionView::region_sync_changed ()
 {
 	int sync_dir;
-	samplecnt_t sync_offset;
+	timecnt_t sync_offset;
 
 	sync_offset = _region->sync_offset (sync_dir);
 
-	if (sync_offset == 0) {
+	if (sync_offset.is_zero ()) {
 		/* no need for a sync mark */
 		if (sync_mark) {
 			sync_mark->hide();
@@ -877,7 +875,7 @@ RegionView::region_sync_changed ()
 
 			//points = sync_mark->property_points().get_value();
 
-			double offset = sync_offset / samples_per_pixel;
+			double offset = trackview.editor().duration_to_pixels (sync_offset);
 			points.push_back (ArdourCanvas::Duple (offset - ((sync_mark_width-1)/2), 1));
 			points.push_back (ArdourCanvas::Duple (offset + ((sync_mark_width-1)/2), 1));
 			points.push_back (ArdourCanvas::Duple (offset, sync_mark_width - 1));
@@ -913,17 +911,17 @@ RegionView::move (double x_delta, double y_delta)
 
 	/* note: ghosts never leave their tracks so y_delta for them is always zero */
 
-	for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
-		(*i)->group->move (ArdourCanvas::Duple (x_delta, 0.0));
+	for (auto& ghost : ghosts) {
+		ghost->group->move (ArdourCanvas::Duple (x_delta, 0.0));
 	}
 }
 
 void
 RegionView::remove_ghost_in (TimeAxisView& tv)
 {
-	for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
-		if (&(*i)->trackview == &tv) {
-			delete *i;
+	for (auto& ghost : ghosts) {
+		if (&ghost->trackview == &tv) {
+			delete ghost;
 			break;
 		}
 	}
@@ -936,7 +934,7 @@ RegionView::remove_ghost (GhostRegion* ghost)
 		return;
 	}
 
-	for (vector<GhostRegion*>::iterator i = ghosts.begin(); i != ghosts.end(); ++i) {
+	for (auto i = ghosts.begin(); i != ghosts.end(); ++i) {
 		if (*i == ghost) {
 			ghosts.erase (i);
 			break;
@@ -952,9 +950,9 @@ RegionView::set_height (double h)
 	if (sync_line) {
 		Points points;
 		int sync_dir;
-		samplecnt_t sync_offset;
+		timecnt_t sync_offset;
 		sync_offset = _region->sync_offset (sync_dir);
-		double offset = sync_offset / samples_per_pixel;
+		double offset = trackview.editor().duration_to_pixels (sync_offset);
 
 		sync_line->set (
 			ArdourCanvas::Duple (offset, 0),
@@ -962,16 +960,16 @@ RegionView::set_height (double h)
 			);
 	}
 
-	for (list<ArdourCanvas::Rectangle*>::iterator i = _coverage_frame.begin(); i != _coverage_frame.end(); ++i) {
-		(*i)->set_y1 (h + 1);
+	for (auto& i : _coverage_frame) {
+		i->set_y1 (h + 1);
 	}
 
-	for (list<ArdourCanvas::Rectangle*>::iterator i = _silent_frames.begin(); i != _silent_frames.end(); ++i) {
-		(*i)->set_y1 (h + 1);
+	for (auto& _silent_frame : _silent_frames) {
+		_silent_frame->set_y1 (h + 1);
 	}
 
-	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
-		(*v)->view_marker->set_line_height (h - (1.5 * UIConfiguration::instance().get_ui_scale()));
+	for (auto& _cue_marker : _cue_markers) {
+		_cue_marker->view_marker->set_line_height (h - (1.5 * UIConfiguration::instance ().get_ui_scale ()));
 	}
 }
 
@@ -981,8 +979,8 @@ void
 RegionView::update_coverage_frame (LayerDisplay d)
 {
 	/* remove old coverage frame */
-	for (list<ArdourCanvas::Rectangle*>::iterator i = _coverage_frame.begin (); i != _coverage_frame.end (); ++i) {
-		delete *i;
+	for (auto& i : _coverage_frame) {
+		delete i;
 	}
 
 	_coverage_frame.clear ();
@@ -997,11 +995,11 @@ RegionView::update_coverage_frame (LayerDisplay d)
 		return;
 	}
 
-	samplepos_t const position = _region->first_sample ();
-	samplepos_t t = position;
-	samplepos_t const end = _region->last_sample ();
+	timepos_t const position = _region->position ();
+	timepos_t t (position);
+	timepos_t const end = _region->nt_last ();
 
-	ArdourCanvas::Rectangle* cr = 0;
+	ArdourCanvas::Rectangle* cr = nullptr;
 	bool me = false;
 
 	/* the color that will be used to show parts of regions that will not be heard */
@@ -1010,42 +1008,37 @@ RegionView::update_coverage_frame (LayerDisplay d)
 
 	while (t < end) {
 
-		t++;
+		t = t.increment ();
 
 		/* is this region is on top at time t? */
-		bool const new_me = (pl->top_unmuted_region_at (t) == _region);
+		bool const new_me = pl->region_is_audible_at (_region, t);
 		/* finish off any old rect, if required */
 		if (cr && me != new_me) {
-			cr->set_x1 (trackview.editor().sample_to_pixel (t - position));
+			cr->set_x1 (trackview.editor().duration_to_pixels (position.distance (t)));
+			cr = 0;
 		}
 
 		/* start off any new rect, if required */
-		if (cr == 0 || me != new_me) {
+		if (!cr && !new_me) {
 			cr = new ArdourCanvas::Rectangle (group);
 			_coverage_frame.push_back (cr);
-			cr->set_x0 (trackview.editor().sample_to_pixel (t - position));
+			cr->set_x0 (trackview.editor().duration_to_pixels (position.distance (t)));
 			cr->set_y0 (1);
 			cr->set_y1 (_height + 1);
 			cr->set_outline (false);
 			cr->set_ignore_events (true);
-			if (new_me) {
-				cr->set_fill_color (UINT_RGBA_CHANGE_A (non_playing_color, 0));
-			} else {
-				cr->set_fill_color (non_playing_color);
-			}
+			cr->set_fill_color (non_playing_color);
 		}
 		t = pl->find_next_region_boundary (t, 1);
-		if (t < 0) {
+		if (t == timepos_t::max (t.time_domain())) {
 			break;
 		}
 		me = new_me;
 	}
 
-	t = pl->find_next_region_boundary (t, 1);
-
 	if (cr) {
 		/* finish off the last rectangle */
-		cr->set_x1 (trackview.editor().sample_to_pixel (end - position));
+		cr->set_x1 (trackview.editor().duration_to_pixels (position.distance (end)));
 	}
 
 	if (frame_handle_start) {
@@ -1066,67 +1059,67 @@ RegionView::update_coverage_frame (LayerDisplay d)
 }
 
 bool
-RegionView::trim_front (samplepos_t new_bound, bool no_overlap, const int32_t sub_num)
+RegionView::trim_front (timepos_t const & new_bound, bool no_overlap)
 {
 	if (_region->locked()) {
 		return false;
 	}
 
-	samplepos_t const pre_trim_first_sample = _region->first_sample();
-
 	if (_region->position() == new_bound) {
 		return false;
 	}
 
-	_region->trim_front (new_bound, sub_num);
+	timepos_t const pos = _region->position();
+
+	_region->trim_front (new_bound);
 
 	if (no_overlap) {
 		/* Get the next region on the left of this region and shrink/expand it. */
 		boost::shared_ptr<Playlist> playlist (_region->playlist());
-		boost::shared_ptr<Region> region_left = playlist->find_next_region (pre_trim_first_sample, End, 0);
+		boost::shared_ptr<Region> region_left = playlist->find_next_region (pos, End, 0);
 
 		bool regions_touching = false;
 
-		if (region_left != 0 && (pre_trim_first_sample == region_left->last_sample() + 1)) {
+		if (region_left && (pos == region_left->end())) {
 			regions_touching = true;
 		}
 
 		/* Only trim region on the left if the first sample has gone beyond the left region's last sample. */
-		if (region_left != 0 && (region_left->last_sample() > _region->first_sample() || regions_touching)) {
-			region_left->trim_end (_region->first_sample() - 1);
+		if (region_left && (region_left->nt_last() > _region->position() || regions_touching)) {
+			region_left->trim_end (_region->position().decrement());
 		}
 	}
 
 	region_changed (ARDOUR::bounds_change);
 
-	return (pre_trim_first_sample != _region->first_sample()); // return true if we actually changed something
+	return (pos != _region->position()); // return true if we actually changed something
 }
 
 bool
-RegionView::trim_end (samplepos_t new_bound, bool no_overlap, const int32_t sub_num)
+RegionView::trim_end (timepos_t const & new_bound, bool no_overlap)
 {
 	if (_region->locked()) {
 		return false;
 	}
 
-	samplepos_t const pre_trim_last_sample = _region->last_sample();
+	timepos_t const last = _region->nt_last();
 
-	_region->trim_end (new_bound, sub_num);
+	_region->trim_end (new_bound);
 
 	if (no_overlap) {
 		/* Get the next region on the right of this region and shrink/expand it. */
 		boost::shared_ptr<Playlist> playlist (_region->playlist());
-		boost::shared_ptr<Region> region_right = playlist->find_next_region (pre_trim_last_sample, Start, 1);
+		boost::shared_ptr<Region> region_right = playlist->find_next_region (last, Start, 1);
 
 		bool regions_touching = false;
 
-		if (region_right != 0 && (pre_trim_last_sample == region_right->first_sample() - 1)) {
+		if (region_right && (last == region_right->position().decrement())) {
 			regions_touching = true;
 		}
 
 		/* Only trim region on the right if the last sample has gone beyond the right region's first sample. */
-		if (region_right != 0 && (region_right->first_sample() < _region->last_sample() || regions_touching)) {
-			region_right->trim_front (_region->last_sample() + 1, sub_num);
+		if (region_right && (region_right->position() < _region->nt_last() || regions_touching)) {
+			region_right->trim_front (_region->end());
 		}
 
 		region_changed (ARDOUR::bounds_change);
@@ -1135,7 +1128,7 @@ RegionView::trim_end (samplepos_t new_bound, bool no_overlap, const int32_t sub_
 		region_changed (PropertyChange (ARDOUR::Properties::length));
 	}
 
-	return (pre_trim_last_sample != _region->last_sample()); // return true if we actually changed something
+	return (last != _region->nt_last()); // return true if we actually changed something
 }
 
 
@@ -1151,7 +1144,7 @@ RegionView::thaw_after_trim ()
 
 
 void
-RegionView::move_contents (sampleoffset_t distance)
+RegionView::move_contents (timecnt_t const & distance)
 {
 	if (_region->locked()) {
 		return;
@@ -1160,31 +1153,44 @@ RegionView::move_contents (sampleoffset_t distance)
 	region_changed (PropertyChange (ARDOUR::Properties::start));
 }
 
-/** Snap a sample offset within our region using the current snap settings.
- *  @param x Frame offset from this region's position.
+
+/** Snap a time offset within our region using the current snap settings.
+ *  @param x Time offset from this region's position.
  *  @param ensure_snap whether to ignore snap_mode (in the case of SnapOff) and magnetic snap.
  *  Used when inverting snap mode logic with key modifiers, or snap distance calculation.
- *  @return Snapped sample offset from this region's position.
+ *  @return Snapped time offset from this region's position.
  */
-MusicSample
-RegionView::snap_sample_to_sample (sampleoffset_t x, bool ensure_snap) const
+timepos_t
+RegionView::snap_region_time_to_region_time (timepos_t const & x, bool ensure_snap) const
 {
 	PublicEditor& editor = trackview.editor();
-	/* x is region relative, convert it to global absolute samples */
-	samplepos_t const session_sample = x + _region->position();
+	/* x is region relative, convert it to global absolute time */
+	timepos_t const session_pos = _region->position() + x;
 
 	/* try a snap in either direction */
-	MusicSample sample (session_sample, 0);
-	editor.snap_to (sample, RoundNearest, SnapToAny_Visual, ensure_snap);
+	timepos_t snapped = session_pos;
+	editor.snap_to (snapped, Temporal::RoundNearest, SnapToAny_Visual, ensure_snap);
 
 	/* if we went off the beginning of the region, snap forwards */
-	if (sample.sample < _region->position ()) {
-		sample.sample = session_sample;
-		editor.snap_to (sample, RoundUpAlways, SnapToAny_Visual, ensure_snap);
+	if (snapped < _region->position ()) {
+		snapped = session_pos;
+		editor.snap_to (snapped, Temporal::RoundUpAlways, SnapToAny_Visual, ensure_snap);
 	}
 
-	/* back to region relative, keeping the relevant divisor */
-	return MusicSample (sample.sample - _region->position(), sample.division);
+	/* back to region relative */
+	return _region->region_relative_position (snapped);
+}
+
+timecnt_t
+RegionView::region_relative_distance (timecnt_t const & duration, Temporal::TimeDomain domain)
+{
+	return Temporal::TempoMap::use()->convert_duration (duration, _region->position(), domain);
+}
+
+timecnt_t
+RegionView::source_relative_distance (timecnt_t const & duration, Temporal::TimeDomain domain)
+{
+	return Temporal::TempoMap::use()->convert_duration (duration, _region->source_position(), domain);
 }
 
 void
@@ -1211,27 +1217,27 @@ RegionView::set_selected (bool yn)
 void
 RegionView::maybe_raise_cue_markers ()
 {
-	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
-		(*v)->view_marker->the_item().raise_to_top ();
+	for (auto& _cue_marker : _cue_markers) {
+		_cue_marker->view_marker->the_item ().raise_to_top ();
 	}
 }
 
 CueMarker
 RegionView::find_model_cue_marker (ArdourMarker* m)
 {
-	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
-		if ((*v)->view_marker == m) {
-			return (*v)->model_marker;
+	for (auto& _cue_marker : _cue_markers) {
+		if (_cue_marker->view_marker == m) {
+			return _cue_marker->model_marker;
 		}
 	}
 
-	return CueMarker (string(), 0); /* empty string signifies invalid */
+	return CueMarker (string(), timepos_t()); /* empty string signifies invalid */
 }
 
 void
 RegionView::drop_cue_marker (ArdourMarker* m)
 {
-	for (ViewCueMarkers::iterator v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
+	for (auto v = _cue_markers.begin(); v != _cue_markers.end(); ++v) {
 		if ((*v)->view_marker == m) {
 			delete m;
 			_cue_markers.erase (v);
@@ -1239,3 +1245,27 @@ RegionView::drop_cue_marker (ArdourMarker* m)
 		}
 	}
 }
+
+void
+RegionView::enable_display (bool view_only)
+{
+	if (_disable_display) {
+		_disable_display--;
+		if (_disable_display == 0) {
+			redisplay (view_only);
+		}
+	}
+}
+
+void
+RegionView::disable_display ()
+{
+	_disable_display++;
+}
+
+bool
+RegionView::display_enabled() const
+{
+	return !_disable_display;
+}
+
