@@ -27,7 +27,6 @@
  */
 
 #include <cstdlib>
-#include <cmath>
 #include <cassert>
 
 #include <algorithm>
@@ -43,9 +42,7 @@
 #include <gtkmm/stock.h>
 
 #include "pbd/error.h"
-#include "pbd/stl_delete.h"
 #include "pbd/whitespace.h"
-#include "pbd/memento_command.h"
 #include "pbd/enumwriter.h"
 #include "pbd/stateful_diff_command.h"
 
@@ -53,7 +50,6 @@
 
 #include "ardour/amp.h"
 #include "ardour/meter.h"
-#include "ardour/event_type_map.h"
 #include "ardour/pannable.h"
 #include "ardour/panner.h"
 #include "ardour/plugin_insert.h"
@@ -61,7 +57,6 @@
 #include "ardour/profile.h"
 #include "ardour/route_group.h"
 #include "ardour/session.h"
-#include "ardour/session_playlists.h"
 #include "ardour/track.h"
 
 #include "canvas/debug.h"
@@ -85,11 +80,9 @@
 #include "keyboard.h"
 #include "paste_context.h"
 #include "patch_change_widget.h"
-#include "playlist_selector.h"
 #include "point_selection.h"
 #include "public_editor.h"
 #include "region_view.h"
-#include "rgb_macros.h"
 #include "selection.h"
 #include "streamview.h"
 #include "ui_config.h"
@@ -107,6 +100,8 @@ using namespace Editing;
 using namespace std;
 using std::list;
 
+sigc::signal<void, bool> RouteTimeAxisView::signal_ctrl_touched;
+
 RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session* sess, ArdourCanvas::Canvas& canvas)
 	: RouteUI(sess)
 	, StripableTimeAxisView(ed, sess, canvas)
@@ -118,13 +113,13 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session* sess, ArdourCan
 	, automation_action_menu (0)
 	, plugins_submenu_item (0)
 	, route_group_menu (0)
-	, playlist_action_menu (0)
 	, overlaid_menu_item (0)
 	, stacked_menu_item (0)
 	, gm (sess, true, 75, 14)
 	, _ignore_set_layer_display (false)
 	, pan_automation_item(NULL)
 {
+	subplugin_menu.set_name ("ArdourContextMenu");
 	number_label.set_name("tracknumber label");
 	number_label.set_elements((ArdourButton::Element)(ArdourButton::Edge|ArdourButton::Body|ArdourButton::Text|ArdourButton::Inactive));
 	number_label.set_alignment(.5, .5);
@@ -133,6 +128,10 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session* sess, ArdourCan
 	sess->config.ParameterChanged.connect (*this, invalidator (*this), boost::bind (&RouteTimeAxisView::parameter_changed, this, _1), gui_context());
 	UIConfiguration::instance().ParameterChanged.connect (sigc::mem_fun (*this, &RouteTimeAxisView::parameter_changed));
 
+	Controllable::ControlTouched.connect (
+			ctrl_touched_connection, invalidator (*this), boost::bind (&RouteTimeAxisView::show_touched_automation, this, _1), gui_context ()
+    );
+
 	parameter_changed ("editor-stereo-only-meters");
 }
 
@@ -170,19 +169,9 @@ RouteTimeAxisView::set_route (boost::shared_ptr<Route> rt)
 		set_height (preset_height (HeightNormal));
 	}
 
-	if (!_route->is_auditioner()) {
-		if (gui_property ("visible").empty()) {
-			set_gui_property ("visible", true);
-		}
-	} else {
-		set_gui_property ("visible", false);
-	}
-
 	timestretch_rect = 0;
 	no_redraw = false;
 
-	ignore_toggle = false;
-
 	route_group_button.set_name ("route button");
 	playlist_button.set_name ("route button");
 	automation_button.set_name ("route button");
@@ -348,7 +337,6 @@ RouteTimeAxisView::~RouteTimeAxisView ()
 		delete *i;
 	}
 
-	delete playlist_action_menu;
 	delete automation_action_menu;
 
 	delete _view;
@@ -387,8 +375,20 @@ RouteTimeAxisView::setup_processor_menu_and_curves ()
 {
 	_subplugin_menu_map.clear ();
 	subplugin_menu.items().clear ();
+	ctrl_item_map.clear ();
 	_route->foreach_processor (sigc::mem_fun (*this, &RouteTimeAxisView::add_processor_to_subplugin_menu));
 	_route->foreach_processor (sigc::mem_fun (*this, &RouteTimeAxisView::add_existing_processor_automation_curves));
+
+	/* update controllable LUT */
+	for (list<ProcessorAutomationInfo*>::iterator i = processor_automation.begin(); i != processor_automation.end(); ++i) {
+		if (!(*i)->valid) {
+			continue;
+		}
+		for (vector<ProcessorAutomationNode*>::iterator ii = (*i)->lines.begin(); ii != (*i)->lines.end(); ++ii) {
+			boost::shared_ptr<PBD::Controllable> c = boost::dynamic_pointer_cast <PBD::Controllable>((*i)->processor->control((*ii)->what));
+			ctrl_item_map[c] = (*ii)->menu_item;
+		}
+	}
 }
 
 bool
@@ -464,7 +464,7 @@ RouteTimeAxisView::update_track_number_visibility ()
 		}
 		// see ArdourButton::on_size_request(), we should probably use a global size-group here instead.
 		// except the width of the number label is subtracted from the name-hbox, so we
-		// need to explictly calculate it anyway until the name-label & entry become ArdourWidgets.
+		// need to explicitly calculate it anyway until the name-label & entry become ArdourWidgets.
 		int tnw = (2 + std::max(2u, _session->track_number_decimals())) * number_label.char_pixel_width();
 		if (tnw & 1) --tnw;
 		number_label.set_size_request(tnw, -1);
@@ -492,6 +492,8 @@ RouteTimeAxisView::parameter_changed (string const & p)
 		} else {
 			gm.get_level_meter().set_max_audio_meter_count (0);
 		}
+	} else if (p == "use-route-color-widely") {
+		route_color_changed ();
 	}
 }
 
@@ -620,12 +622,15 @@ RouteTimeAxisView::build_display_menu ()
 
 	/* prepare it */
 
+	if (automation_action_menu) {
+		detach_menu (*automation_action_menu);
+	}
+
 	TimeAxisView::build_display_menu ();
 
 	/* now fill it with our stuff */
 
 	MenuList& items = display_menu->items();
-	display_menu->set_name ("ArdourContextMenu");
 
 	items.push_back (MenuElem (_("Color..."), sigc::mem_fun (*this, &RouteUI::choose_color)));
 
@@ -637,9 +642,6 @@ RouteTimeAxisView::build_display_menu ()
 
 	items.push_back (SeparatorElem());
 
-	if (_size_menu) {
-		detach_menu (*_size_menu);
-	}
 	build_size_menu ();
 	items.push_back (MenuElem (_("Height"), *_size_menu));
 	items.push_back (SeparatorElem());
@@ -817,13 +819,17 @@ RouteTimeAxisView::build_display_menu ()
 
 	int active = 0;
 	int inactive = 0;
+	bool always_active = false;
 	TrackSelection const & s = _editor.get_selection().tracks;
 	for (TrackSelection::const_iterator i = s.begin(); i != s.end(); ++i) {
 		RouteTimeAxisView* r = dynamic_cast<RouteTimeAxisView*> (*i);
 		if (!r) {
 			continue;
 		}
-
+		always_active |= r->route()->is_master();
+#ifdef MIXBUS
+		always_active |= r->route()->mixbus() != 0;
+#endif
 		if (r->route()->active()) {
 			++active;
 		} else {
@@ -840,7 +846,7 @@ RouteTimeAxisView::build_display_menu ()
 	} else if (active > 0 && inactive > 0) {
 		i->set_inconsistent (true);
 	}
-	i->set_sensitive(! _session->transport_rolling());
+	i->set_sensitive(! _session->transport_rolling() && ! always_active);
 	i->signal_activate().connect (sigc::bind (sigc::mem_fun (*this, &RouteUI::set_route_active), click_sets_active, true));
 
 	items.push_back (SeparatorElem());
@@ -870,7 +876,7 @@ RouteTimeAxisView::layer_display_menu_change (Gtk::MenuItem* item)
 }
 
 void
-RouteTimeAxisView::show_timestretch (samplepos_t start, samplepos_t end, int layers, int layer)
+RouteTimeAxisView::show_timestretch (timepos_t const & start, timepos_t const & end, int layers, int layer)
 {
 	TimeAxisView::show_timestretch (start, end, layers, layer);
 
@@ -907,8 +913,13 @@ RouteTimeAxisView::show_timestretch (samplepos_t start, samplepos_t end, int lay
 	timestretch_rect->show ();
 	timestretch_rect->raise_to_top ();
 
-	double const x1 = start / _editor.get_current_zoom();
-	double const x2 = (end - 1) / _editor.get_current_zoom();
+	/* we use samples here since that is the canonical GUI<=>timeline
+	 * mapping (samples/pixels). This is just a dragging rect, it doesn't
+	 * by itself determine the parameters for the stretch.
+	 */
+
+	double const x1 = start.samples() / _editor.get_current_zoom();
+	double const x2 = (end.samples() - 1) / _editor.get_current_zoom();
 
 	timestretch_rect->set (ArdourCanvas::Rect (x1, current_height() * (layers - layer - 1) / layers,
 						   x2, current_height() * (layers - layer) / layers));
@@ -945,7 +956,7 @@ RouteTimeAxisView::show_selection (TimeSelection& ts)
 }
 
 void
-RouteTimeAxisView::set_height (uint32_t h, TrackHeightMode m)
+RouteTimeAxisView::set_height (uint32_t h, TrackHeightMode m, bool from_idle)
 {
 	int gmlen = h - 9;
 	bool height_changed = (height == 0) || (h != height);
@@ -956,7 +967,7 @@ RouteTimeAxisView::set_height (uint32_t h, TrackHeightMode m)
 	}
 	gm.get_level_meter().setup_meters (gmlen, meter_width);
 
-	TimeAxisView::set_height (h, m);
+	TimeAxisView::set_height (h, m, from_idle);
 
 	if (_view) {
 		_view->set_height ((double) current_height());
@@ -1020,6 +1031,12 @@ RouteTimeAxisView::route_color_changed ()
 		_view->apply_color (color(), StreamView::RegionColor);
 	}
 	number_label.set_fixed_colors (gdk_color_to_rgba (color()), gdk_color_to_rgba (color()));
+
+	if (!is_master() && UIConfiguration::instance().get_use_route_color_widely()) {
+		gm.set_fader_fg (gdk_color_to_rgba (route_color_tint ()));
+	} else {
+		gm.unset_fader_fg ();
+	}
 }
 
 void
@@ -1051,167 +1068,6 @@ RouteTimeAxisView::set_align_choice (RadioMenuItem* mitem, AlignChoice choice, b
 	}
 }
 
-void
-RouteTimeAxisView::rename_current_playlist ()
-{
-	Prompter prompter (true);
-	string name;
-
-	boost::shared_ptr<Track> tr = track();
-	if (!tr) {
-		return;
-	}
-
-	boost::shared_ptr<Playlist> pl = tr->playlist();
-	if (!pl) {
-		return;
-	}
-
-	prompter.set_title (_("Rename Playlist"));
-	prompter.set_prompt (_("New name for playlist:"));
-	prompter.add_button (_("Rename"), Gtk::RESPONSE_ACCEPT);
-	prompter.set_initial_text (pl->name());
-	prompter.set_response_sensitive (Gtk::RESPONSE_ACCEPT, false);
-
-	while (true) {
-		if (prompter.run () != Gtk::RESPONSE_ACCEPT) {
-			break;
-		}
-		prompter.get_result (name);
-		if (name.length()) {
-			if (_session->playlists()->by_name (name)) {
-				MessageDialog msg (_("Given playlist name is not unique."));
-				msg.run ();
-				prompter.set_initial_text (Playlist::bump_name (name, *_session));
-			} else {
-				pl->set_name (name);
-				break;
-			}
-		}
-	}
-}
-
-std::string
-RouteTimeAxisView::resolve_new_group_playlist_name(std::string &basename, vector<boost::shared_ptr<Playlist> > const & playlists)
-{
-	std::string ret (basename);
-
-	std::string const group_string = "." + route_group()->name() + ".";
-
-	// iterate through all playlists
-	int maxnumber = 0;
-	for (vector<boost::shared_ptr<Playlist> >::const_iterator i = playlists.begin(); i != playlists.end(); ++i) {
-		std::string tmp = (*i)->name();
-
-		std::string::size_type idx = tmp.find(group_string);
-		// find those which belong to this group
-		if (idx != string::npos) {
-			tmp = tmp.substr(idx + group_string.length());
-
-			// and find the largest current number
-			int x = atoi(tmp);
-			if (x > maxnumber) {
-				maxnumber = x;
-			}
-		}
-	}
-
-	maxnumber++;
-
-	char buf[32];
-	snprintf (buf, sizeof(buf), "%d", maxnumber);
-
-	ret = this->name() + "." + route_group()->name () + "." + buf;
-
-	return ret;
-}
-
-void
-RouteTimeAxisView::use_new_playlist (bool prompt, vector<boost::shared_ptr<Playlist> > const & playlists_before_op, bool copy)
-{
-	string name;
-
-	boost::shared_ptr<Track> tr = track ();
-	if (!tr) {
-		return;
-	}
-
-	boost::shared_ptr<const Playlist> pl = tr->playlist();
-	if (!pl) {
-		return;
-	}
-
-	name = pl->name();
-
-	if (route_group() && route_group()->is_active() && route_group()->enabled_property (ARDOUR::Properties::group_select.property_id)) {
-		name = resolve_new_group_playlist_name(name,playlists_before_op);
-	}
-
-	while (_session->playlists()->by_name(name)) {
-		name = Playlist::bump_name (name, *_session);
-	}
-
-	if (prompt) {
-		// TODO: The prompter "new" button should be de-activated if the user
-		// specifies a playlist name which already exists in the session.
-
-		Prompter prompter (true);
-
-		if (copy) {
-			prompter.set_title (_("New Copy Playlist"));
-			prompter.set_prompt (_("Name for playlist copy:"));
-		} else {
-			prompter.set_title (_("New Playlist"));
-			prompter.set_prompt (_("Name for new playlist:"));
-		}
-		prompter.set_initial_text (name);
-		prompter.add_button (Gtk::Stock::NEW, Gtk::RESPONSE_ACCEPT);
-		prompter.set_response_sensitive (Gtk::RESPONSE_ACCEPT, true);
-		prompter.show_all ();
-
-		while (true) {
-			if (prompter.run () != Gtk::RESPONSE_ACCEPT) {
-				return;
-			}
-			prompter.get_result (name);
-			if (name.length()) {
-				if (_session->playlists()->by_name (name)) {
-					MessageDialog msg (_("Given playlist name is not unique."));
-					msg.run ();
-					prompter.set_initial_text (Playlist::bump_name (name, *_session));
-				} else {
-					break;
-				}
-			}
-		}
-	}
-
-	if (name.length()) {
-		if (copy) {
-			tr->use_copy_playlist ();
-		} else {
-			tr->use_default_new_playlist ();
-		}
-		tr->playlist()->set_name (name);
-	}
-}
-
-void
-RouteTimeAxisView::clear_playlist ()
-{
-	boost::shared_ptr<Track> tr = track ();
-	if (!tr) {
-		return;
-	}
-
-	boost::shared_ptr<Playlist> pl = tr->playlist();
-	if (!pl) {
-		return;
-	}
-
-	_editor.clear_playlist (pl);
-}
-
 void
 RouteTimeAxisView::speed_changed ()
 {
@@ -1295,7 +1151,7 @@ RouteTimeAxisView::set_selected_regionviews (RegionSelection& regions)
  * @param results List to add things to.
  */
 void
-RouteTimeAxisView::get_selectables (samplepos_t start, samplepos_t end, double top, double bot, list<Selectable*>& results, bool within)
+RouteTimeAxisView::get_selectables (timepos_t const & start, timepos_t const & end, double top, double bot, list<Selectable*>& results, bool within)
 {
 	if ((_view && ((top < 0.0 && bot < 0.0))) || touched (top, bot)) {
 		_view->get_selectables (start, end, top, bot, results, within);
@@ -1318,6 +1174,16 @@ RouteTimeAxisView::get_inverted_selectables (Selection& sel, list<Selectable*>&
 	StripableTimeAxisView::get_inverted_selectables (sel, results);
 }
 
+void
+RouteTimeAxisView::get_regionviews_at_or_after (timepos_t const & pos, RegionSelection& regions)
+{
+	if (!_view) {
+		return;
+	}
+
+	_view->get_regionviews_at_or_after (pos, regions);
+}
+
 RouteGroup*
 RouteTimeAxisView::route_group () const
 {
@@ -1363,7 +1229,7 @@ RouteTimeAxisView::name_entry_changed (string const& str)
 }
 
 boost::shared_ptr<Region>
-RouteTimeAxisView::find_next_region (samplepos_t pos, RegionPoint point, int32_t dir)
+RouteTimeAxisView::find_next_region (timepos_t const & pos, RegionPoint point, int32_t dir)
 {
 	boost::shared_ptr<Playlist> pl = playlist ();
 
@@ -1374,8 +1240,8 @@ RouteTimeAxisView::find_next_region (samplepos_t pos, RegionPoint point, int32_t
 	return boost::shared_ptr<Region> ();
 }
 
-samplepos_t
-RouteTimeAxisView::find_next_region_boundary (samplepos_t pos, int32_t dir)
+timepos_t
+RouteTimeAxisView::find_next_region_boundary (timepos_t const & pos, int32_t dir)
 {
 	boost::shared_ptr<Playlist> pl = playlist ();
 
@@ -1383,7 +1249,7 @@ RouteTimeAxisView::find_next_region_boundary (samplepos_t pos, int32_t dir)
 		return pl->find_next_region_boundary (pos, dir);
 	}
 
-	return -1;
+	return timepos_t::max (pos.time_domain());
 }
 
 void
@@ -1436,32 +1302,20 @@ RouteTimeAxisView::cut_copy_clear (Selection& selection, CutCopyOp op)
 	switch (op) {
 	case Delete:
 		if (playlist->cut (time) != 0) {
-			if (Config->get_edit_mode() == Ripple) {
-				playlist->ripple(time.start(), -time.length(), NULL);
+			if (_editor.should_ripple()) {
+				playlist->ripple (time.start_time(), -time.length(), NULL);
 			}
-			// no need to exclude any regions from rippling here
-
-			vector<Command*> cmds;
-			playlist->rdiff (cmds);
-			_session->add_commands (cmds);
-
-			_session->add_command (new StatefulDiffCommand (playlist));
+			playlist->rdiff_and_add_command (_session);
 		}
 		break;
 
 	case Cut:
 		if ((what_we_got = playlist->cut (time)) != 0) {
 			_editor.get_cut_buffer().add (what_we_got);
-			if (Config->get_edit_mode() == Ripple) {
-				playlist->ripple(time.start(), -time.length(), NULL);
+			if (_editor.should_ripple()) {
+				playlist->ripple (time.start_time(), -time.length(), NULL);
 			}
-			// no need to exclude any regions from rippling here
-
-			vector<Command*> cmds;
-			playlist->rdiff (cmds);
-			_session->add_commands (cmds);
-
-			_session->add_command (new StatefulDiffCommand (playlist));
+			playlist->rdiff_and_add_command (_session);
 		}
 		break;
 	case Copy:
@@ -1472,15 +1326,10 @@ RouteTimeAxisView::cut_copy_clear (Selection& selection, CutCopyOp op)
 
 	case Clear:
 		if ((what_we_got = playlist->cut (time)) != 0) {
-			if (Config->get_edit_mode() == Ripple) {
-				playlist->ripple(time.start(), -time.length(), NULL);
+			if (_editor.should_ripple()) {
+				playlist->ripple (time.start_time(), -time.length(), NULL);
 			}
-			// no need to exclude any regions from rippling here
-
-			vector<Command*> cmds;
-			playlist->rdiff (cmds);
-			_session->add_commands (cmds);
-			_session->add_command (new StatefulDiffCommand (playlist));
+			playlist->rdiff_and_add_command (_session);
 			what_we_got->release ();
 		}
 		break;
@@ -1488,7 +1337,7 @@ RouteTimeAxisView::cut_copy_clear (Selection& selection, CutCopyOp op)
 }
 
 bool
-RouteTimeAxisView::paste (samplepos_t pos, const Selection& selection, PasteContext& ctx, const int32_t sub_num)
+RouteTimeAxisView::paste (timepos_t const & pos, const Selection& selection, PasteContext& ctx)
 {
 	if (!is_track()) {
 		return false;
@@ -1506,18 +1355,20 @@ RouteTimeAxisView::paste (samplepos_t pos, const Selection& selection, PasteCont
 	DEBUG_TRACE (DEBUG::CutNPaste, string_compose ("paste to %1\n", pos));
 
 	/* add multi-paste offset if applicable */
-	std::pair<samplepos_t, samplepos_t> extent  = (*p)->get_extent();
-	const samplecnt_t                  duration = extent.second - extent.first;
-	pos += _editor.get_paste_offset(pos, ctx.count, duration);
+	std::pair<timepos_t, timepos_t> extent  = (*p)->get_extent();
+	const timecnt_t                 duration = extent.first.distance (extent.second);
+
+	timepos_t ppos = pos;
+	ppos += _editor.get_paste_offset (ppos, ctx.count, duration);
 
 	pl->clear_changes ();
 	pl->clear_owned_changes ();
-	if (Config->get_edit_mode() == Ripple) {
-		std::pair<samplepos_t, samplepos_t> extent = (*p)->get_extent_with_endspace();
-		samplecnt_t amount = extent.second - extent.first;
-		pl->ripple(pos, amount * ctx.times, boost::shared_ptr<Region>());
+	if (_editor.should_ripple()) {
+		std::pair<timepos_t, timepos_t> extent = (*p)->get_extent_with_endspace();
+		timecnt_t amount = extent.first.distance (extent.second);
+		pl->ripple (ppos, amount.scale (ctx.times), boost::shared_ptr<Region>());
 	}
-	pl->paste (*p, pos, ctx.times, sub_num);
+	pl->paste (*p, ppos, ctx.times);
 
 	vector<Command*> cmds;
 	pl->rdiff (cmds);
@@ -1528,140 +1379,6 @@ RouteTimeAxisView::paste (samplepos_t pos, const Selection& selection, PasteCont
 	return true;
 }
 
-
-struct PlaylistSorter {
-	bool operator() (boost::shared_ptr<Playlist> a, boost::shared_ptr<Playlist> b) const {
-		return a->sort_id() < b->sort_id();
-	}
-};
-
-void
-RouteTimeAxisView::build_playlist_menu ()
-{
-	using namespace Menu_Helpers;
-
-	if (!is_track()) {
-		return;
-	}
-
-	delete playlist_action_menu;
-	playlist_action_menu = new Menu;
-	playlist_action_menu->set_name ("ArdourContextMenu");
-
-	MenuList& playlist_items = playlist_action_menu->items();
-	playlist_action_menu->set_name ("ArdourContextMenu");
-	playlist_items.clear();
-
-	RadioMenuItem::Group playlist_group;
-	boost::shared_ptr<Track> tr = track ();
-
-	vector<boost::shared_ptr<Playlist> > playlists_tr = _session->playlists()->playlists_for_track (tr);
-
-	/* sort the playlists */
-	PlaylistSorter cmp;
-	sort (playlists_tr.begin(), playlists_tr.end(), cmp);
-
-	/* add the playlists to the menu */
-	for (vector<boost::shared_ptr<Playlist> >::iterator i = playlists_tr.begin(); i != playlists_tr.end(); ++i) {
-		playlist_items.push_back (RadioMenuElem (playlist_group, (*i)->name()));
-		RadioMenuItem *item = static_cast<RadioMenuItem*>(&playlist_items.back());
-		item->signal_toggled().connect(sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::use_playlist), item, boost::weak_ptr<Playlist> (*i)));
-
-		if (tr->playlist()->id() == (*i)->id()) {
-			item->set_active();
-		}
-	}
-
-	playlist_items.push_back (SeparatorElem());
-	playlist_items.push_back (MenuElem (_("Rename..."), sigc::mem_fun(*this, &RouteTimeAxisView::rename_current_playlist)));
-	playlist_items.push_back (SeparatorElem());
-
-	if (!route_group() || !route_group()->is_active() || !route_group()->enabled_property (ARDOUR::Properties::group_select.property_id)) {
-		playlist_items.push_back (MenuElem (_("New..."), sigc::bind(sigc::mem_fun(_editor, &PublicEditor::new_playlists), this)));
-		playlist_items.push_back (MenuElem (_("New Copy..."), sigc::bind(sigc::mem_fun(_editor, &PublicEditor::copy_playlists), this)));
-
-	} else {
-		// Use a label which tells the user what is happening
-		playlist_items.push_back (MenuElem (_("New Take"), sigc::bind(sigc::mem_fun(_editor, &PublicEditor::new_playlists), this)));
-		playlist_items.push_back (MenuElem (_("Copy Take"), sigc::bind(sigc::mem_fun(_editor, &PublicEditor::copy_playlists), this)));
-
-	}
-
-	playlist_items.push_back (SeparatorElem());
-	playlist_items.push_back (MenuElem (_("Clear Current"), sigc::bind(sigc::mem_fun(_editor, &PublicEditor::clear_playlists), this)));
-	playlist_items.push_back (SeparatorElem());
-
-	playlist_items.push_back (MenuElem(_("Select from All..."), sigc::mem_fun(*this, &RouteTimeAxisView::show_playlist_selector)));
-}
-
-void
-RouteTimeAxisView::use_playlist (RadioMenuItem *item, boost::weak_ptr<Playlist> wpl)
-{
-	assert (is_track());
-
-	// exit if we were triggered by deactivating the old playlist
-	if (!item->get_active()) {
-		return;
-	}
-
-	boost::shared_ptr<Playlist> pl (wpl.lock());
-
-	if (!pl) {
-		return;
-	}
-
-	if (track()->playlist() == pl) {
-		// exit when use_playlist is called by the creation of the playlist menu
-		// or the playlist choice is unchanged
-		return;
-	}
-
-	track()->use_playlist (track()->data_type(), pl);
-
-	RouteGroup* rg = route_group();
-
-	if (rg && rg->is_active() && rg->enabled_property (ARDOUR::Properties::group_select.property_id)) {
-		std::string group_string = "." + rg->name() + ".";
-
-		std::string take_name = pl->name();
-		std::string::size_type idx = take_name.find(group_string);
-
-		if (idx == std::string::npos)
-			return;
-
-		take_name = take_name.substr(idx + group_string.length()); // find the bit containing the take number / name
-
-		boost::shared_ptr<RouteList> rl (rg->route_list());
-
-		for (RouteList::const_iterator i = rl->begin(); i != rl->end(); ++i) {
-			if ((*i) == this->route()) {
-				continue;
-			}
-
-			std::string playlist_name = (*i)->name()+group_string+take_name;
-
-			boost::shared_ptr<Track> track = boost::dynamic_pointer_cast<Track>(*i);
-			if (!track) {
-				continue;
-			}
-
-			if (track->freeze_state() == Track::Frozen) {
-				/* Don't change playlists of frozen tracks */
-				continue;
-			}
-
-			boost::shared_ptr<Playlist> ipl = session()->playlists()->by_name(playlist_name);
-			if (!ipl) {
-				// No playlist for this track for this take yet, make it
-				track->use_default_new_playlist();
-				track->playlist()->set_name(playlist_name);
-			} else {
-				track->use_playlist(track->data_type(), ipl);
-			}
-		}
-	}
-}
-
 void
 RouteTimeAxisView::update_playlist_tip ()
 {
@@ -1692,13 +1409,6 @@ RouteTimeAxisView::update_playlist_tip ()
 	set_tooltip (playlist_button, _("Playlist") + std::string(": ") + Gtkmm2ext::markup_escape_text (track()->playlist()->name()));
 }
 
-
-void
-RouteTimeAxisView::show_playlist_selector ()
-{
-	_editor.playlist_selector().show_for (this);
-}
-
 void
 RouteTimeAxisView::map_frozen ()
 {
@@ -1893,6 +1603,76 @@ RouteTimeAxisView::show_existing_automation (bool apply_to_selection)
 	}
 }
 
+void
+RouteTimeAxisView::maybe_hide_automation (bool hide, boost::weak_ptr<PBD::Controllable> wctrl)
+{
+	ctrl_autohide_connection.disconnect ();
+	if (!hide) {
+		/* disconnect only, leave lane visible */
+		return;
+	}
+	boost::shared_ptr<AutomationControl> ac = boost::dynamic_pointer_cast<AutomationControl> (wctrl.lock ());
+  if (!ac) {
+		return;
+	}
+
+	Gtk::CheckMenuItem* cmi = find_menu_item_by_ctrl (ac);
+	if (cmi) {
+		cmi->set_active (false);
+		return;
+	}
+
+	boost::shared_ptr<AutomationTimeAxisView> atav = find_atav_by_ctrl (ac);
+	if (atav) {
+		atav->set_marked_for_display (false);
+		request_redraw ();
+	}
+}
+
+void
+RouteTimeAxisView::show_touched_automation (boost::weak_ptr<PBD::Controllable> wctrl)
+{
+	boost::shared_ptr<AutomationControl> ac = boost::dynamic_pointer_cast<AutomationControl> (wctrl.lock ());
+	if (!ac) {
+		return;
+	}
+
+	if (!_editor.show_touched_automation ()) {
+		if (ctrl_autohide_connection.connected ()) {
+			signal_ctrl_touched (true);
+		}
+		return;
+	}
+
+	boost::shared_ptr<AutomationTimeAxisView> atav;
+	Gtk::CheckMenuItem* cmi = find_menu_item_by_ctrl (ac);
+	if (!cmi) {
+		atav = find_atav_by_ctrl (ac);
+		if (!atav) {
+			return;
+		}
+	}
+
+	/* hide any lanes */
+	signal_ctrl_touched (true);
+
+	if (cmi && !cmi->get_active ()) {
+		cmi->set_active (true);
+		ctrl_autohide_connection = signal_ctrl_touched.connect (sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::maybe_hide_automation), wctrl));
+		/* search ctrl to scroll to */
+		atav = find_atav_by_ctrl (ac, false);
+	} else if (atav && ! string_to<bool>(atav->gui_property ("visible"))) {
+		atav->set_marked_for_display (true);
+		ctrl_autohide_connection = signal_ctrl_touched.connect (sigc::bind (sigc::mem_fun (*this, &RouteTimeAxisView::maybe_hide_automation), wctrl));
+		request_redraw ();
+	}
+
+	if (atav) {
+		_editor.ensure_time_axis_view_is_visible (*atav, false);
+		return;
+	}
+}
+
 void
 RouteTimeAxisView::hide_all_automation (bool apply_to_selection)
 {
@@ -1970,6 +1750,53 @@ RouteTimeAxisView::find_processor_automation_node (boost::shared_ptr<Processor>
 	return 0;
 }
 
+Gtk::CheckMenuItem*
+RouteTimeAxisView::find_menu_item_by_ctrl (boost::shared_ptr<AutomationControl> ac)
+{
+	std::map<boost::shared_ptr<PBD::Controllable>, Gtk::CheckMenuItem*>::const_iterator i;
+	i = ctrl_item_map.find (ac);
+	if (i != ctrl_item_map.end ()) {
+		return i->second;
+	}
+	return 0;
+}
+
+boost::shared_ptr<AutomationTimeAxisView>
+RouteTimeAxisView::find_atav_by_ctrl (boost::shared_ptr<ARDOUR::AutomationControl> ac, bool route_owned_only)
+{
+	if (gain_track && gain_track->control () == ac) {
+		return gain_track;
+	}
+	else if (trim_track && trim_track->control () == ac) {
+		return trim_track;
+	}
+	else if (mute_track && mute_track->control () == ac) {
+		return mute_track;
+	}
+
+	if (!pan_tracks.empty() && !ARDOUR::Profile->get_mixbus()) {
+		// XXX this can lead to inconsistent CheckMenuItem state (azimuth, width are treated separately)
+		for (list<boost::shared_ptr<AutomationTimeAxisView> >::iterator i = pan_tracks.begin(); i != pan_tracks.end(); ++i) {
+			if ((*i)->control () == ac) {
+				return *i;
+			}
+		}
+	}
+
+	if (route_owned_only) {
+		return boost::shared_ptr<AutomationTimeAxisView> ();
+	}
+
+	for (Children::iterator j = children.begin(); j != children.end(); ++j) {
+		boost::shared_ptr<AutomationTimeAxisView> atv = boost::dynamic_pointer_cast<AutomationTimeAxisView> (*j);
+		if (atv && atv->control () == ac) {
+			return atv;
+		}
+	}
+	return boost::shared_ptr<AutomationTimeAxisView> ();
+}
+
+
 /** Add an AutomationTimeAxisView to display automation for a processor's parameter */
 void
 RouteTimeAxisView::add_processor_automation_curve (boost::shared_ptr<Processor> processor, Evoral::Parameter what)
@@ -2117,19 +1944,57 @@ RouteTimeAxisView::add_processor_to_subplugin_menu (boost::weak_ptr<Processor> p
 
 	items.clear ();
 
+	size_t total_ctrls = 0;
+	for (std::set<Evoral::Parameter>::const_iterator i = automatable.begin(); i != automatable.end(); ++i) {
+		string const& name = processor->describe_parameter (*i);
+		if (name == X_("hidden")) {
+			continue;
+		}
+		++total_ctrls;
+	}
+
+	const int max_items   = 32; // 32 per submenu, next menu begins with 33nd at the top
+	unsigned  n_items     = 0;
+	unsigned  n_groups    = 1;
+	bool      use_submenu = total_ctrls > max_items + 5; // allow for some slack
+	Menu*     menu        = NULL;
+
+	if (use_submenu) {
+		menu = manage (new Menu);
+		menu->set_name ("ArdourContextMenu");
+		items.push_back (MenuElem (string_compose (_("Parameters %1 - %2"), 1, max_items), *menu));
+	} else {
+		menu = rai->menu;
+	}
+
 	for (std::set<Evoral::Parameter>::const_iterator i = automatable.begin(); i != automatable.end(); ++i) {
 
 		ProcessorAutomationNode* pan;
 		Gtk::CheckMenuItem* mitem;
 
-		string name = processor->describe_parameter (*i);
+		string const& name = processor->describe_parameter (*i);
 
 		if (name == X_("hidden")) {
 			continue;
 		}
 
-		items.push_back (CheckMenuElem (name));
-		mitem = dynamic_cast<Gtk::CheckMenuItem*> (&items.back());
+		if (use_submenu && ++n_items > max_items) {
+			n_items = 1;
+			menu = manage (new Menu);
+			menu->set_name ("ArdourContextMenu");
+			size_t start = n_groups * max_items + 1;
+			size_t end   = ++n_groups * max_items;
+			/* at least 2 items per sub-menu */
+			if (end + 1 >= total_ctrls) {
+				end = total_ctrls;
+				use_submenu = false;
+			}
+			items.push_back (MenuElem (string_compose (_("Parameters %1 - %2"), start, end), *menu));
+		}
+
+		MenuList& mitems = menu->items();
+		mitems.push_back (CheckMenuElem (name));
+		mitem = dynamic_cast<Gtk::CheckMenuItem*> (&mitems.back());
 
 		_subplugin_menu_map[*i] = mitem;
 
@@ -2150,7 +2015,7 @@ RouteTimeAxisView::add_processor_to_subplugin_menu (boost::weak_ptr<Processor> p
 		boost::shared_ptr<AutomationTimeAxisView> atav = pan->view;
 		bool visible;
 		if (atav && atav->get_gui_property ("visible", visible)) {
-			mitem->set_active(true);
+			mitem->set_active(visible);
 		} else {
 			mitem->set_active(false);
 		}
@@ -2184,7 +2049,8 @@ RouteTimeAxisView::processor_menu_item_toggled (RouteTimeAxisView::ProcessorAuto
 		redraw = true;
 	}
 
-	if (pan->view && pan->view->set_marked_for_display (showit)) {
+	boost::shared_ptr<AutomationTimeAxisView> atav = pan->view;
+	if (atav && atav->set_marked_for_display (showit)) {
 		redraw = true;
 	}
 
@@ -2414,7 +2280,7 @@ void
 RouteTimeAxisView::io_changed (IOChange /*change*/, void */*src*/)
 {
 	reset_meter ();
-	if (_route && !no_redraw) {
+	if (_route && !no_redraw && !_session->routes_deletion_in_progress ()) {
 		request_redraw ();
 	}
 }
@@ -2670,13 +2536,6 @@ void add_region_to_list (RegionView* rv, RegionList* l)
 RegionView*
 RouteTimeAxisView::combine_regions ()
 {
-	/* as of may 2011, we do not offer uncombine for MIDI tracks
-	 */
-
-	if (!is_audio_track()) {
-		return 0;
-	}
-
 	if (!_view) {
 		return 0;
 	}
