@@ -21,6 +21,7 @@
  */
 
 #define BASELINESTRETCH (1.25)
+#define MARKER_SIZE (0.5) // * height
 
 #include <algorithm>
 
@@ -31,6 +32,7 @@
 #include "ardour/ardour.h"
 #include "ardour/audioengine.h"
 #include "ardour/rc_configuration.h"
+#include "ardour/port.h"
 #include "ardour/session.h"
 
 #include "gtkmm2ext/colors.h"
@@ -161,7 +163,7 @@ ShuttleControl::ShuttleControl ()
 	_hovering             = false;
 	_ignore_change        = false;
 
-	set_flags (CAN_FOCUS);
+	set_can_focus ();
 	add_events (Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON_PRESS_MASK | Gdk::POINTER_MOTION_MASK | Gdk::SCROLL_MASK);
 	set_name (X_("ShuttleControl"));
 
@@ -209,7 +211,7 @@ ShuttleControl::~ShuttleControl ()
 void
 ShuttleControl::varispeed_button_clicked ()
 {
-	if (_session->default_play_speed () == 1.0 && !_vari_dialog.is_visible ()) {
+	if (_session->default_play_speed () == 1.0 && !_vari_dialog.get_visible ()) {
 		_vari_dialog.present ();
 	} else {
 		_vari_dialog.hide ();
@@ -325,12 +327,9 @@ ShuttleControl::map_transport_state ()
 		shuttle_fract = 0;
 	} else {
 		if (Config->get_shuttle_units () == Semitones) {
-			bool reverse;
-			int  semi     = speed_as_semitones (speed, reverse);
-			semi          = std::max (-24, std::min (24, semi));
-			shuttle_fract = semitones_as_fract (semi, reverse);
-		} else {
 			shuttle_fract = speed / shuttle_max_speed;
+		} else {
+			shuttle_fract = speed_as_fract (speed);
 		}
 	}
 
@@ -347,40 +346,57 @@ ShuttleControl::build_shuttle_context_menu ()
 	shuttle_context_menu = new Menu ();
 	MenuList& items      = shuttle_context_menu->items ();
 
-	{
-		RadioMenuItem::Group speed_group;
+	float max_transport_speed = Config->get_max_transport_speed ();
 
-		/* XXX this code assumes that Config->get_max_transport_speed() returns 8 */
-		Menu*     speed_menu  = manage (new Menu ());
-		MenuList& speed_items = speed_menu->items ();
+	RadioMenuItem::Group speed_group;
 
+	Menu*     speed_menu  = manage (new Menu ());
+	MenuList& speed_items = speed_menu->items ();
+
+	if (max_transport_speed >= 8) {
 		speed_items.push_back (RadioMenuElem (speed_group, "8", sigc::bind (sigc::mem_fun (*this, &ShuttleControl::set_shuttle_max_speed), 8.0f)));
 		if (shuttle_max_speed == 8.0) {
 			static_cast<RadioMenuItem*> (&speed_items.back ())->set_active ();
 		}
+	}
+	if (max_transport_speed >= 6) {
 		speed_items.push_back (RadioMenuElem (speed_group, "6", sigc::bind (sigc::mem_fun (*this, &ShuttleControl::set_shuttle_max_speed), 6.0f)));
 		if (shuttle_max_speed == 6.0) {
 			static_cast<RadioMenuItem*> (&speed_items.back ())->set_active ();
 		}
+	}
+	if (max_transport_speed >= 4) {
 		speed_items.push_back (RadioMenuElem (speed_group, "4", sigc::bind (sigc::mem_fun (*this, &ShuttleControl::set_shuttle_max_speed), 4.0f)));
 		if (shuttle_max_speed == 4.0) {
 			static_cast<RadioMenuItem*> (&speed_items.back ())->set_active ();
 		}
+	}
+	if (max_transport_speed >= 3) {
 		speed_items.push_back (RadioMenuElem (speed_group, "3", sigc::bind (sigc::mem_fun (*this, &ShuttleControl::set_shuttle_max_speed), 3.0f)));
 		if (shuttle_max_speed == 3.0) {
 			static_cast<RadioMenuItem*> (&speed_items.back ())->set_active ();
 		}
+	}
+	if (max_transport_speed >= 2) {
 		speed_items.push_back (RadioMenuElem (speed_group, "2", sigc::bind (sigc::mem_fun (*this, &ShuttleControl::set_shuttle_max_speed), 2.0f)));
 		if (shuttle_max_speed == 2.0) {
 			static_cast<RadioMenuItem*> (&speed_items.back ())->set_active ();
 		}
+	}
+	if (max_transport_speed >= 1.5) {
 		speed_items.push_back (RadioMenuElem (speed_group, "1.5", sigc::bind (sigc::mem_fun (*this, &ShuttleControl::set_shuttle_max_speed), 1.5f)));
 		if (shuttle_max_speed == 1.5) {
 			static_cast<RadioMenuItem*> (&speed_items.back ())->set_active ();
 		}
-
-		items.push_back (MenuElem (_("Maximum speed"), *speed_menu));
+	} else {
+		assert (max_transport_speed >= 1);
+		speed_items.push_back (RadioMenuElem (speed_group, "1", sigc::bind (sigc::mem_fun (*this, &ShuttleControl::set_shuttle_max_speed), 1.f)));
+		if (shuttle_max_speed == 1) {
+			static_cast<RadioMenuItem*> (&speed_items.back ())->set_active ();
+		}
 	}
+
+	items.push_back (MenuElem (_("Maximum speed"), *speed_menu));
 }
 
 void
@@ -389,6 +405,9 @@ ShuttleControl::set_shuttle_max_speed (float speed)
 	if (_ignore_change) {
 		return;
 	}
+
+	assert (speed <= Config->get_max_transport_speed ());
+
 	Config->set_shuttle_max_speed (speed);
 }
 
@@ -501,7 +520,10 @@ ShuttleControl::mouse_shuttle (double x, bool force)
 	   center, negative values indicate left of center
 	*/
 
-	shuttle_fract = distance_from_center / center; // center == half the width
+	float marker_size = round (get_height () * MARKER_SIZE);
+	float avail_width = get_width () - marker_size;
+
+	shuttle_fract = 2 * distance_from_center / avail_width;
 	use_shuttle_fract (force);
 	return true;
 }
@@ -542,39 +564,26 @@ ShuttleControl::speed_as_cents (float speed, bool& reverse)
 }
 
 float
-ShuttleControl::cents_as_speed (int cents, bool reverse)
+ShuttleControl::speed_as_fract (float speed) const
 {
-	if (reverse) {
-		return -pow (2.0, (cents / 1200.0));
+	float fract = speed / shuttle_max_speed;
+	if (fract < 0) {
+		return -sqrtf (-fract);
 	} else {
-		return pow (2.0, (cents / 1200.0));
+		return sqrtf (fract);
 	}
 }
 
 float
-ShuttleControl::semitones_as_speed (int semi, bool reverse)
+ShuttleControl::fract_as_speed (float fract) const
 {
-	if (reverse) {
-		return -pow (2.0, (semi / 12.0));
+	if (fract < 0) {
+		return -fract * fract * shuttle_max_speed;
 	} else {
-		return pow (2.0, (semi / 12.0));
+		return fract * fract * shuttle_max_speed;
 	}
 }
 
-float
-ShuttleControl::semitones_as_fract (int semi, bool reverse)
-{
-	float speed = semitones_as_speed (semi, reverse);
-	return speed / 4.0; /* 4.0 is the maximum speed for a 24 semitone shift */
-}
-
-int
-ShuttleControl::fract_as_semitones (float fract, bool& reverse)
-{
-	assert (fract != 0.0);
-	return speed_as_semitones (fract * 4.0, reverse);
-}
-
 void
 ShuttleControl::use_shuttle_fract (bool force, bool zero_ok)
 {
@@ -596,16 +605,9 @@ ShuttleControl::use_shuttle_fract (bool force, bool zero_ok)
 	double speed = 0;
 
 	if (Config->get_shuttle_units () == Semitones) {
-		if (shuttle_fract != 0.0) {
-			bool reverse;
-			int  semi = fract_as_semitones (shuttle_fract, reverse);
-			speed     = semitones_as_speed (semi, reverse);
-		} else {
-			speed = 0.0;
-		}
+		speed = shuttle_fract * shuttle_max_speed;
 	} else {
-		shuttle_fract = shuttle_fract * shuttle_fract * shuttle_fract; // ^3 preserves the sign;
-		speed         = shuttle_max_speed * shuttle_fract;
+		speed = fract_as_speed (shuttle_fract);
 	}
 
 	requested_speed = speed;
@@ -671,8 +673,9 @@ ShuttleControl::render (Cairo::RefPtr<Cairo::Context> const& ctx, cairo_rectangl
 	}
 
 	/* marker */
-	float visual_fraction = std::max (-1.0f, std::min (1.0f, speed / shuttle_max_speed));
-	float marker_size     = round (get_height () * 0.66);
+	float visual_fraction = std::max (-1.0f, std::min (1.0f, shuttle_fract));
+
+	float marker_size     = round (get_height () * MARKER_SIZE);
 	float avail_width     = get_width () - marker_size;
 	float x               = 0.5 * (get_width () + visual_fraction * avail_width - marker_size);
 
@@ -765,7 +768,7 @@ ShuttleControl::parameter_changed (std::string p)
 		delete shuttle_context_menu;
 		shuttle_context_menu = 0;
 	} else if (p == "external-sync") {
-		if (_session->config.get_external_sync()) {
+		if (_session->config.get_external_sync() || !Port::can_varispeed ()) {
 			_vari_dialog.hide ();
 			_vari_button.set_sensitive (false);
 			if (shuttle_grabbed) {
