@@ -105,13 +105,15 @@ struct TransportFSM
 
 		static void init_pool ();
 
-          private:
-		static Pool* pool;
+	private:
+		static PBD::Pool* pool;
 
 	};
 
 	TransportFSM (TransportAPI& tapi);
 
+	void hard_stop ();
+
 	void start () {
 		init ();
 	}
@@ -186,7 +188,7 @@ struct TransportFSM
 	bool forwards() const             { return _direction_state == Forwards; }
 	bool backwards() const             { return _direction_state == Backwards; }
 	bool reversing() const             { return _direction_state == Reversing; }
-	bool will_roll_fowards() const;
+	bool will_roll_forwards() const;
 
 	void enqueue (Event* ev);
 
