@@ -27,8 +27,8 @@
 
 #include <glibmm/threads.h>
 
-#include "lv2.h"
-#include "lv2/lv2plug.in/ns/ext/urid/urid.h"
+#include <lv2/lv2plug.in/ns/lv2core/lv2.h>
+#include <lv2/lv2plug.in/ns/ext/urid/urid.h>
 
 #include "ardour/libardour_visibility.h"
 
