@@ -28,8 +28,9 @@
 #include <sstream>
 #include <algorithm>
 
+#include "temporal/types_convert.h"
+
 #include "ardour/automation_list.h"
-#include "ardour/beats_samples_converter.h"
 #include "ardour/event_type_map.h"
 #include "ardour/parameter_descriptor.h"
 #include "ardour/parameter_types.h"
@@ -60,8 +61,8 @@ static void dumpit (const AutomationList& al, string prefix = "")
 	cerr << "\n";
 }
 #endif
-AutomationList::AutomationList (const Evoral::Parameter& id, const Evoral::ParameterDescriptor& desc)
-	: ControlList(id, desc)
+AutomationList::AutomationList (const Evoral::Parameter& id, const Evoral::ParameterDescriptor& desc, Temporal::TimeDomain time_domain)
+	: ControlList(id, desc, time_domain)
 	, _before (0)
 {
 	_state = Off;
@@ -74,8 +75,8 @@ AutomationList::AutomationList (const Evoral::Parameter& id, const Evoral::Param
 	AutomationListCreated(this);
 }
 
-AutomationList::AutomationList (const Evoral::Parameter& id)
-	: ControlList(id, ARDOUR::ParameterDescriptor(id))
+AutomationList::AutomationList (const Evoral::Parameter& id, Temporal::TimeDomain time_domain)
+	: ControlList(id, ARDOUR::ParameterDescriptor(id), time_domain)
 	, _before (0)
 {
 	_state = Off;
@@ -102,7 +103,7 @@ AutomationList::AutomationList (const AutomationList& other)
 	AutomationListCreated(this);
 }
 
-AutomationList::AutomationList (const AutomationList& other, double start, double end)
+AutomationList::AutomationList (const AutomationList& other, timepos_t const & start, timepos_t const & end)
 	: ControlList(other, start, end)
 	, _before (0)
 {
@@ -119,7 +120,7 @@ AutomationList::AutomationList (const AutomationList& other, double start, doubl
  * in or below the AutomationList node.  It is used if @param id is non-null.
  */
 AutomationList::AutomationList (const XMLNode& node, Evoral::Parameter id)
-	: ControlList(id, ARDOUR::ParameterDescriptor(id))
+	: ControlList(id, ARDOUR::ParameterDescriptor(id), Temporal::AudioTime) /* domain may change in ::set_state */
 	, _before (0)
 {
 	g_atomic_int_set (&_touching, 0);
@@ -145,9 +146,10 @@ AutomationList::~AutomationList()
 
 boost::shared_ptr<Evoral::ControlList>
 AutomationList::create(const Evoral::Parameter&           id,
-                       const Evoral::ParameterDescriptor& desc)
+                       const Evoral::ParameterDescriptor& desc,
+                       Temporal::TimeDomain time_domain)
 {
-	return boost::shared_ptr<Evoral::ControlList>(new AutomationList(id, desc));
+	return boost::shared_ptr<Evoral::ControlList>(new AutomationList(id, desc, time_domain));
 }
 
 void
@@ -245,20 +247,20 @@ AutomationList::default_interpolation () const
 }
 
 void
-AutomationList::start_write_pass (double when)
+AutomationList::start_write_pass (timepos_t const & when)
 {
 	snapshot_history (true);
 	ControlList::start_write_pass (when);
 }
 
 void
-AutomationList::write_pass_finished (double when, double thinning_factor)
+AutomationList::write_pass_finished (timepos_t const & when, double thinning_factor)
 {
 	ControlList::write_pass_finished (when, thinning_factor);
 }
 
 void
-AutomationList::start_touch (double when)
+AutomationList::start_touch (timepos_t const & when)
 {
 	if (_state == Touch) {
 		start_write_pass (when);
@@ -268,7 +270,7 @@ AutomationList::start_touch (double when)
 }
 
 void
-AutomationList::stop_touch (double)
+AutomationList::stop_touch (timepos_t const & /* not used */)
 {
 	if (g_atomic_int_get (&_touching) == 0) {
 		/* this touch has already been stopped (probably by Automatable::transport_stopped),
@@ -314,31 +316,6 @@ AutomationList::thaw ()
 	}
 }
 
-bool
-AutomationList::paste (const ControlList& alist, double pos, DoubleBeatsSamplesConverter const& bfc)
-{
-	AutomationType src_type = (AutomationType)alist.parameter().type();
-	AutomationType dst_type = (AutomationType)_parameter.type();
-
-	if (parameter_is_midi (src_type) == parameter_is_midi (dst_type)) {
-		return ControlList::paste (alist, pos);
-	}
-	bool to_sample = parameter_is_midi (src_type);
-
-	ControlList cl (alist);
-	cl.clear ();
-	for (const_iterator i = alist.begin ();i != alist.end (); ++i) {
-		double when = (*i)->when;
-		if (to_sample) {
-			when = bfc.to ((*i)->when);
-		} else {
-			when = bfc.from ((*i)->when);
-		}
-		cl.fast_simple_add (when, (*i)->value);
-	}
-	return ControlList::paste (cl, pos);
-}
-
 Command*
 AutomationList::memento_command (XMLNode* before, XMLNode* after)
 {
@@ -346,19 +323,20 @@ AutomationList::memento_command (XMLNode* before, XMLNode* after)
 }
 
 XMLNode&
-AutomationList::get_state ()
+AutomationList::get_state () const
 {
 	return state (true, true);
 }
 
 XMLNode&
-AutomationList::state (bool save_auto_state, bool need_lock)
+AutomationList::state (bool save_auto_state, bool need_lock) const
 {
 	XMLNode* root = new XMLNode (X_("AutomationList"));
 
 	root->set_property ("automation-id", EventTypeMap::instance().to_symbol(_parameter));
 	root->set_property ("id", id());
 	root->set_property ("interpolation-style", _interpolation);
+	root->set_property ("time-domain", enum_2_string (time_domain()));
 
 	if (save_auto_state) {
 		/* never serialize state with Write enabled - too dangerous
@@ -386,7 +364,7 @@ AutomationList::state (bool save_auto_state, bool need_lock)
 }
 
 XMLNode&
-AutomationList::serialize_events (bool need_lock)
+AutomationList::serialize_events (bool need_lock) const
 {
 	XMLNode* node = new XMLNode (X_("events"));
 	stringstream str;
@@ -395,14 +373,14 @@ AutomationList::serialize_events (bool need_lock)
 	if (need_lock) {
 		lm.acquire ();
 	}
-	for (iterator xx = _events.begin(); xx != _events.end(); ++xx) {
+	for (const_iterator xx = _events.begin(); xx != _events.end(); ++xx) {
 		str << PBD::to_string ((*xx)->when);
 		str << ' ';
 		str << PBD::to_string ((*xx)->value);
 		str << '\n';
 	}
 
-	/* XML is a bit wierd */
+	/* XML is a bit weird */
 
 	XMLNode* content_node = new XMLNode (X_("foo")); /* it gets renamed by libxml when we set content */
 	content_node->set_content (str.str());
@@ -432,13 +410,13 @@ AutomationList::deserialize_events (const XMLNode& node)
 
 	std::string x_str;
 	std::string y_str;
-	double x;
+	timepos_t x;
 	double y;
 	bool ok = true;
 
 	while (str) {
 		str >> x_str;
-		if (!str || !PBD::string_to<double> (x_str, x)) {
+		if (!str || !PBD::string_to<timepos_t> (x_str, x)) {
 			break;
 		}
 		str >> y_str;
@@ -469,6 +447,11 @@ AutomationList::set_state (const XMLNode& node, int version)
 	XMLNodeList nlist = node.children();
 	XMLNode* nsos;
 	XMLNodeIterator niter;
+	Temporal::TimeDomain time_domain;
+
+	if (node.get_property ("time-domain", time_domain)) {
+		set_time_domain_empty (time_domain);
+	}
 
 	if (node.name() == X_("events")) {
 		/* partial state setting*/
@@ -505,7 +488,7 @@ AutomationList::set_state (const XMLNode& node, int version)
 			}
 
 			y = std::min ((double)_desc.upper, std::max ((double)_desc.lower, y));
-			fast_simple_add (x, y);
+			fast_simple_add (timepos_t (x), y);
 		}
 
 		thaw ();
