@@ -23,7 +23,7 @@
 #include "pbd/memento_command.h"
 #include "pbd/playback_buffer.h"
 
-#include "evoral/Range.h"
+#include "temporal/range.h"
 
 #include "ardour/amp.h"
 #include "ardour/audio_buffer.h"
@@ -57,8 +57,8 @@ DiskReader::Declicker DiskReader::loop_declick_in;
 DiskReader::Declicker DiskReader::loop_declick_out;
 samplecnt_t           DiskReader::loop_fade_length (0);
 
-DiskReader::DiskReader (Session& s, Track& t, string const& str, DiskIOProcessor::Flag f)
-	: DiskIOProcessor (s, t, X_("player:") + str, f)
+DiskReader::DiskReader (Session& s, Track& t, string const& str,  Temporal::TimeDomain td, DiskIOProcessor::Flag f)
+	: DiskIOProcessor (s, t, X_("player:") + str, f, td)
 	, overwrite_sample (0)
 	, run_must_resolve (false)
 	, _declick_amp (s.nominal_sample_rate ())
@@ -166,7 +166,7 @@ DiskReader::set_name (string const& str)
 }
 
 XMLNode&
-DiskReader::state ()
+DiskReader::state () const
 {
 	XMLNode& node (DiskIOProcessor::state ());
 	node.set_property (X_("type"), X_("diskreader"));
@@ -186,6 +186,10 @@ DiskReader::set_state (const XMLNode& node, int version)
 void
 DiskReader::realtime_handle_transport_stopped ()
 {
+
+	if (_session.exporting () && !_session.realtime_export ()) {
+		_declick_amp.set_gain (0);
+	}
 	/* can't do the resolve here because we don't have a place to put the
 	 * note resolving data. Defer to
 	 * MidiTrack::realtime_handle_transport_stopped() which will call
@@ -277,17 +281,8 @@ DiskReader::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 	const bool                     midi_only = (c->empty() || !_playlists[DataType::AUDIO]);
 	bool                           no_disk_output = g_atomic_int_get (&_no_disk_output) != 0;
 
-	if (_active) {
-		if (!_pending_active) {
-			_active = false;
-			return;
-		}
-	} else {
-		if (_pending_active) {
-			_active = true;
-		} else {
-			return;
-		}
+	if (!check_active()) {
+		return;
 	}
 
 	const gain_t target_gain = ((speed == 0.0) || ((ms & MonitoringDisk) == 0)) ? 0.0 : 1.0;
@@ -401,8 +396,9 @@ DiskReader::run (BufferSet& bufs, samplepos_t start_sample, samplepos_t end_samp
 				samplepos_t ss  = start_sample;
 				Location*   loc = _loop_location;
 				if (loc) {
-					Evoral::Range<samplepos_t> loop_range (loc->start (), loc->end () - 1);
-					ss = loop_range.squish (playback_sample);
+					Temporal::Range loop_range (loc->start (), loc->end ());
+					ss = loop_range.squish (timepos_t (playback_sample)).samples();
+					playback_sample = ss;
 				}
 				if (ss != playback_sample) {
 					if (can_internal_playback_seek (ss - playback_sample)) {
@@ -492,8 +488,8 @@ midi:
 
 		Location* loc = _loop_location;
 		if (loc) {
-			Evoral::Range<samplepos_t> loop_range (loc->start (), loc->end () - 1);
-			playback_sample = loop_range.squish (playback_sample);
+			Temporal::Range loop_range (loc->start (), loc->end ());
+			playback_sample = loop_range.squish (timepos_t (playback_sample)).samples();
 		}
 
 		if (_playlists[DataType::AUDIO]) {
@@ -769,14 +765,18 @@ DiskReader::overwrite_existing_midi ()
 		MidiTrack* mt = dynamic_cast<MidiTrack*> (&_track);
 		MidiChannelFilter* filter = mt ? &mt->playback_filter () : 0;
 
+#ifdef PROFILE_MIDI_IO
 		PBD::Timing minsert;
 		minsert.start ();
+#endif
 
 		midi_playlist ()->render (filter);
-
-		minsert.update ();
 		assert (midi_playlist ()->rendered ());
+
+#ifdef PROFILE_MIDI_IO
+		minsert.update ();
 		cerr << "Reading " << name () << " took " << minsert.elapsed () << " microseconds, final size = " << midi_playlist ()->rendered ()->size () << endl;
+#endif
 	}
 
 	return true;
@@ -827,7 +827,7 @@ DiskReader::seek (samplepos_t sample, bool complete_refill)
 	/* There are two possible shortcuts we can take that will completely
 	 * skip reading from disk. However, they are invalid if we need to read
 	 * data in the opposite direction than we did last time, or if our need
-	 * for looped data has chaned since the last read. Both of these change
+	 * for looped data has changed since the last read. Both of these change
 	 * the semantics of a read from disk, even if the position we are
 	 * reading from is the same.
 	 */
@@ -990,7 +990,7 @@ DiskReader::audio_read (Sample*            sum_buffer,
 {
 	samplecnt_t       this_read  = 0;
 	bool              reloop     = false;
-	samplepos_t       loop_end   = 0;
+	samplepos_t       loop_end = 0;
 	samplepos_t       loop_start = 0;
 	Location*         loc        = 0;
 	const samplecnt_t rcnt       = cnt;
@@ -1008,12 +1008,11 @@ DiskReader::audio_read (Sample*            sum_buffer,
 		*/
 
 		if ((loc = _loop_location) != 0) {
-			loop_start  = loc->start ();
-			loop_end    = loc->end ();
+			loop_start  = loc->start_sample ();
+			loop_end    = loc->end_sample ();
 
-			/* Evoral::Range has inclusive range semantics. Ugh. Hence the -1 */
-			const Evoral::Range<samplepos_t> loop_range (loop_start, loop_end - 1);
-			start = loop_range.squish (start);
+			const Temporal::Range loop_range (loc->start(), loc->end());
+			start = loop_range.squish (timepos_t (start)).samples();
 		}
 
 	} else {
@@ -1047,7 +1046,7 @@ DiskReader::audio_read (Sample*            sum_buffer,
 		 * useful after the return from AudioPlayback::read()
 		 */
 
-		if (audio_playlist ()->read (sum_buffer, mixdown_buffer, gain_buffer, start, this_read, channel) != this_read) {
+		if (audio_playlist ()->read (sum_buffer, mixdown_buffer, gain_buffer, timepos_t (start), timecnt_t::from_samples (this_read), channel) != this_read) {
 			error << string_compose (_("DiskReader %1: cannot read %2 from playlist at sample %3"), id (), this_read, start) << endmsg;
 			return 0;
 		}
@@ -1066,9 +1065,9 @@ DiskReader::audio_read (Sample*            sum_buffer,
 					loop_declick_out.run (sum_buffer, start, start + this_read);
 					break;
 				case XFadeLoop:
-					if (last_refill_loop_start != loc->start() || rci->pre_loop_buffer == 0) {
+					if (last_refill_loop_start != loop_start || rci->pre_loop_buffer == 0) {
 						setup_preloop_buffer ();
-						last_refill_loop_start = loc->start();
+						last_refill_loop_start = loop_start;
 					}
 					maybe_xfade_loop (sum_buffer, start, start + this_read, rci);
 					break;
@@ -1291,12 +1290,12 @@ DiskReader::refill_audio (Sample* sum_buffer, Sample* mixdown_buffer, float* gai
 		file_sample_tmp = fsa;
 		samplecnt_t ts  = total_space;
 
-		samplecnt_t to_read = min (ts, (samplecnt_t)chan->rbuf->write_space ());
+		const guint wr_space = chan->rbuf->write_space ();
+
+		samplecnt_t to_read = min (ts, (samplecnt_t)wr_space);
 		to_read             = min (to_read, samples_to_read);
 		assert (to_read >= 0);
 
-		// cerr << owner()->name() << " to-read: " << to_read << endl;
-
 		if (to_read) {
 			ReaderChannelInfo* rci = dynamic_cast<ReaderChannelInfo*> (chan);
 
@@ -1304,15 +1303,15 @@ DiskReader::refill_audio (Sample* sum_buffer, Sample* mixdown_buffer, float* gai
 				chan->rbuf->write_zero (to_read);
 
 			} else {
-				samplecnt_t nread;
+				samplecnt_t nread, nwritten;
 				if ((nread = audio_read (sum_buffer, mixdown_buffer, gain_buffer, file_sample_tmp, to_read, rci, chan_n, reversed)) != to_read) {
-					error << string_compose (_("DiskReader %1: when refilling, cannot read %2 from playlist at sample %3"), name (), to_read, fsa) << endmsg;
+					error << string_compose (_("DiskReader %1: when refilling, cannot read %2 from playlist at sample %3 (rv: %4)"), name (), to_read, fsa, nread) << endmsg;
 					ret = -1;
 					goto out;
 				}
 
-				if (chan->rbuf->write (sum_buffer, nread) != nread) {
-					error << string_compose (_("DiskReader %1: when refilling, cannot write %2 into buffer"), name (), nread) << endmsg;
+				if ((nwritten = chan->rbuf->write (sum_buffer, nread)) != nread) {
+					error << string_compose (_("DiskReader %1: when refilling, cannot write %2 into buffer (wrote %3, space %4)"), name (), nread, nwritten, wr_space) << endmsg;
 					ret = -1;
 				}
 			}
@@ -1345,7 +1344,7 @@ out:
 }
 
 void
-DiskReader::playlist_ranges_moved (list<Evoral::RangeMove<samplepos_t> > const& movements_samples, bool from_undo_or_shift)
+DiskReader::playlist_ranges_moved (list<Temporal::RangeMove> const& movements, bool from_undo_or_shift)
 {
 	/* If we're coming from an undo, it will have handled
 	 * automation undo (it must, since automation-follows-regions
@@ -1364,14 +1363,6 @@ DiskReader::playlist_ranges_moved (list<Evoral::RangeMove<samplepos_t> > const&
 		return;
 	}
 
-	list<Evoral::RangeMove<double> > movements;
-
-	for (list<Evoral::RangeMove<samplepos_t> >::const_iterator i = movements_samples.begin ();
-	     i != movements_samples.end ();
-	     ++i) {
-		movements.push_back (Evoral::RangeMove<double> (i->from, i->length, i->to));
-	}
-
 	/* move panner automation */
 	boost::shared_ptr<Pannable>   pannable = _track.pannable ();
 	Evoral::ControlSet::Controls& c (pannable->controls ());
@@ -1393,22 +1384,17 @@ DiskReader::playlist_ranges_moved (list<Evoral::RangeMove<samplepos_t> > const&
 		}
 	}
 	/* move processor automation */
-	_track.foreach_processor (boost::bind (&DiskReader::move_processor_automation, this, _1, movements_samples));
+	_track.foreach_processor (boost::bind (&DiskReader::move_processor_automation, this, _1, movements));
 }
 
 void
-DiskReader::move_processor_automation (boost::weak_ptr<Processor> p, list<Evoral::RangeMove<samplepos_t> > const& movements_samples)
+DiskReader::move_processor_automation (boost::weak_ptr<Processor> p, list<Temporal::RangeMove> const& movements)
 {
 	boost::shared_ptr<Processor> processor (p.lock ());
 	if (!processor) {
 		return;
 	}
 
-	list<Evoral::RangeMove<double> > movements;
-	for (list<Evoral::RangeMove<samplepos_t> >::const_iterator i = movements_samples.begin (); i != movements_samples.end (); ++i) {
-		movements.push_back (Evoral::RangeMove<double> (i->from, i->length, i->to));
-	}
-
 	set<Evoral::Parameter> const a = processor->what_can_be_automated ();
 
 	for (set<Evoral::Parameter>::const_iterator i = a.begin (); i != a.end (); ++i) {
@@ -1470,67 +1456,79 @@ DiskReader::get_midi_playback (MidiBuffer& dst, samplepos_t start_sample, sample
 		target = &dst;
 	}
 
-	if (!g_atomic_int_get (&_no_disk_output)) {
-		const samplecnt_t nframes = abs (end_sample - start_sample);
+	if (g_atomic_int_get (&_no_disk_output)) {
+		return;
+	}
 
-		if (ms & MonitoringDisk) {
-			/* disk data needed */
+	const samplecnt_t nframes = abs (end_sample - start_sample);
 
-			Location* loc = _loop_location;
+	if (ms & MonitoringDisk) {
+		/* disk data needed */
 
-			if (loc) {
-				/* Evoral::Range has inclusive range semantics. Ugh. Hence the -1 */
-				const Evoral::Range<samplepos_t> loop_range (loc->start (), loc->end () - 1);
-				samplepos_t                      effective_start = start_sample;
-				samplecnt_t                      cnt             = nframes;
-				sampleoffset_t                   offset          = 0;
+		Location* loc = _loop_location;
 
-				DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("LOOP read, loop is %1..%2 range is %3..%4 nf %5\n", loc->start (), loc->end (), start_sample, end_sample, nframes));
-
-				do {
-					samplepos_t effective_end;
-
-					effective_start = loop_range.squish (effective_start);
-					effective_end   = min (effective_start + cnt, loc->end ());
-					assert (effective_end > effective_start);
-
-					const samplecnt_t this_read = effective_end - effective_start;
-
-					DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("playback buffer LOOP read, from %1 to %2 (%3)\n", effective_start, effective_end, this_read));
-
-					size_t events_read = rtmb->read (*target, effective_start, effective_end, _tracker, offset);
-					cnt -= this_read;
-					effective_start += this_read;
-					offset += this_read;
-
-					DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("%1 MDS events LOOP read %2 cnt now %3\n", _name, events_read, cnt));
-
-					if (cnt) {
-						/* We re going to have to read across the loop end. Resolve any notes the extend across the loop end.
-						 * Time is relative to start_sample.
-						 */
-						DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("read crosses loop end, resolve @ %1\n", effective_end - start_sample));
-						_tracker.resolve_notes (*target, effective_end - start_sample);
-					}
-
-				} while (cnt);
-
-			} else {
-				DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("playback buffer read, from %1 to %2 (%3)\n", start_sample, end_sample, nframes));
-				size_t events_read = rtmb->read (*target, start_sample, end_sample, _tracker);
-				DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("%1 MDS events read %2 range %3 .. %4\n", _name, events_read, playback_sample, playback_sample + nframes));
-			}
-		}
-
-		if (ms & MonitoringInput) {
-			/* merges data from disk (in "target", which is a scratch
-			 * buffer in this case) into the actual destination buffer
-			 * (which holds existing input data).
+		if (loc) {
+			/* squish() operates in the location's time-domain. When the location was created
+			 * using music-time, and later converted to audio-time, it can happen that the
+			 * corresponding super-clock is "between samples". e.g loop-end is at sample 1000.12.
+			 * if start_sample = 1000; squish() does nothing because 1000 < 1000.12.
+			 * This is solved by creating the range using (rounded) sample-times.
 			 */
-			dst.merge_from (*target, nframes);
+			const Temporal::Range loop_range (loc->start ().samples(), loc->end ().samples());
+			samplepos_t           effective_start = start_sample;
+			samplecnt_t           cnt             = nframes;
+			sampleoffset_t        offset          = 0;
+			const samplepos_t     loop_end        = loc->end_sample();
+
+			DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("LOOP read, loop is %1..%2 range is %3..%4 nf %5\n", loc->start (), loc->end (), start_sample, end_sample, nframes));
+
+			do {
+				samplepos_t effective_end;
+
+				effective_start = loop_range.squish (timepos_t (effective_start)).samples();
+				effective_end   = min (effective_start + cnt, loop_end);
+				assert (effective_end > effective_start);
+
+				const samplecnt_t this_read = effective_end - effective_start;
+
+				DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("playback buffer LOOP read, from %1 to %2 (%3)\n", effective_start, effective_end, this_read));
+
+#ifndef NDEBUG
+				size_t events_read =
+#endif
+					rtmb->read (*target, effective_start, effective_end, _tracker, offset);
+
+				cnt -= this_read;
+				effective_start += this_read;
+				offset += this_read;
+
+				DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("%1 MDS events LOOP read %2 cnt now %3\n", _name, events_read, cnt));
+
+				if (cnt) {
+					/* We re going to have to read across the loop end. Resolve any notes the extend across the loop end.
+					 * Time is relative to start_sample.
+					 */
+					DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("read crosses loop end, resolve @ %1\n", effective_end - start_sample));
+					_tracker.resolve_notes (*target, effective_end - start_sample);
+				}
+
+			} while (cnt);
+
+		} else {
+			DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("playback buffer read, from %1 to %2 (%3)\n", start_sample, end_sample, nframes));
+			DEBUG_RESULT (size_t, events_read, rtmb->read (*target, start_sample, end_sample, _tracker));
+			DEBUG_TRACE (DEBUG::MidiDiskIO, string_compose ("%1 MDS events read %2 range %3 .. %4\n", _name, events_read, playback_sample, playback_sample + nframes));
 		}
 	}
 
+	if (ms & MonitoringInput) {
+		/* merges data from disk (in "target", which is a scratch
+		 * buffer in this case) into the actual destination buffer
+		 * (which holds existing input data).
+		 */
+		dst.merge_from (*target, nframes);
+	}
+
 #if 0
 	if (!target->empty ()) {
 		cerr << "======== MIDI OUT ========\n";
@@ -1725,9 +1723,9 @@ DiskReader::Declicker::reset (samplepos_t loop_start, samplepos_t loop_end, bool
 void
 DiskReader::Declicker::run (Sample* buf, samplepos_t read_start, samplepos_t read_end)
 {
-	samplecnt_t    n;  /* how many samples to process */
-	sampleoffset_t bo; /* offset into buffer */
-	sampleoffset_t vo; /* offset into gain vector */
+	samplecnt_t    n  = 0; /* how many samples to process */
+	sampleoffset_t bo = 0; /* offset into buffer */
+	sampleoffset_t vo = 0; /* offset into gain vector */
 
 	if (fade_start == fade_end) {
 		return;
@@ -1739,8 +1737,8 @@ DiskReader::Declicker::run (Sample* buf, samplepos_t read_start, samplepos_t rea
 	 * see also DiskReader::maybe_xfade_loop()
 	 */
 
-	switch (Evoral::coverage (fade_start, fade_end, read_start, read_end)) {
-		case Evoral::OverlapInternal:
+	switch (Temporal::coverage_exclusive_ends (fade_start, fade_end, read_start, read_end)) {
+		case Temporal::OverlapInternal:
 			/* note: start and end points cannot coincide (see evoral/Range.h)
 			 *
 			 * read range is entirely within fade range
@@ -1750,7 +1748,7 @@ DiskReader::Declicker::run (Sample* buf, samplepos_t read_start, samplepos_t rea
 			n  = read_end - read_start;
 			break;
 
-		case Evoral::OverlapExternal:
+		case Temporal::OverlapExternal:
 			/* read range extends on either side of fade range
 			 *
 			 * External allows coincidental start & end points, so check for that
@@ -1767,14 +1765,14 @@ DiskReader::Declicker::run (Sample* buf, samplepos_t read_start, samplepos_t rea
 			}
 			break;
 
-		case Evoral::OverlapStart:
+		case Temporal::OverlapStart:
 			/* read range starts before and ends within fade or at same end as fade */
 			n  = fade_end - read_start;
 			vo = 0;
 			bo = fade_start - read_start;
 			break;
 
-		case Evoral::OverlapEnd:
+		case Temporal::OverlapEnd:
 			/* read range starts within fade range, but possibly at it's end, so check */
 			if (read_start == fade_end) {
 				/* nothing to do */
@@ -1785,7 +1783,7 @@ DiskReader::Declicker::run (Sample* buf, samplepos_t read_start, samplepos_t rea
 			n  = fade_end - read_start;
 			break;
 
-		case Evoral::OverlapNone:
+		case Temporal::OverlapNone:
 			/* no overlap ... nothing to do */
 			return;
 	}
@@ -1801,9 +1799,9 @@ DiskReader::Declicker::run (Sample* buf, samplepos_t read_start, samplepos_t rea
 void
 DiskReader::maybe_xfade_loop (Sample* buf, samplepos_t read_start, samplepos_t read_end, ReaderChannelInfo* chan)
 {
-	samplecnt_t    n;  /* how many samples to process */
-	sampleoffset_t bo; /* offset into buffer */
-	sampleoffset_t vo; /* offset into gain vector */
+	samplecnt_t    n  = 0;  /* how many samples to process */
+	sampleoffset_t bo = 0; /* offset into buffer */
+	sampleoffset_t vo = 0; /* offset into gain vector */
 
 	const samplepos_t fade_start = loop_declick_out.fade_start;
 	const samplepos_t fade_end   = loop_declick_out.fade_end;
@@ -1818,8 +1816,8 @@ DiskReader::maybe_xfade_loop (Sample* buf, samplepos_t read_start, samplepos_t r
 	 * see also DiskReader::Declicker::run()
 	 */
 
-	switch (Evoral::coverage (fade_start, fade_end, read_start, read_end)) {
-		case Evoral::OverlapInternal:
+	switch (Temporal::coverage_exclusive_ends (fade_start, fade_end, read_start, read_end)) {
+		case Temporal::OverlapInternal:
 			/* note: start and end points cannot coincide (see evoral/Range.h)
 			 *
 			 * read range is entirely within fade range
@@ -1829,7 +1827,7 @@ DiskReader::maybe_xfade_loop (Sample* buf, samplepos_t read_start, samplepos_t r
 			n  = read_end - read_start;
 			break;
 
-		case Evoral::OverlapExternal:
+		case Temporal::OverlapExternal:
 			/* read range extends on either side of fade range
 			 *
 			 * External allows coincidental start & end points, so check for that
@@ -1846,14 +1844,14 @@ DiskReader::maybe_xfade_loop (Sample* buf, samplepos_t read_start, samplepos_t r
 			}
 			break;
 
-		case Evoral::OverlapStart:
+		case Temporal::OverlapStart:
 			/* read range starts before and ends within fade or at same end as fade */
 			n  = read_end - fade_start;
 			vo = 0;
 			bo = fade_start - read_start;
 			break;
 
-		case Evoral::OverlapEnd:
+		case Temporal::OverlapEnd:
 			/* read range starts within fade range, but possibly at it's end, so check */
 			if (read_start == fade_end) {
 				/* nothing to do */
@@ -1864,7 +1862,7 @@ DiskReader::maybe_xfade_loop (Sample* buf, samplepos_t read_start, samplepos_t r
 			n  = fade_end - read_start;
 			break;
 
-		case Evoral::OverlapNone:
+		case Temporal::OverlapNone:
 			/* no overlap ... nothing to do */
 			return;
 	}
@@ -1912,8 +1910,8 @@ void
 DiskReader::reset_loop_declick (Location* loc, samplecnt_t sr)
 {
 	if (loc) {
-		loop_declick_in.reset (loc->start (), loc->end (), true, sr);
-		loop_declick_out.reset (loc->start (), loc->end (), false, sr);
+		loop_declick_in.reset (loc->start_sample (), loc->end_sample (), true, sr);
+		loop_declick_out.reset (loc->start_sample (), loc->end_sample (), false, sr);
 	} else {
 		loop_declick_in.reset (0, 0, true, sr);
 		loop_declick_out.reset (0, 0, false, sr);
@@ -1946,6 +1944,8 @@ DiskReader::setup_preloop_buffer ()
 	Location*                   loc = _loop_location;
 	boost::scoped_array<Sample> mix_buf (new Sample[loop_fade_length]);
 	boost::scoped_array<Sample> gain_buf (new Sample[loop_fade_length]);
+	const timepos_t read_start = timepos_t (loc->start_sample() - loop_declick_out.fade_length);
+	const timecnt_t read_cnt = timecnt_t (loop_declick_out.fade_length);
 
 	uint32_t channel = 0;
 
@@ -1955,7 +1955,7 @@ DiskReader::setup_preloop_buffer ()
 		rci->resize_preloop (loop_fade_length);
 
 		if (loc->start () > loop_fade_length) {
-			audio_playlist ()->read (rci->pre_loop_buffer, mix_buf.get (), gain_buf.get (), loc->start () - loop_declick_out.fade_length, loop_declick_out.fade_length, channel);
+			audio_playlist ()->read (rci->pre_loop_buffer, mix_buf.get (), gain_buf.get (), read_start, read_cnt, channel);
 		} else {
 			memset (rci->pre_loop_buffer, 0, sizeof (Sample) * loop_fade_length);
 		}
