@@ -86,8 +86,8 @@ ExportGraphBuilder::process (samplecnt_t samples, bool last_cycle)
 
 	sampleoffset_t off = 0;
 	for (ChannelMap::iterator it = channels.begin(); it != channels.end(); ++it) {
-		Sample const * process_buffer = 0;
-		it->first->read (process_buffer, samples);
+		Buffer const* buf;
+		it->first->read (buf, samples);
 
 		if (session.remaining_latency_preroll () >= _master_align + samples) {
 			/* Skip processing during pre-roll, only read/write export ringbuffers */
@@ -100,9 +100,17 @@ ExportGraphBuilder::process (samplecnt_t samples, bool last_cycle)
 			assert (off < samples);
 		}
 
-		ConstProcessContext<Sample> context(&process_buffer[off], samples - off, 1);
-		if (last_cycle) { context().set_flag (ProcessContext<Sample>::EndOfInput); }
-		it->second->process (context);
+		AudioBuffer const* ab = dynamic_cast<AudioBuffer const*> (buf);
+		MidiBuffer const*  mb;
+		if (ab) {
+			Sample const* process_buffer = ab->data ();
+			ConstProcessContext<Sample> context(&process_buffer[off], samples - off, 1);
+			if (last_cycle) { context().set_flag (ProcessContext<Sample>::EndOfInput); }
+			it->second->process (context);
+		}
+		if  ((mb = dynamic_cast<MidiBuffer const*> (buf))) {
+			it->second->process (*mb, off, samples - off, last_cycle);
+		}
 	}
 
 	return samples - off;
@@ -140,6 +148,7 @@ ExportGraphBuilder::reset ()
 	channels.clear ();
 	intermediates.clear ();
 	analysis_map.clear();
+	_exported_files.clear();
 	_realtime = false;
 	_master_align = 0;
 }
@@ -369,9 +378,15 @@ ExportGraphBuilder::Encoder::init_writer (boost::shared_ptr<AudioGrapher::CmdPip
 	char tmp[64];
 	argp[a++] = strdup(ffmpeg_exe.c_str());
 	argp[a++] = strdup ("-f");
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
 	argp[a++] = strdup ("f32le");
 	argp[a++] = strdup ("-acodec");
 	argp[a++] = strdup ("pcm_f32le");
+#else
+	argp[a++] = strdup ("f32be");
+	argp[a++] = strdup ("-acodec");
+	argp[a++] = strdup ("pcm_f32be");
+#endif
 	argp[a++] = strdup ("-ac");
 	snprintf (tmp, sizeof(tmp), "%d", channels);
 	argp[a++] = strdup (tmp);
@@ -410,7 +425,7 @@ ExportGraphBuilder::Encoder::init_writer (boost::shared_ptr<AudioGrapher::CmdPip
 
 	/* argp is free()d in ~SystemExec,
 	 * SystemExec is deleted when writer is destroyed */
-	ARDOUR::SystemExec* exec = new ARDOUR::SystemExec (ffmpeg_exe, argp);
+	ARDOUR::SystemExec* exec = new ARDOUR::SystemExec (ffmpeg_exe, argp, true);
 	PBD::info << "Encode command: { " << exec->to_s () << "}" << endmsg;
 	if (exec->start (SystemExec::MergeWithStdin)) {
 		throw ExportFailed ("External encoder (ffmpeg) cannot be started.");
@@ -447,6 +462,9 @@ ExportGraphBuilder::SFC::SFC (ExportGraphBuilder &parent, FileSpec const & new_c
 
 	boost::shared_ptr<AudioGrapher::ListedSource<float> > intermediate = limiter;
 
+	config.filename->set_channel_config (config.channel_config);
+	parent.add_export_fn (config.filename->get_path (config.format));
+
 	if (_analyse) {
 		samplecnt_t sample_rate = parent.session.nominal_sample_rate();
 		samplecnt_t sb = config.format->silence_beginning_at (parent.timespan->get_start(), sample_rate);
@@ -456,9 +474,10 @@ ExportGraphBuilder::SFC::SFC (ExportGraphBuilder &parent, FileSpec const & new_c
 		max_samples = std::min ((samplecnt_t) 8192 * channels, std::max ((samplecnt_t) 4096 * channels, max_samples));
 		chunker.reset (new Chunker<Sample> (max_samples));
 		analyser.reset (new Analyser (config.format->sample_rate(), channels, max_samples,
-					(samplecnt_t) ceil (duration * config.format->sample_rate () / (double) sample_rate)));
+		                              (samplecnt_t) ceil (duration * config.format->sample_rate () / (double) sample_rate),
+		                              800 * ui_scale_factor, 200 * ui_scale_factor
+		                             ));
 
-		config.filename->set_channel_config (config.channel_config);
 		parent.add_analyser (config.filename->get_path (config.format), analyser);
 		limiter->set_result (analyser->result (true));
 
@@ -931,18 +950,33 @@ ExportGraphBuilder::ChannelConfig::ChannelConfig (ExportGraphBuilder & parent, F
 
 	ChannelList const & channel_list = config.channel_config->get_channels();
 	unsigned chan = 0;
+	unsigned n_audio = 0;
 	for (ChannelList::const_iterator it = channel_list.begin(); it != channel_list.end(); ++it, ++chan) {
 		ChannelMap::iterator map_it = channel_map.find (*it);
 		if (map_it == channel_map.end()) {
 			std::pair<ChannelMap::iterator, bool> result_pair =
-				channel_map.insert (std::make_pair (*it, IdentityVertexPtr (new IdentityVertex<Sample> ())));
+				channel_map.insert (std::make_pair (*it, AnyExportPtr (new AnyExport ())));
 			assert (result_pair.second);
 			map_it = result_pair.first;
 		}
-		map_it->second->add_output (interleaver->input (chan));
+		if ((*it)->midi ()) {
+			config.filename->set_channel_config(config.channel_config);
+			std::string writer_filename = config.filename->get_path (ExportFormatSpecPtr ()) + ".mid";
+			map_it->second->midi.init (writer_filename, parent.timespan->get_start ());
+			parent.add_export_fn (writer_filename);
+		}
+		if ((*it)->audio ()) {
+			++n_audio;
+			map_it->second->add_output (interleaver->input (chan));
+		}
 	}
 
-	add_child (new_config);
+	if (n_audio > 0) {
+		add_child (new_config);
+	} else {
+		chunker.reset ();
+		interleaver.reset ();
+	}
 }
 
 void
