@@ -53,46 +53,56 @@ namespace PBD {
 using namespace std;
 using namespace ARDOUR;
 using namespace PBD;
+using namespace Temporal;
 
-PBD::Signal0<void> Location::scene_changed;
 PBD::Signal1<void,Location*> Location::name_changed;
 PBD::Signal1<void,Location*> Location::end_changed;
 PBD::Signal1<void,Location*> Location::start_changed;
 PBD::Signal1<void,Location*> Location::flags_changed;
 PBD::Signal1<void,Location*> Location::lock_changed;
-PBD::Signal1<void,Location*> Location::position_lock_style_changed;
+PBD::Signal1<void,Location*> Location::cue_change;
+PBD::Signal1<void,Location*> Location::scene_changed;
+PBD::Signal1<void,Location*> Location::time_domain_changed;
 PBD::Signal1<void,Location*> Location::changed;
 
 Location::Location (Session& s)
 	: SessionHandleRef (s)
-	, _start (0)
-	, _start_beat (0.0)
-	, _end (0)
-	, _end_beat (0.0)
 	, _flags (Flags (0))
 	, _locked (false)
-	, _position_lock_style (AudioTime)
 	, _timestamp (time (0))
+	, _cue (0)
+	, _signals_suspended (0)
 {
-	assert (_start >= 0);
-	assert (_end >= 0);
 }
 
 /** Construct a new Location, giving it the position lock style determined by glue-new-markers-to-bars-and-beats */
-Location::Location (Session& s, samplepos_t sample_start, samplepos_t sample_end, const std::string &name, Flags bits, const uint32_t sub_num)
+Location::Location (Session& s, timepos_t const & start, timepos_t const & end, const std::string &name, Flags bits, int32_t cue_id)
 	: SessionHandleRef (s)
 	, _name (name)
-	, _start (sample_start)
-	, _end (sample_end)
+	, _start (start)
+	, _end (end)
 	, _flags (bits)
 	, _locked (false)
-	, _position_lock_style (s.config.get_glue_new_markers_to_bars_and_beats() ? MusicTime : AudioTime)
 	, _timestamp (time (0))
+	, _cue (cue_id)
+	, _signals_suspended (0)
 {
-	recompute_beat_from_samples (sub_num);
 
-	assert (_start >= 0);
-	assert (_end >= 0);
+	/* it would be nice if the caller could ensure that the start and end
+	   values simply use the correct domain, but that would involve
+	   enforcing/checking that at every place where we create a
+	   Location. So instead we centralize this here.
+
+	   NUTEMPO: it might make sense to switch time domains when <something>
+	   happens, but it's not clear what the <something> might be? Maybe
+	   changing some setting of the tempo map.
+	*/
+
+	if (s.config.get_glue_new_markers_to_bars_and_beats()) {
+		set_position_time_domain (Temporal::BeatTime);
+	} else {
+		set_position_time_domain (Temporal::AudioTime);
+	}
 }
 
 Location::Location (const Location& other)
@@ -100,39 +110,36 @@ Location::Location (const Location& other)
 	, StatefulDestructible()
 	, _name (other._name)
 	, _start (other._start)
-	, _start_beat (other._start_beat)
 	, _end (other._end)
-	, _end_beat (other._end_beat)
 	, _flags (other._flags)
-	, _position_lock_style (other._position_lock_style)
 	, _timestamp (time (0))
+	, _cue (other._cue)
+	, _signals_suspended (0)
 {
 	/* copy is not locked even if original was */
+	assert (other._signals_suspended == 0);
 
 	_locked = false;
 
-	assert (_start >= 0);
-	assert (_end >= 0);
-
 	/* scene change is NOT COPIED */
 }
 
 Location::Location (Session& s, const XMLNode& node)
 	: SessionHandleRef (s)
 	, _flags (Flags (0))
-	, _position_lock_style (AudioTime)
 	, _timestamp (time (0))
+	, _signals_suspended (0)
 {
-	/* Note: _position_lock_style is initialised above in case set_state doesn't set it
-	   (for 2.X session file compatibility).
-	*/
+	//_start.set_time_domain (AudioTime);
+	//_end.set_time_domain (AudioTime);
+
+	/* Note: _position_time_domain is initialised above in case set_state
+	 * doesn't set it
+	 */
 
 	if (set_state (node, Stateful::loading_state_version)) {
 		throw failed_constructor ();
 	}
-
-	assert (_start >= 0);
-	assert (_end >= 0);
 }
 
 bool
@@ -141,10 +148,7 @@ Location::operator== (const Location& other)
 	if (_name != other._name ||
 	    _start != other._start ||
 	    _end != other._end ||
-	    _start_beat != other._start_beat ||
-	    _end_beat != other._end_beat ||
-	    _flags != other._flags ||
-	    _position_lock_style != other._position_lock_style) {
+	    _flags != other._flags) {
 		return false;
 	}
 	return true;
@@ -159,11 +163,10 @@ Location::operator= (const Location& other)
 
 	_name = other._name;
 	_start = other._start;
-	_start_beat = other._start_beat;
 	_end = other._end;
-	_end_beat = other._end_beat;
 	_flags = other._flags;
-	_position_lock_style = other._position_lock_style;
+
+	assert (other._signals_suspended == 0);
 
 	/* XXX need to copy scene change */
 
@@ -173,35 +176,98 @@ Location::operator= (const Location& other)
 
 	/* "changed" not emitted on purpose */
 
-	assert (_start >= 0);
-	assert (_end >= 0);
-
 	return this;
 }
 
-/** Set location name
- */
+void
+Location::suspend_signals ()
+{
+	++_signals_suspended;
+}
 
+void
+Location::resume_signals ()
+{
+	assert (_signals_suspended > 0);
+	if (--_signals_suspended > 0) {
+		return;
+	}
+	for (auto const& s : _postponed_signals) {
+		actually_emit_signal (s);
+	}
+}
+
+void
+Location::emit_signal (Signal s)
+{
+	if (_signals_suspended > 0) {
+		_postponed_signals.insert (s);
+		return;
+	}
+	actually_emit_signal (s);
+}
+
+void
+Location::actually_emit_signal (Signal s)
+{
+	switch (s) {
+		case Name:
+			name_changed (this);
+			NameChanged ();
+			break;
+		case StartEnd:
+			changed (this);
+			Changed ();
+			break;
+		case End:
+			end_changed (this);
+			EndChanged ();
+			break;
+		case Start:
+			start_changed (this);
+			StartChanged ();
+			break;
+		case Flag:
+			flags_changed (this);
+			FlagsChanged ();
+			break;
+		case Lock:
+			lock_changed (this);
+			LockChanged ();
+			break;
+		case Cue:
+			cue_change (this);
+			CueChanged ();
+			break;
+		case Scene:
+			scene_changed (this);
+			SceneChanged ();
+			break;
+		case Domain:
+			time_domain_changed (this);
+			TimeDomainChanged ();
+			break;
+		default:
+			assert (0);
+			break;
+	}
+}
+
+/** Set location name */
 void
 Location::set_name (const std::string& str)
 {
 	_name = str;
-
-	name_changed (this); /* EMIT SIGNAL */
-	NameChanged  (); /* EMIT SIGNAL */
+	emit_signal (Name); /* EMIT SIGNAL*/
 }
 
 /** Set start position.
  *  @param s New start.
  *  @param force true to force setting, even if the given new start is after the current end.
- *  @param allow_beat_recompute True to recompute BEAT start time from the new given start time.
  */
 int
-Location::set_start (samplepos_t s, bool force, bool allow_beat_recompute, const uint32_t sub_num)
+Location::set_start (Temporal::timepos_t const & s, bool force)
 {
-	if (s < 0) {
-		return -1;
-	}
 
 	if (_locked) {
 		return -1;
@@ -217,14 +283,7 @@ Location::set_start (samplepos_t s, bool force, bool allow_beat_recompute, const
 		if (_start != s) {
 			_start = s;
 			_end = s;
-			if (allow_beat_recompute) {
-				recompute_beat_from_samples (sub_num);
-			}
-
-			start_changed (this); /* EMIT SIGNAL */
-			StartChanged (); /* EMIT SIGNAL */
-			//end_changed (this); /* EMIT SIGNAL */
-			//EndChanged (); /* EMIT SIGNAL */
+			emit_signal (Start); /* EMIT SIGNAL*/
 		}
 
 		/* moving the start (position) of a marker with a scene change
@@ -232,54 +291,48 @@ Location::set_start (samplepos_t s, bool force, bool allow_beat_recompute, const
 		*/
 
 		if (_scene_change) {
-			scene_changed (); /* EMIT SIGNAL */
+			emit_signal (Scene); /* EMIT SIGNAL */
 		}
 
-		assert (_start >= 0);
-		assert (_end >= 0);
+		assert (s.is_zero() || s.is_positive());
+
+		if (is_cue_marker()) {
+			emit_signal (Cue); /* EMIT SIGNAL */
+		}
 
 		return 0;
 	} else if (!force) {
 		/* range locations must exceed a minimum duration */
-		if (_end - s < Config->get_range_location_minimum()) {
+		if (s.distance (_end) < Config->get_range_location_minimum()) {
 			return -1;
 		}
 	}
 
 	if (s != _start) {
 
-		samplepos_t const old = _start;
+		Temporal::timepos_t const old = _start;
 
 		_start = s;
-		if (allow_beat_recompute) {
-			recompute_beat_from_samples (sub_num);
-		}
-		start_changed (this); /* EMIT SIGNAL */
-		StartChanged (); /* EMIT SIGNAL */
+		emit_signal (Start); /* EMIT SIGNAL*/
 
 		if (is_session_range ()) {
-			Session::StartTimeChanged (old); /* EMIT SIGNAL */
-			AudioFileSource::set_header_position_offset (s);
+			Session::StartTimeChanged (old.samples()); /* emit signal */
+			AudioFileSource::set_header_position_offset (s.samples());
 		}
 	}
 
-	assert (_start >= 0);
+	assert (_start.is_positive() || _start.is_zero());
 
 	return 0;
 }
 
-/** Set end position.
- *  @param s New end.
+/** set end position.
+ *  @param s new end.
  *  @param force true to force setting, even if the given new end is before the current start.
- *  @param allow_beat_recompute True to recompute BEAT end time from the new given end time.
  */
 int
-Location::set_end (samplepos_t e, bool force, bool allow_beat_recompute, const uint32_t sub_num)
+Location::set_end (Temporal::timepos_t const & e, bool force)
 {
-	if (e < 0) {
-		return -1;
-	}
-
 	if (_locked) {
 		return -1;
 	}
@@ -294,13 +347,7 @@ Location::set_end (samplepos_t e, bool force, bool allow_beat_recompute, const u
 		if (_start != e) {
 			_start = e;
 			_end = e;
-			if (allow_beat_recompute) {
-				recompute_beat_from_samples (sub_num);
-			}
-			//start_changed (this); /* EMIT SIGNAL */
-			//StartChanged (); /* EMIT SIGNAL */
-			end_changed (this); /* EMIT SIGNAL */
-			EndChanged (); /* EMIT SIGNAL */
+			emit_signal (End); /* EMIT SIGNAL*/
 		}
 
 		assert (_start >= 0);
@@ -309,40 +356,31 @@ Location::set_end (samplepos_t e, bool force, bool allow_beat_recompute, const u
 		return 0;
 	} else if (!force) {
 		/* range locations must exceed a minimum duration */
-		if (e - _start < Config->get_range_location_minimum()) {
+		if (_start.distance (e) < Config->get_range_location_minimum()) {
 			return -1;
 		}
 	}
 
 	if (e != _end) {
 
-		samplepos_t const old = _end;
+		timepos_t const old = _end;
 
 		_end = e;
-		if (allow_beat_recompute) {
-			recompute_beat_from_samples (sub_num);
-		}
-
-		end_changed(this); /* EMIT SIGNAL */
-		EndChanged(); /* EMIT SIGNAL */
+		emit_signal (End); /* EMIT SIGNAL*/
 
 		if (is_session_range()) {
-			Session::EndTimeChanged (old); /* EMIT SIGNAL */
+			Session::EndTimeChanged (old.samples()); /* EMIT SIGNAL */
 		}
 	}
 
-	assert (_end >= 0);
+	assert (_end.is_positive() || _end.is_zero());
 
 	return 0;
 }
 
 int
-Location::set (samplepos_t s, samplepos_t e, bool allow_beat_recompute, const uint32_t sub_num)
+Location::set (Temporal::timepos_t const & s, Temporal::timepos_t const & e)
 {
-	if (s < 0 || e < 0) {
-		return -1;
-	}
-
 	/* check validity */
 	if (((is_auto_punch() || is_auto_loop()) && s >= e) || (!is_mark() && s > e)) {
 		return -1;
@@ -356,11 +394,6 @@ Location::set (samplepos_t s, samplepos_t e, bool allow_beat_recompute, const ui
 		if (_start != s) {
 			_start = s;
 			_end = s;
-
-			if (allow_beat_recompute) {
-				recompute_beat_from_samples (sub_num);
-			}
-
 			start_change = true;
 			end_change = true;
 		}
@@ -371,79 +404,68 @@ Location::set (samplepos_t s, samplepos_t e, bool allow_beat_recompute, const ui
 	} else {
 
 		/* range locations must exceed a minimum duration */
-		if (e - s < Config->get_range_location_minimum()) {
+		if (s.distance (e) < Config->get_range_location_minimum()) {
 			return -1;
 		}
 
 		if (s != _start) {
 
-			samplepos_t const old = _start;
+			Temporal::timepos_t const old = _start;
 			_start = s;
-
-			if (allow_beat_recompute) {
-				recompute_beat_from_samples (sub_num);
-			}
-
 			start_change = true;
 
 			if (is_session_range ()) {
-				Session::StartTimeChanged (old); /* EMIT SIGNAL */
-				AudioFileSource::set_header_position_offset (s);
+				Session::StartTimeChanged (old.samples()); /* EMIT SIGNAL */
+				AudioFileSource::set_header_position_offset (s.samples());
 			}
 		}
 
 
 		if (e != _end) {
 
-			samplepos_t const old = _end;
+			Temporal::timepos_t const old = _end;
 			_end = e;
-
-			if (allow_beat_recompute) {
-				recompute_beat_from_samples (sub_num);
-			}
-
 			end_change = true;
 
 			if (is_session_range()) {
-				Session::EndTimeChanged (old); /* EMIT SIGNAL */
+				Session::EndTimeChanged (old.samples()); /* EMIT SIGNAL */
 			}
 		}
 
-		assert (_end >= 0);
+		assert (e.is_positive() || e.is_zero());
 	}
 
 	if (start_change && end_change) {
-		changed (this);
-		Changed ();
+		emit_signal (StartEnd); /* EMIT SIGNAL */
 	} else if (start_change) {
-		start_changed(this); /* EMIT SIGNAL */
-		StartChanged(); /* EMIT SIGNAL */
+		emit_signal (Start); /* EMIT SIGNAL */
 	} else if (end_change) {
-		end_changed(this); /* EMIT SIGNAL */
-		EndChanged(); /* EMIT SIGNAL */
+		emit_signal (End); /* EMIT SIGNAL*/
+	}
+
+	if (is_cue_marker()) {
+		emit_signal (Cue); /* EMIT SIGNAL */
 	}
 
 	return 0;
 }
 
 int
-Location::move_to (samplepos_t pos, const uint32_t sub_num)
+Location::move_to (Temporal::timepos_t const & pos)
 {
-	if (pos < 0) {
-		return -1;
-	}
-
 	if (_locked) {
 		return -1;
 	}
 
 	if (_start != pos) {
+		const timecnt_t len = _start.distance (_end);
 		_start = pos;
-		_end = _start + length();
-		recompute_beat_from_samples (sub_num);
+		_end = pos + len;
 
-		changed (this); /* EMIT SIGNAL */
-		Changed (); /* EMIT SIGNAL */
+		emit_signal (StartEnd); /* EMIT SIGNAL */
+		if (is_cue_marker()) {
+			emit_signal (Cue); /* EMIT SIGNAL */
+		}
 	}
 
 	assert (_start >= 0);
@@ -461,8 +483,7 @@ Location::set_hidden (bool yn, void*)
 	}
 
 	if (set_flag_internal (yn, IsHidden)) {
-		flags_changed (this); /* EMIT SIGNAL */
-		FlagsChanged ();
+		emit_signal (Flag); /* EMIT SIGNAL */
 	}
 }
 
@@ -470,8 +491,19 @@ void
 Location::set_cd (bool yn, void*)
 {
 	if (set_flag_internal (yn, IsCDMarker)) {
-		flags_changed (this); /* EMIT SIGNAL */
-		FlagsChanged ();
+		emit_signal (Flag); /* EMIT SIGNAL */
+	}
+}
+
+void
+Location::set_cue_id (int32_t cue_id)
+{
+	if (!is_cue_marker()) {
+		return;
+	}
+	if (_cue != cue_id) {
+		_cue = cue_id;
+		emit_signal (Cue); /* EMIT SIGNAL */
 	}
 }
 
@@ -479,8 +511,7 @@ void
 Location::set_is_range_marker (bool yn, void*)
 {
 	if (set_flag_internal (yn, IsRangeMarker)) {
-		flags_changed (this);
-		FlagsChanged (); /* EMIT SIGNAL */
+		emit_signal (Flag); /* EMIT SIGNAL */
 	}
 }
 
@@ -488,18 +519,16 @@ void
 Location::set_is_clock_origin (bool yn, void*)
 {
 	if (set_flag_internal (yn, IsClockOrigin)) {
-		flags_changed (this);
-		FlagsChanged (); /* EMIT SIGNAL */
+		emit_signal (Flag); /* EMIT SIGNAL */
 	}
 }
 
 void
 Location::set_skip (bool yn)
 {
-	if (is_range_marker() && length() > 0) {
+	if (is_range_marker() && length().is_positive()) {
 		if (set_flag_internal (yn, IsSkip)) {
-			flags_changed (this);
-			FlagsChanged ();
+			emit_signal (Flag); /* EMIT SIGNAL */
 		}
 	}
 }
@@ -507,10 +536,9 @@ Location::set_skip (bool yn)
 void
 Location::set_skipping (bool yn)
 {
-	if (is_range_marker() && is_skip() && length() > 0) {
+	if (is_range_marker() && is_skip() && length().is_positive()) {
 		if (set_flag_internal (yn, IsSkipping)) {
-			flags_changed (this);
-			FlagsChanged ();
+			emit_signal (Flag); /* EMIT SIGNAL */
 		}
 	}
 }
@@ -523,8 +551,7 @@ Location::set_auto_punch (bool yn, void*)
 	}
 
 	if (set_flag_internal (yn, IsAutoPunch)) {
-		flags_changed (this); /* EMIT SIGNAL */
-		FlagsChanged (); /* EMIT SIGNAL */
+		emit_signal (Flag); /* EMIT SIGNAL */
 	}
 }
 
@@ -536,8 +563,7 @@ Location::set_auto_loop (bool yn, void*)
 	}
 
 	if (set_flag_internal (yn, IsAutoLoop)) {
-		flags_changed (this); /* EMIT SIGNAL */
-		FlagsChanged (); /* EMIT SIGNAL */
+		emit_signal (Flag); /* EMIT SIGNAL */
 	}
 }
 
@@ -570,7 +596,6 @@ Location::set_mark (bool yn)
 	set_flag_internal (yn, IsMark);
 }
 
-
 XMLNode&
 Location::cd_info_node(const string & name, const string & value)
 {
@@ -584,7 +609,7 @@ Location::cd_info_node(const string & name, const string & value)
 
 
 XMLNode&
-Location::get_state ()
+Location::get_state () const
 {
 	XMLNode *node = new XMLNode ("Location");
 
@@ -598,14 +623,10 @@ Location::get_state ()
 	node->set_property ("name", name());
 	node->set_property ("start", start());
 	node->set_property ("end", end());
-	if (position_lock_style() == MusicTime) {
-		node->set_property ("start-beat", _start_beat);
-		node->set_property ("end-beat", _end_beat);
-	}
 	node->set_property ("flags", _flags);
 	node->set_property ("locked", _locked);
-	node->set_property ("position-lock-style", _position_lock_style);
 	node->set_property ("timestamp", _timestamp);
+	node->set_property ("cue", _cue);
 	if (_scene_change) {
 		node->add_child_nocopy (_scene_change->get_state());
 	}
@@ -655,6 +676,7 @@ Location::set_state (const XMLNode& node, int version)
 	}
 
 	node.get_property ("timestamp", _timestamp);
+	node.get_property ("cue", _cue);
 
 	Flags old_flags (_flags);
 
@@ -664,7 +686,7 @@ Location::set_state (const XMLNode& node, int version)
 	}
 
 	if (old_flags != _flags) {
-		FlagsChanged ();
+		emit_signal (Flag); /* EMIT SIGNAL */
 	}
 
 	if (!node.get_property ("locked", _locked)) {
@@ -690,95 +712,55 @@ Location::set_state (const XMLNode& node, int version)
 		cd_info[cd_name] = cd_value;
 	}
 
-	node.get_property ("position-lock-style", _position_lock_style);
-
 	XMLNode* scene_child = find_named_node (node, SceneChange::xml_node_name);
 
 	if (scene_child) {
 		_scene_change = SceneChange::factory (*scene_child, version);
 	}
 
-	if (position_lock_style() == AudioTime) {
-		recompute_beat_from_samples (0);
-	} else{
-		/* music */
-		if (!node.get_property ("start-beat", _start_beat) ||
-		    !node.get_property ("end-beat", _end_beat)) {
-			recompute_beat_from_samples (0);
-		}
-	}
+	emit_signal (StartEnd); /* EMIT SIGNAL */
 
-
-	changed (this); /* EMIT SIGNAL */
-	Changed (); /* EMIT SIGNAL */
-
-	assert (_start >= 0);
-	assert (_end >= 0);
+	assert (_start.is_positive() || _start.is_zero());
+	assert (_end.is_positive() || _end.is_zero());
 
 	return 0;
 }
 
 void
-Location::set_position_lock_style (PositionLockStyle ps)
+Location::set_position_time_domain (TimeDomain domain)
 {
-	if (_position_lock_style == ps) {
+	if (_start.time_domain() == domain) {
 		return;
 	}
 
-	_position_lock_style = ps;
+	_start.set_time_domain (domain);
+	_end.set_time_domain (domain);
 
-	if (ps == MusicTime) {
-		recompute_beat_from_samples (0);
-	}
-
-	position_lock_style_changed (this); /* EMIT SIGNAL */
-	PositionLockStyleChanged (); /* EMIT SIGNAL */
-}
-
-void
-Location::recompute_beat_from_samples (const uint32_t sub_num)
-{
-	_start_beat = _session.tempo_map().exact_beat_at_sample (_start, sub_num);
-	_end_beat = _session.tempo_map().exact_beat_at_sample (_end, sub_num);
-}
-
-void
-Location::recompute_samples_from_beat ()
-{
-	if (_position_lock_style != MusicTime) {
-		return;
-	}
-
-	TempoMap& map (_session.tempo_map());
-	set (map.sample_at_beat (_start_beat), map.sample_at_beat (_end_beat), false);
+	emit_signal (Domain); /* EMIT SIGNAL */
 }
 
 void
 Location::lock ()
 {
 	_locked = true;
-	lock_changed (this);
-	LockChanged ();
+	emit_signal (Lock); /* EMIT SIGNAL */
 }
 
 void
 Location::unlock ()
 {
 	_locked = false;
-	lock_changed (this);
-	LockChanged ();
+	emit_signal (Lock); /* EMIT SIGNAL */
 }
 
 void
 Location::set_scene_change (boost::shared_ptr<SceneChange>  sc)
 {
-        if (_scene_change != sc) {
-                _scene_change = sc;
-                _session.set_dirty ();
-
-                scene_changed (); /* EMIT SIGNAL */
-                SceneChangeChanged (); /* EMIT SIGNAL */
-        }
+	if (_scene_change != sc) {
+		_scene_change = sc;
+		_session.set_dirty ();
+		emit_signal (Scene); /* EMIT SIGNAL */
+	}
 }
 
 /*---------------------------------------------------------------------- */
@@ -1048,6 +1030,18 @@ Locations::add (Location *loc, bool make_current)
 
 	{
 		Glib::Threads::RWLock::WriterLock lm (_lock);
+
+		/* Do not allow multiple cue markers in the same location */
+
+		if (loc->is_cue_marker()) {
+			for (LocationList::iterator i = locations.begin(); i != locations.end(); ++i) {
+				if ((*i)->is_cue_marker() && (*i)->start() == loc->start()) {
+					locations.erase (i);
+					break;
+				}
+			}
+		}
+
 		locations.push_back (loc);
 
 		if (make_current) {
@@ -1060,7 +1054,9 @@ Locations::add (Location *loc, bool make_current)
 	if (loc->name().empty()) {
 		string new_name;
 
-		if (loc->is_mark()) {
+		if (loc->is_cue_marker()) {
+			next_available_name (new_name, _("cue"));
+		} else if (loc->is_mark()) {
 			next_available_name (new_name, _("mark"));
 		} else {
 			next_available_name (new_name, _("range"));
@@ -1077,10 +1073,14 @@ Locations::add (Location *loc, bool make_current)
 		Session::StartTimeChanged (0);
 		Session::EndTimeChanged (1);
 	}
+
+	if (loc->is_cue_marker()) {
+		Location::cue_change (loc); /* EMIT SIGNAL */
+	}
 }
 
 Location*
-Locations::add_range(samplepos_t start, samplepos_t end)
+Locations::add_range (timepos_t const & start, timepos_t const &  end)
 {
 	string name;
 	next_available_name(name, _("range"));
@@ -1144,6 +1144,10 @@ Locations::remove (Location *loc)
 
 		removed (loc); /* EMIT SIGNAL */
 
+		if (loc->is_cue_marker()) {
+			Location::cue_change (loc);
+		}
+
 		if (was_current) {
 			current_changed (0); /* EMIT SIGNAL */
 		}
@@ -1151,14 +1155,13 @@ Locations::remove (Location *loc)
 }
 
 XMLNode&
-Locations::get_state ()
+Locations::get_state () const
 {
 	XMLNode *node = new XMLNode ("Locations");
-	LocationList::iterator iter;
 	Glib::Threads::RWLock::ReaderLock lm (_lock);
 
-	for (iter = locations.begin(); iter != locations.end(); ++iter) {
-		node->add_child_nocopy ((*iter)->get_state ());
+	for (auto const & l : locations) {
+		node->add_child_nocopy (l->get_state ());
 	}
 
 	return *node;
@@ -1178,17 +1181,17 @@ Locations::set_state (const XMLNode& node, int version)
 	LocationList new_locations;
 
 	{
+		std::vector<Location::ChangeSuspender> lcs;
 		Glib::Threads::RWLock::WriterLock lm (_lock);
 
 		current_location = 0;
 
 		Location* session_range_location = 0;
 		if (version < 3000) {
-			session_range_location = new Location (_session, 0, 0, _("session"), Location::IsSessionRange, 0);
+			session_range_location = new Location (_session, timepos_t (Temporal::AudioTime), timepos_t (Temporal::AudioTime), _("session"), Location::IsSessionRange);
 			new_locations.push_back (session_range_location);
 		}
 
-
 		XMLNodeConstIterator niter;
 		for (niter = nlist.begin(); niter != nlist.end(); ++niter) {
 
@@ -1207,8 +1210,7 @@ Locations::set_state (const XMLNode& node, int version)
 				if (i != locations.end()) {
 					/* we can re-use an old Location object */
 					loc = *i;
-
-					// changed locations will be updated by Locations::changed signal
+					lcs.emplace_back (std::move (loc));
 					loc->set_state (**niter, version);
 				} else {
 					loc = new Location (_session, **niter);
@@ -1292,7 +1294,7 @@ Locations::set_state (const XMLNode& node, int version)
 }
 
 
-typedef std::pair<samplepos_t,Location*> LocationPair;
+typedef std::pair<timepos_t,Location*> LocationPair;
 
 struct LocationStartEarlierComparison
 {
@@ -1308,8 +1310,8 @@ struct LocationStartLaterComparison
 	}
 };
 
-samplepos_t
-Locations::first_mark_before (samplepos_t sample, bool include_special_ranges)
+timepos_t
+Locations::first_mark_before (timepos_t const & pos, bool include_special_ranges)
 {
 	vector<LocationPair> locs;
 	{
@@ -1335,20 +1337,20 @@ Locations::first_mark_before (samplepos_t sample, bool include_special_ranges)
 		if (!include_special_ranges && ((*i).second->is_auto_loop() || (*i).second->is_auto_punch())) {
 			continue;
 		}
-		if ((*i).first < sample) {
+		if ((*i).first < pos) {
 			return (*i).first;
 		}
 	}
 
-	return -1;
+	return timepos_t::max (pos.time_domain());
 }
 
 Location*
-Locations::mark_at (samplepos_t pos, samplecnt_t slop) const
+Locations::mark_at (timepos_t const & pos, timecnt_t const & slop) const
 {
 	Location* closest = 0;
-	sampleoffset_t mindelta = max_samplepos;
-	sampleoffset_t delta;
+	timecnt_t mindelta = timecnt_t::max (pos.time_domain());
+	timecnt_t delta;
 
 	/* locations are not necessarily stored in linear time order so we have
 	 * to iterate across all of them to find the one closest to a give point.
@@ -1359,12 +1361,12 @@ Locations::mark_at (samplepos_t pos, samplecnt_t slop) const
 
 		if ((*i)->is_mark()) {
 			if (pos > (*i)->start()) {
-				delta = pos - (*i)->start();
+				delta = (*i)->start().distance (pos);
 			} else {
-				delta = (*i)->start() - pos;
+				delta = pos.distance ((*i)->start());
 			}
 
-			if (slop == 0 && delta == 0) {
+			if (slop.is_zero() && delta.is_zero()) {
 				/* special case: no slop, and direct hit for position */
 				return *i;
 			}
@@ -1381,13 +1383,14 @@ Locations::mark_at (samplepos_t pos, samplecnt_t slop) const
 	return closest;
 }
 
-samplepos_t
-Locations::first_mark_after (samplepos_t sample, bool include_special_ranges)
+timepos_t
+Locations::first_mark_after (timepos_t const & pos, bool include_special_ranges)
 {
 	vector<LocationPair> locs;
 
 	{
 		Glib::Threads::RWLock::ReaderLock lm (_lock);
+
 		for (LocationList::iterator i = locations.begin(); i != locations.end(); ++i) {
 			locs.push_back (make_pair ((*i)->start(), (*i)));
 			if (!(*i)->is_mark()) {
@@ -1408,25 +1411,25 @@ Locations::first_mark_after (samplepos_t sample, bool include_special_ranges)
 		if (!include_special_ranges && ((*i).second->is_auto_loop() || (*i).second->is_auto_punch())) {
 			continue;
 		}
-		if ((*i).first > sample) {
+		if ((*i).first > pos) {
 			return (*i).first;
 		}
 	}
 
-	return -1;
+	return timepos_t::max (pos.time_domain());
 }
 
 /** Look for the `marks' (either locations which are marks, or start/end points of range markers) either
  *  side of a sample.  Note that if sample is exactly on a `mark', that mark will not be considered for returning
  *  as before/after.
- *  @param sample Frame to look for.
- *  @param before Filled in with the position of the last `mark' before `sample' (or max_samplepos if none exists)
- *  @param after Filled in with the position of the next `mark' after `sample' (or max_samplepos if none exists)
+ *  @param pos position to be used
+ *  @param before Filled in with the position of the last `mark' before `pos' (or max_timepos if none exists)
+ *  @param after Filled in with the position of the next `mark' after `pos' (or max_timepos if none exists)
  */
 void
-Locations::marks_either_side (samplepos_t const sample, samplepos_t& before, samplepos_t& after) const
+Locations::marks_either_side (timepos_t const & pos, timepos_t& before, timepos_t& after) const
 {
-	before = after = max_samplepos;
+	before = after = timepos_t::max (pos.time_domain());
 
 	LocationList locs;
 
@@ -1437,23 +1440,23 @@ Locations::marks_either_side (samplepos_t const sample, samplepos_t& before, sam
 
 	/* Get a list of positions; don't store any that are exactly on our requested position */
 
-	std::list<samplepos_t> positions;
+	std::list<timepos_t> positions;
 
 	for (LocationList::const_iterator i = locs.begin(); i != locs.end(); ++i) {
-		if (((*i)->is_auto_loop() || (*i)->is_auto_punch()) || (*i)->is_xrun()) {
+		if (((*i)->is_auto_loop() || (*i)->is_auto_punch()) || (*i)->is_xrun() || (*i)->is_cue_marker()) {
 			continue;
 		}
 
 		if (!(*i)->is_hidden()) {
 			if ((*i)->is_mark ()) {
-				if ((*i)->start() != sample) {
+				if ((*i)->start() != pos) {
 					positions.push_back ((*i)->start ());
 				}
 			} else {
-				if ((*i)->start() != sample) {
+				if ((*i)->start() != pos) {
 					positions.push_back ((*i)->start ());
 				}
-				if ((*i)->end() != sample) {
+				if ((*i)->end() != pos) {
 					positions.push_back ((*i)->end ());
 				}
 			}
@@ -1466,8 +1469,9 @@ Locations::marks_either_side (samplepos_t const sample, samplepos_t& before, sam
 
 	positions.sort ();
 
-	std::list<samplepos_t>::iterator i = positions.begin ();
-	while (i != positions.end () && *i < sample) {
+	std::list<timepos_t>::iterator i = positions.begin ();
+
+	while (i != positions.end () && *i < pos) {
 		++i;
 	}
 
@@ -1567,7 +1571,7 @@ Locations::get_location_by_id(PBD::ID id)
 }
 
 void
-Locations::find_all_between (samplepos_t start, samplepos_t end, LocationList& ll, Location::Flags flags)
+Locations::find_all_between (timepos_t const & start, timepos_t const & end, LocationList& ll, Location::Flags flags)
 {
 	Glib::Threads::RWLock::ReaderLock lm (_lock);
 	for (LocationList::const_iterator i = locations.begin(); i != locations.end(); ++i) {
@@ -1579,10 +1583,10 @@ Locations::find_all_between (samplepos_t start, samplepos_t end, LocationList& l
 }
 
 Location *
-Locations::range_starts_at(samplepos_t pos, samplecnt_t slop, bool incl) const
+Locations::range_starts_at (timepos_t const & pos, timecnt_t const & slop, bool incl) const
 {
 	Location *closest = 0;
-	sampleoffset_t mindelta = max_samplepos;
+	timecnt_t mindelta = timecnt_t (pos.time_domain());
 
 	Glib::Threads::RWLock::ReaderLock lm (_lock);
 	for (LocationList::const_iterator i = locations.begin(); i != locations.end(); ++i) {
@@ -1594,9 +1598,9 @@ Locations::range_starts_at(samplepos_t pos, samplecnt_t slop, bool incl) const
 			continue;
 		}
 
-		sampleoffset_t delta = std::abs((double)(pos - (*i)->start()));
+		timecnt_t delta = (*i)->start().distance (pos).abs ();
 
-		if (delta == 0) {
+		if (delta.is_zero()) {
 			return *i;
 		}
 
@@ -1614,32 +1618,43 @@ Locations::range_starts_at(samplepos_t pos, samplecnt_t slop, bool incl) const
 }
 
 void
-Locations::ripple (samplepos_t at, samplecnt_t distance, bool include_locked, bool notify)
+Locations::ripple (timepos_t const & at, timecnt_t const & distance, bool include_locked, bool notify)
 {
-	Glib::Threads::RWLock::WriterLock lm (_lock);
+	LocationList copy;
 
-	for (LocationList::iterator i = locations.begin(); i != locations.end(); ++i) {
+	{
+		Glib::Threads::RWLock::WriterLock lm (_lock);
+		copy = locations;
+	}
 
-		if (!include_locked && (*i)->locked()) {
+	for (LocationList::iterator i = copy.begin(); i != copy.end(); ++i) {
+
+		if ( (*i)->is_session_range() || (*i)->is_auto_punch() || (*i)->is_auto_loop()  ) {
 			continue;
 		}
 
 		bool locked = (*i)->locked();
 
 		if (locked) {
+			if (!include_locked) {
+				continue;
+			}
+		} else {
 			(*i)->unlock ();
 		}
 
 		if ((*i)->start() >= at) {
-			(*i)->set_start ((*i)->start() - distance);
+			(*i)->set_start ((*i)->start() + distance);
 
 			if (!(*i)->is_mark()) {
-				(*i)->set_end ((*i)->end() - distance);
+				(*i)->set_end ((*i)->end() + distance);
 			}
+		} else if ((*i)->end() >= at) {
+			(*i)->set_end ((*i)->end() + distance);
 		}
 
 		if (locked) {
-			(*i)->locked();
+			(*i)->lock();
 		}
 	}
 
@@ -1647,3 +1662,54 @@ Locations::ripple (samplepos_t at, samplecnt_t distance, bool include_locked, bo
 		changed(); /* EMIT SIGNAL */
 	}
 }
+
+void
+Locations::clear_cue_markers (samplepos_t start, samplepos_t end)
+{
+	TempoMap::SharedPtr tmap (TempoMap::use());
+	Temporal::Beats sb;
+	Temporal::Beats eb;
+	bool have_beats = false;
+	vector<Location*> r;
+
+
+	{
+		Glib::Threads::RWLock::WriterLock lm (_lock);
+
+		for (LocationList::iterator i = locations.begin(); i != locations.end(); ) {
+
+			if ((*i)->is_cue_marker()) {
+				Location* l (*i);
+
+				if (l->start().time_domain() == AudioTime) {
+					samplepos_t when = l->start().samples();
+					if (when >= start && when < end) {
+						i = locations.erase (i);
+						r.push_back (l);
+						continue;
+					}
+				} else {
+					if (!have_beats) {
+						sb = tmap->quarters_at (timepos_t (start));
+						eb = tmap->quarters_at (timepos_t (end));
+						have_beats = true;
+					}
+
+					Temporal::Beats when = l->start().beats();
+					if (when >= sb && when < eb) {
+						r.push_back (l);
+						i = locations.erase (i);
+						continue;
+					}
+				}
+			}
+
+			++i;
+		}
+	} /* end lock scope */
+
+	for (auto & l : r) {
+		removed (l); /* EMIT SIGNAL */
+		delete l;
+	}
+}
