@@ -28,7 +28,6 @@
 #include "evoral/EventList.h"
 #include "evoral/Control.h"
 
-#include "ardour/beats_samples_converter.h"
 #include "ardour/debug.h"
 #include "ardour/midi_model.h"
 #include "ardour/midi_playlist.h"
@@ -36,6 +35,7 @@
 #include "ardour/midi_source.h"
 #include "ardour/midi_state_tracker.h"
 #include "ardour/region_factory.h"
+#include "ardour/region_sorters.h"
 #include "ardour/rt_midibuffer.h"
 #include "ardour/session.h"
 #include "ardour/tempo.h"
@@ -78,8 +78,8 @@ MidiPlaylist::MidiPlaylist (boost::shared_ptr<const MidiPlaylist> other, string
 }
 
 MidiPlaylist::MidiPlaylist (boost::shared_ptr<const MidiPlaylist> other,
-                            samplepos_t                            start,
-                            samplecnt_t                            dur,
+                            timepos_t  const &                    start,
+                            timepos_t  const &                    dur,
                             string                                name,
                             bool                                  hidden)
 	: Playlist (other, start, dur, name, hidden)
@@ -91,20 +91,21 @@ MidiPlaylist::~MidiPlaylist ()
 {
 }
 
-template<typename Time>
+template <typename Time>
 struct EventsSortByTimeAndType {
-    bool operator() (const Evoral::Event<Time>* a, const Evoral::Event<Time>* b) {
-	    if (a->time() == b->time()) {
-		    if (parameter_is_midi ((AutomationType)a->event_type()) &&
-		        parameter_is_midi ((AutomationType)b->event_type())) {
-			    /* negate return value since we must return whether
-			     * or not a should sort before b, not b before a
-			     */
-			    return !MidiBuffer::second_simultaneous_midi_byte_is_first (a->buffer()[0], b->buffer()[0]);
-		    }
-	    }
-	    return a->time() < b->time();
-    }
+	bool operator() (const Evoral::Event<Time>* a, const Evoral::Event<Time>* b)
+	{
+		if (a->time () == b->time ()) {
+			if (a->is_midi () && b->is_midi ()) {
+				/* negate return value since we must return whether
+				 * or not a should sort before b, not b before a
+				 */
+				return !MidiBuffer::second_simultaneous_midi_byte_is_first (a->buffer ()[0], b->buffer ()[0]);
+			}
+			return a->type () < b->type ();
+		}
+		return a->time () < b->time ();
+	}
 };
 
 void
@@ -196,14 +197,14 @@ MidiPlaylist::destroy_region (boost::shared_ptr<Region> region)
 	return changed;
 }
 void
-MidiPlaylist::_split_region (boost::shared_ptr<Region> region, const MusicSample& playlist_position, ThawList& thawlist)
+MidiPlaylist::_split_region (boost::shared_ptr<Region> region, timepos_t const & playlist_position, ThawList& thawlist)
 {
-	if (!region->covers (playlist_position.sample)) {
+	if (!region->covers (playlist_position)) {
 		return;
 	}
 
-	if (region->position() == playlist_position.sample ||
-	    region->last_sample() == playlist_position.sample) {
+	if (region->position() == playlist_position ||
+	    region->nt_last() == playlist_position) {
 		return;
 	}
 
@@ -218,22 +219,16 @@ MidiPlaylist::_split_region (boost::shared_ptr<Region> region, const MusicSample
 
 	string before_name;
 	string after_name;
-	const double before_qn = _session.tempo_map().exact_qn_at_sample (playlist_position.sample, playlist_position.division) - region->quarter_note();
-	const double after_qn = mr->length_beats() - before_qn;
-	MusicSample before (playlist_position.sample - region->position(), playlist_position.division);
-	MusicSample after (region->length() - before.sample, playlist_position.division);
 
-	/* split doesn't change anything about length, so don't try to splice */
-	bool old_sp = _splicing;
-	_splicing = true;
+	const timecnt_t before = region->position().distance (playlist_position);
+	const timecnt_t after = region->length() - before;
 
 	RegionFactory::region_name (before_name, region->name(), false);
 
 	{
 		PropertyList plist;
 
-		plist.add (Properties::length, before.sample);
-		plist.add (Properties::length_beats, before_qn);
+		plist.add (Properties::length, before);
 		plist.add (Properties::name, before_name);
 		plist.add (Properties::left_of_split, true);
 		plist.add (Properties::layering_index, region->layering_index ());
@@ -243,7 +238,7 @@ MidiPlaylist::_split_region (boost::shared_ptr<Region> region, const MusicSample
 		   since it supplies that offset to the Region constructor, which
 		   is necessary to get audio region gain envelopes right.
 		*/
-		left = RegionFactory::create (region, MusicSample (0, 0), plist, true, &thawlist);
+	        left = RegionFactory::create (region, plist, true, &thawlist);
 	}
 
 	RegionFactory::region_name (after_name, region->name(), false);
@@ -251,8 +246,7 @@ MidiPlaylist::_split_region (boost::shared_ptr<Region> region, const MusicSample
 	{
 		PropertyList plist;
 
-		plist.add (Properties::length, after.sample);
-		plist.add (Properties::length_beats, after_qn);
+		plist.add (Properties::length, after);
 		plist.add (Properties::name, after_name);
 		plist.add (Properties::right_of_split, true);
 		plist.add (Properties::layering_index, region->layering_index ());
@@ -262,12 +256,10 @@ MidiPlaylist::_split_region (boost::shared_ptr<Region> region, const MusicSample
 		right = RegionFactory::create (region, before, plist, true, &thawlist);
 	}
 
-	add_region_internal (left, region->position(), thawlist, 0, region->quarter_note(), true);
-	add_region_internal (right, region->position() + before.sample, thawlist, before.division, region->quarter_note() + before_qn, true);
+	add_region_internal (left, region->position(), thawlist);
+	add_region_internal (right, region->position() + before, thawlist);
 
 	remove_region_internal (region, thawlist);
-
-	_splicing = old_sp;
 }
 
 set<Evoral::Parameter>
@@ -297,13 +289,11 @@ MidiPlaylist::contained_automation()
 void
 MidiPlaylist::render (MidiChannelFilter* filter)
 {
-	typedef pair<MidiStateTracker*,samplepos_t> TrackerInfo;
-
 	Playlist::RegionReadLock rl (this);
 
 	DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("---- MidiPlaylist::render (regions: %1)-----\n", regions.size()));
 
-	std::vector< boost::shared_ptr<Region> > regs;
+	std::list<boost::shared_ptr<MidiRegion>> regs;
 
 	for (RegionList::iterator i = regions.begin(); i != regions.end(); ++i) {
 
@@ -317,14 +307,13 @@ MidiPlaylist::render (MidiChannelFilter* filter)
 			continue;
 		}
 
-		regs.push_back (*i);
-	}
+		boost::shared_ptr<MidiRegion> mr = boost::dynamic_pointer_cast<MidiRegion> (*i);
+		if (!mr) {
+			continue;
+		}
 
-	/* If we are reading from a single region, we can read directly into _rendered.  Otherwise,
-	   we read into a temporarily list, sort it, then write that to _rendered.
-	*/
-	Evoral::EventList<samplepos_t>  evlist;
-	Evoral::EventSink<samplepos_t>* tgt;
+		regs.push_back (mr);
+	}
 
 	/* RAII */
 	RTMidiBuffer::WriteProtectRender wpr (_rendered);
@@ -332,49 +321,118 @@ MidiPlaylist::render (MidiChannelFilter* filter)
 	if (regs.empty()) {
 		wpr.acquire ();
 		_rendered.clear ();
-	} else {
+		DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("---- End MidiPlaylist::render, events: %1\n", _rendered.size()));
+		return;
+	}
 
-		if (regs.size() == 1) {
-			tgt = &_rendered;
-			wpr.acquire ();
-			_rendered.clear ();
-		} else {
-			tgt = &evlist;
-		}
+	if (regs.size() == 1) {
+		wpr.acquire ();
+		_rendered.clear ();
+		boost::shared_ptr<MidiRegion> mr = regs.front ();
+		DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("render from %1\n", mr->name()));
+		mr->render (_rendered, 0, _note_mode, filter);
+		DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("---- End MidiPlaylist::render, events: %1\n", _rendered.size()));
+		return;
+	}
 
-		DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("\t%1 regions to read, direct: %2\n", regs.size(), (regs.size() == 1)));
+	RegionSortByLayer cmp;
+	regs.sort (cmp);
 
-		for (vector<boost::shared_ptr<Region> >::iterator i = regs.begin(); i != regs.end(); ++i) {
+	bool all_transparent = true;
 
-			boost::shared_ptr<MidiRegion> mr = boost::dynamic_pointer_cast<MidiRegion>(*i);
-
-			if (!mr) {
-				continue;
-			}
-
-			DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("render from %1\n", mr->name()));
-			mr->render (*tgt, 0, _note_mode, filter);
-		}
-
-		if (!evlist.empty()) {
-			/* We've read from multiple regions into evlist, sort the event list by time. */
-			EventsSortByTimeAndType<samplepos_t> cmp;
-			evlist.sort (cmp);
-
-			/* Copy ordered events from event list to _rendered. */
-
-			wpr.acquire ();
-			_rendered.clear ();
-
-			for (Evoral::EventList<samplepos_t>::iterator e = evlist.begin(); e != evlist.end(); ++e) {
-				Evoral::Event<samplepos_t>* ev (*e);
-				_rendered.write (ev->time(), ev->event_type(), ev->size(), ev->buffer());
-				delete ev;
-			}
+	/* skip bottom-most region, transparency is irrelevant */
+	for (auto i = ++regs.begin(); i != regs.end(); ++i) {
+		if ((*i)->opaque ()) {
+			all_transparent = false;
+			break;
 		}
 	}
 
-	/* no need to release - RAII with WriteProtectRender takes care of it */
+	Evoral::EventList<samplepos_t> evlist;
+
+	if (all_transparent) {
+
+		DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("\t%1 regions to read\n", regs.size()));
+
+		for (auto i = regs.rbegin(); i != regs.rend(); ++i) {
+			boost::shared_ptr<MidiRegion> mr = *i;
+			DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("render from %1\n", mr->name()));
+			mr->render (evlist, 0, _note_mode, filter);
+		}
+		EventsSortByTimeAndType<samplepos_t> cmp;
+		evlist.sort (cmp);
+
+	} else {
+
+		DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("\t%1 layered regions to read\n", regs.size()));
+
+		bool top = true;
+		std::vector<samplepos_t> bounds;
+		EventsSortByTimeAndType<samplepos_t> cmp;
+
+		/* iterate, top-most region first */
+		for (auto i = regs.rbegin(); i != regs.rend(); ++i) {
+			boost::shared_ptr<MidiRegion> mr = *i;
+			DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("maybe render from %1\n", mr->name()));
+
+			if (top) {
+				/* render topmost region as-is */
+				mr->render (evlist, 0, _note_mode, filter);
+				top = false;
+			} else {
+				Evoral::EventList<samplepos_t> tmp;
+				mr->render (tmp, 0, _note_mode, filter);
+
+				/* insert region-bound markers of opaque regions above */
+				for (auto const& p : bounds) {
+					tmp.write (p, Evoral::NO_EVENT, 0, 0);
+				}
+				tmp.sort (cmp);
+
+				MidiStateTracker mtr;
+				Evoral::EventList<samplepos_t> const slist (evlist);
+
+				for (Evoral::EventList<samplepos_t>::iterator e = tmp.begin(); e != tmp.end(); ++e) {
+					Evoral::Event<samplepos_t>* ev (*e);
+					timepos_t t (ev->time());
+
+					if (ev->event_type () == Evoral::NO_EVENT) {
+						/* reached region bound of an opaque region above this region. */
+						mtr.resolve_state (evlist, slist, ev->time());
+					} else if (region_is_audible_at (mr, t)) {
+						/* no opaque region above this event */
+						uint8_t* evbuf = ev->buffer();
+						if (3 == ev->size() && (evbuf[0] & 0xf0) == MIDI_CMD_NOTE_OFF && !mtr.active (evbuf[1], evbuf[0] & 0x0f)) {
+							; /* skip note off */
+						} else {
+							evlist.write (ev->time(), ev->event_type(), ev->size(), evbuf);
+							mtr.track (evbuf);
+						}
+					} else {
+						/* there is an opaque region above this event, skip this event. */
+					}
+					delete ev;
+				}
+			}
+
+			if (mr->opaque ()) {
+				bounds.push_back (mr->position ().samples ());
+			}
+
+			EventsSortByTimeAndType<samplepos_t> cmp;
+			evlist.sort (cmp);
+		}
+	}
+
+	wpr.acquire ();
+	_rendered.clear ();
+
+	/* Copy ordered events from event list to _rendered. */
+	for (Evoral::EventList<samplepos_t>::iterator e = evlist.begin(); e != evlist.end(); ++e) {
+		Evoral::Event<samplepos_t>* ev (*e);
+		_rendered.write (ev->time(), ev->event_type(), ev->size(), ev->buffer());
+		delete ev;
+	}
 
 	DEBUG_TRACE (DEBUG::MidiPlaylistIO, string_compose ("---- End MidiPlaylist::render, events: %1\n", _rendered.size()));
 }
@@ -384,3 +442,56 @@ MidiPlaylist::rendered ()
 {
 	return &_rendered;
 }
+
+boost::shared_ptr<Region>
+MidiPlaylist::combine (RegionList const & rl)
+{
+	RegionWriteLock rwl (this, true);
+
+	if (rl.size() < 2) {
+		return boost::shared_ptr<Region> ();
+	}
+
+	RegionList sorted (rl);
+	sorted.sort (RegionSortByLayerAndPosition());
+
+	boost::shared_ptr<Region> first = sorted.front();
+	RegionList::const_iterator i = sorted.begin();
+	++i;
+
+	timepos_t earliest (timepos_t::max (Temporal::BeatTime));
+	timepos_t latest (Temporal::BeatTime);
+
+	for (auto const & r : rl) {
+		assert (boost::dynamic_pointer_cast<MidiRegion> (r));
+		if (r->position() < earliest) {
+			earliest = r->position();
+		}
+		if (r->end() > latest) {
+			latest = r->end();
+		}
+	}
+
+	boost::shared_ptr<MidiRegion> new_region = boost::dynamic_pointer_cast<MidiRegion> (RegionFactory::create (first, true, true, &rwl.thawlist));
+
+	timepos_t pos (first->position());
+
+	remove_region_internal (first, rwl.thawlist);
+
+	while (i != sorted.end()) {
+		new_region->merge (boost::dynamic_pointer_cast<MidiRegion> (*i));
+		remove_region_internal (*i, rwl.thawlist);
+		++i;
+	}
+
+	new_region->set_length (earliest.distance (latest));
+
+	add_region_internal (new_region, pos, rwl.thawlist);
+
+	return new_region;
+}
+
+void
+MidiPlaylist::uncombine (boost::shared_ptr<Region> r)
+{
+}
