@@ -257,7 +257,7 @@ RouteGroup::set_rgba (uint32_t color) {
 }
 
 XMLNode&
-RouteGroup::get_state ()
+RouteGroup::get_state () const
 {
 	XMLNode *node = new XMLNode ("RouteGroup");
 
@@ -556,7 +556,7 @@ RouteGroup::make_subgroup (bool aux, Placement placement)
 		/* use master bus etc. to determine default nouts.
 		 *
 		 * (since tracks can't have fewer outs than ins,
-		 * "nin" currently defines the number of outpus if nin > 2)
+		 * "nin" currently defines the number of outputs if nin > 2)
 		 */
 		rl = _session.new_audio_route (nin, 2, 0, 1, string(), PresentationInfo::AudioBus, PresentationInfo::max_order);
 	} catch (...) {
