@@ -86,7 +86,7 @@ SlavableAutomationControl::get_value_locked() const
 	/* read or write masters lock must be held */
 
 	if (_masters.empty()) {
-		return Control::get_double (false, _session.transport_sample());
+		return Control::get_double ();
 	}
 
 	if (_desc.toggled) {
@@ -94,30 +94,24 @@ SlavableAutomationControl::get_value_locked() const
 		 * enabled, this slave is enabled. So check our own value
 		 * first, because if we are enabled, we can return immediately.
 		 */
-		if (Control::get_double (false, _session.transport_sample())) {
+		if (Control::get_double ()) {
 			return _desc.upper;
 		}
 	}
 
-	return Control::get_double() * get_masters_value_locked ();
+	return Control::get_double () * get_masters_value_locked ();
 }
 
 /** Get the current effective `user' value based on automation state */
 double
 SlavableAutomationControl::get_value() const
 {
-	bool from_list = _list && boost::dynamic_pointer_cast<AutomationList>(_list)->automation_playback();
-
 	Glib::Threads::RWLock::ReaderLock lm (master_lock);
-	if (!from_list) {
-		if (!_masters.empty() && automation_write ()) {
-			/* writing automation takes the fader value as-is, factor out the master */
-			return Control::user_double ();
-		}
-		return get_value_locked ();
-	} else {
-		return Control::get_double (true, _session.transport_sample()) * get_masters_value_locked();
+	if (!_masters.empty() && automation_write ()) {
+		/* writing automation takes the fader value as-is, factor out the master */
+		return Control::get_double ();
 	}
+	return get_value_locked ();
 }
 
 bool
@@ -133,7 +127,7 @@ SlavableAutomationControl::get_masters_curve_locked (samplepos_t, samplepos_t, f
 }
 
 bool
-SlavableAutomationControl::masters_curve_multiply (samplepos_t start, samplepos_t end, float* vec, samplecnt_t veclen) const
+SlavableAutomationControl::masters_curve_multiply (timepos_t const & start, timepos_t const & end, float* vec, samplecnt_t veclen) const
 {
 	gain_t* scratch = _session.scratch_automation_buffer ();
 	bool from_list = _list && boost::dynamic_pointer_cast<AutomationList>(_list)->automation_playback();
@@ -338,7 +332,7 @@ SlavableAutomationControl::remove_master (boost::shared_ptr<AutomationControl> m
 
 	pre_remove_master (m);
 
-	const double old_val = AutomationControl::get_double();
+	const double old_val = AutomationControl::get_double ();
 
 	bool update_value = false;
 	double master_ratio = 0;
@@ -368,7 +362,8 @@ SlavableAutomationControl::remove_master (boost::shared_ptr<AutomationControl> m
 		double new_val = old_val * master_ratio;
 
 		if (old_val != new_val) {
-			AutomationControl::set_double (new_val, Controllable::NoGroup);
+			timepos_t foo;
+			AutomationControl::set_double (new_val, foo, Controllable::NoGroup);
 		}
 
 		/* ..and update automation */
@@ -405,7 +400,7 @@ SlavableAutomationControl::clear_masters ()
 		return;
 	}
 
-	const double old_val = AutomationControl::get_double();
+	const double old_val = AutomationControl::get_double ();
 
 	ControlList masters;
 	bool update_value = false;
@@ -441,7 +436,8 @@ SlavableAutomationControl::clear_masters ()
 			double new_val = old_val * master_ratio;
 
 			if (old_val != new_val) {
-				AutomationControl::set_double (new_val, Controllable::NoGroup);
+				timepos_t foo;
+				AutomationControl::set_double (new_val, foo, Controllable::NoGroup);
 			}
 
 			/* ..and update automation */
@@ -471,7 +467,7 @@ SlavableAutomationControl::clear_masters ()
 }
 
 bool
-SlavableAutomationControl::find_next_event_locked (double now, double end, Evoral::ControlEvent& next_event) const
+SlavableAutomationControl::find_next_event_locked (timepos_t const & now, timepos_t const & end, Evoral::ControlEvent& next_event) const
 {
 	if (_masters.empty()) {
 		return false;
@@ -537,7 +533,7 @@ SlavableAutomationControl::automation_run (samplepos_t start, pframes_t nframes)
 
 	assert (_list);
 	bool valid = false;
-	double val = _list->rt_safe_eval (start, valid);
+	double val = _list->rt_safe_eval (timepos_t (start), valid);
 	if (!valid) {
 		return;
 	}
@@ -571,7 +567,7 @@ SlavableAutomationControl::boolean_automation_run_locked (samplepos_t start, pfr
 		}
 		boost::shared_ptr<const Evoral::ControlList> alist (ac->list());
 		bool valid = false;
-		const bool yn = alist->rt_safe_eval (start, valid) >= 0.5;
+		const bool yn = alist->rt_safe_eval (timepos_t (start), valid) >= 0.5;
 		if (!valid) {
 			continue;
 		}
@@ -613,6 +609,20 @@ SlavableAutomationControl::slaved () const
 	return !_masters.empty();
 }
 
+std::set<boost::shared_ptr<AutomationControl>>
+SlavableAutomationControl::masters () const
+{
+	std::set<boost::shared_ptr<AutomationControl>> rv;
+	Glib::Threads::RWLock::ReaderLock lm (master_lock);
+	for (auto const& m : _masters) {
+		boost::shared_ptr<AutomationControl> ac (m.second.master ());
+		if (ac) {
+			rv.insert (ac);
+		}
+	}
+	return rv;
+}
+
 int
 SlavableAutomationControl::MasterRecord::set_state (XMLNode const& n, int)
 {
@@ -654,7 +664,7 @@ SlavableAutomationControl::use_saved_master_ratios ()
 
 
 XMLNode&
-SlavableAutomationControl::get_state ()
+SlavableAutomationControl::get_state () const
 {
 	XMLNode& node (AutomationControl::get_state());
 
@@ -664,7 +674,7 @@ SlavableAutomationControl::get_state ()
 		Glib::Threads::RWLock::ReaderLock lm (master_lock);
 		if (!_masters.empty()) {
 			XMLNode* masters_node = new XMLNode (X_("masters"));
-			for (Masters::iterator mr = _masters.begin(); mr != _masters.end(); ++mr) {
+			for (Masters::const_iterator mr = _masters.begin(); mr != _masters.end(); ++mr) {
 				XMLNode* mnode = new XMLNode (X_("master"));
 				mnode->set_property (X_("id"), mr->second.master()->id());
 
