@@ -43,6 +43,8 @@
 #include "evoral/Control.h"
 #include "evoral/SMF.h"
 
+#include "temporal/tempo.h"
+
 #include "ardour/debug.h"
 #include "ardour/midi_channel_filter.h"
 #include "ardour/midi_model.h"
@@ -67,10 +69,7 @@ SMFSource::SMFSource (Session& s, const string& path, Source::Flag flags)
 	, FileSource(s, DataType::MIDI, path, string(), flags)
 	, Evoral::SMF()
 	, _open (false)
-	, _last_ev_time_beats(0.0)
 	, _last_ev_time_samples(0)
-	, _smf_last_read_end (0)
-	, _smf_last_read_time (0)
 {
 	/* note that origin remains empty */
 
@@ -83,17 +82,21 @@ SMFSource::SMFSource (Session& s, const string& path, Source::Flag flags)
 
 	_flags = Source::Flag (_flags | Empty);
 
-	/* file is not opened until write */
-
-	if (flags & Writable) {
-		return;
+	if (_flags & Writable) {
+		if (open_for_write ()) {
+			throw failed_constructor ();
+		}
+		/* no fd left open here */
+	} else {
+		if (open (_path, 1, false)) {
+			throw failed_constructor ();
+		}
+		_open = true;
 	}
 
-	if (open (_path)) {
-		throw failed_constructor ();
-	}
+	/* there's no data to load into the model but create it anyway */
 
-	_open = true;
+	_model = boost::shared_ptr<MidiModel> (new MidiModel (*this));
 }
 
 /** Constructor used for external-to-session files.  File must exist. */
@@ -103,10 +106,7 @@ SMFSource::SMFSource (Session& s, const string& path)
 	, FileSource(s, DataType::MIDI, path, string(), Source::Flag (0))
 	, Evoral::SMF()
 	, _open (false)
-	, _last_ev_time_beats(0.0)
 	, _last_ev_time_samples(0)
-	, _smf_last_read_end (0)
-	, _smf_last_read_time (0)
 {
 	/* note that origin remains empty */
 
@@ -117,16 +117,14 @@ SMFSource::SMFSource (Session& s, const string& path)
         assert (Glib::file_test (_path, Glib::FILE_TEST_EXISTS));
 	existence_check ();
 
-	if (_flags & Writable) {
-		/* file is not opened until write */
-		return;
-	}
-
-	if (open (_path)) {
+	if (open (_path, 1, false)) {
 		throw failed_constructor ();
 	}
 
 	_open = true;
+
+	/* no lock required since we do not actually exist yet */
+	load_model_unlocked (true);
 }
 
 /** Constructor used for existing internal-to-session files. */
@@ -135,10 +133,7 @@ SMFSource::SMFSource (Session& s, const XMLNode& node, bool must_exist)
 	, MidiSource(s, node)
 	, FileSource(s, node, must_exist)
 	, _open (false)
-	, _last_ev_time_beats(0.0)
 	, _last_ev_time_samples(0)
-	, _smf_last_read_end (0)
-	, _smf_last_read_time (0)
 {
 	if (set_state(node, Stateful::loading_state_version)) {
 		throw failed_constructor ();
@@ -179,17 +174,20 @@ SMFSource::SMFSource (Session& s, const XMLNode& node, bool must_exist)
 	if (!(_flags & Source::Empty)) {
 		assert (Glib::file_test (_path, Glib::FILE_TEST_EXISTS));
 		existence_check ();
+		if (open (_path, 1, false)) {
+			throw failed_constructor ();
+		}
+		_open = true;
 	} else {
 		assert (_flags & Source::Writable);
-		/* file will be opened on write */
-		return;
+		if (open_for_write ()) {
+			throw failed_constructor ();
+		}
+		/* no fd left open here */
 	}
 
-	if (open (_path)) {
-		throw failed_constructor ();
-	}
-
-	_open = true;
+	/* no lock required since we do not actually exist yet */
+	load_model_unlocked (true);
 }
 
 SMFSource::~SMFSource ()
@@ -219,23 +217,22 @@ SMFSource::close ()
 
 extern PBD::Timing minsert;
 
-/** All stamps in audio samples */
-samplecnt_t
-SMFSource::read_unlocked (const Lock&                    lock,
+timecnt_t
+SMFSource::read_unlocked (const ReaderLock&               lock,
                           Evoral::EventSink<samplepos_t>& destination,
-                          samplepos_t const               source_start,
-                          samplepos_t                     start,
-                          samplecnt_t                     duration,
-                          Evoral::Range<samplepos_t>*     loop_range,
-                          MidiStateTracker*              tracker,
-                          MidiChannelFilter*             filter) const
+                          timepos_t const &               source_start,
+                          timepos_t const &               start,
+                          timecnt_t const &               duration,
+                          Temporal::Range*                loop_range,
+                          MidiNoteTracker*                tracker,
+                          MidiChannelFilter*              filter) const
 {
 	int      ret  = 0;
-	uint64_t time = 0; // in SMF ticks, 1 tick per _ppqn
+	timepos_t time; // in SMF ticks, 1 tick per _ppqn
 
 	if (writable() && !_open) {
 		/* nothing to read since nothing has ben written */
-		return duration;
+		return timecnt_t();
 	}
 
 	DEBUG_TRACE (DEBUG::MidiSourceIO, string_compose ("SMF read_unlocked: start %1 duration %2\n", start, duration));
@@ -247,12 +244,12 @@ SMFSource::read_unlocked (const Lock&                    lock,
 
 	size_t scratch_size = 0; // keep track of scratch to minimize reallocs
 
-	BeatsSamplesConverter converter(_session.tempo_map(), source_start);
+	/* start of read in SMF ticks (which may differ from our own musical ticks */
+	const uint64_t start_ticks = llrint (start.beats().to_ticks() * (Temporal::Beats::PPQN / ppqn()));
 
-	const uint64_t start_ticks = converter.from(start).to_ticks();
 	DEBUG_TRACE (DEBUG::MidiSourceIO, string_compose ("SMF read_unlocked: start in ticks %1\n", start_ticks));
 
-	if (_smf_last_read_end == 0 || start != _smf_last_read_end) {
+	if (_smf_last_read_end.is_zero() || start != _smf_last_read_end) {
 		DEBUG_TRACE (DEBUG::MidiSourceIO, string_compose ("SMF read_unlocked: seek to %1\n", start));
 		Evoral::SMF::seek_to_start();
 		while (time < start_ticks) {
@@ -261,9 +258,9 @@ SMFSource::read_unlocked (const Lock&                    lock,
 			ret = read_event(&ev_delta_t, &ev_size, &ev_buffer, &ignored);
 			if (ret == -1) { // EOF
 				_smf_last_read_end = start + duration;
-				return duration;
+				return timecnt_t();
 			}
-			time += ev_delta_t; // accumulate delta time
+			time += timepos_t::from_ticks (ev_delta_t); // accumulate delta time
 		}
 	} else {
 		DEBUG_TRACE (DEBUG::MidiSourceIO, string_compose ("SMF read_unlocked: set time to %1\n", _smf_last_read_time));
@@ -280,7 +277,7 @@ SMFSource::read_unlocked (const Lock&                    lock,
 			break;
 		}
 
-		time += ev_delta_t; // accumulate delta time
+		time += timepos_t::from_ticks (ev_delta_t); // accumulate delta time
 		_smf_last_read_time = time;
 
 		if (ret == 0) { // meta-event (skipped, just accumulate time)
@@ -295,15 +292,16 @@ SMFSource::read_unlocked (const Lock&                    lock,
 		/* Note that we add on the source start time (in session samples) here so that ev_sample_time
 		   is in session samples.
 		*/
-		const samplepos_t ev_sample_time = converter.to(Temporal::Beats::ticks_at_rate(time, ppqn())) + source_start;
+		const samplepos_t ev_sample_time = (time + timepos_t (source_start.beats())).samples();
+		timepos_t est (ev_sample_time);
 
 		if (loop_range) {
-			loop_range->squish (ev_sample_time);
+			est = loop_range->squish (est);
 		}
 
-		if (ev_sample_time < start + duration) {
+		if (est < start + duration) {
 			if (!filter || !filter->filter(ev_buffer, ev_size)) {
-				destination.write (ev_sample_time, Evoral::MIDI_EVENT, ev_size, ev_buffer);
+				destination.write (est.samples(), Evoral::MIDI_EVENT, ev_size, ev_buffer);
 				if (tracker) {
 					tracker->track(ev_buffer);
 				}
@@ -321,17 +319,20 @@ SMFSource::read_unlocked (const Lock&                    lock,
 	return duration;
 }
 
-samplecnt_t
-SMFSource::write_unlocked (const Lock&                 lock,
+timecnt_t
+SMFSource::write_unlocked (const WriterLock&            lock,
                            MidiRingBuffer<samplepos_t>& source,
-                           samplepos_t                  position,
-                           samplecnt_t                  cnt)
+                           timepos_t const &            position,
+                           timecnt_t const &            cnt)
 {
+
 	if (!_writing) {
 		mark_streaming_write_started (lock);
 	}
 
 	samplepos_t        time;
+	const samplepos_t        pos_samples = position.samples();
+	const samplecnt_t        cnt_samples = cnt.samples();
 	Evoral::EventType type;
 	uint32_t          size;
 
@@ -351,8 +352,8 @@ SMFSource::write_unlocked (const Lock&                 lock,
 			break;
 		}
 
-		if ((cnt != max_samplecnt) &&
-		    (time > position + _capture_length + cnt)) {
+		if ((cnt != timecnt_t::max (cnt.time_domain())) &&
+		    (time > pos_samples + _capture_length + cnt_samples)) {
 			/* The diskstream doesn't want us to write everything, and this
 			   event is past the end of this block, so we're done for now. */
 			break;
@@ -378,11 +379,11 @@ SMFSource::write_unlocked (const Lock&                 lock,
 		}
 
 		/* Convert event time from absolute to source relative. */
-		if (time < position) {
+		if (time < pos_samples) {
 			error << _("Event time is before MIDI source position") << endmsg;
 			break;
 		}
-		time -= position;
+		time -= pos_samples;
 
 		ev.set(buf, size, time);
 		ev.set_event_type(Evoral::MIDI_EVENT);
@@ -392,7 +393,7 @@ SMFSource::write_unlocked (const Lock&                 lock,
 			continue;
 		}
 
-		append_event_samples(lock, ev, position);
+		append_event_samples(lock, ev, pos_samples);
 	}
 
 	Evoral::SMF::flush ();
@@ -401,9 +402,16 @@ SMFSource::write_unlocked (const Lock&                 lock,
 	return cnt;
 }
 
+void
+SMFSource::update_length (timepos_t const & dur)
+{
+	assert (!_length || (_length.time_domain() == dur.time_domain()));
+	_length = dur;
+}
+
 /** Append an event with a timestamp in beats */
 void
-SMFSource::append_event_beats (const Glib::Threads::Mutex::Lock&   lock,
+SMFSource::append_event_beats (const WriterLock&   lock,
                                const Evoral::Event<Temporal::Beats>& ev)
 {
 	if (!_writing || ev.size() == 0)  {
@@ -419,7 +427,7 @@ SMFSource::append_event_beats (const Glib::Threads::Mutex::Lock&   lock,
 	Temporal::Beats time = ev.time();
 	if (time < _last_ev_time_beats) {
 		const Temporal::Beats difference = _last_ev_time_beats - time;
-		if (difference.to_double() / (double)ppqn() < 1.0) {
+		if (difference < Temporal::Beats::ticks (ppqn())) {
 			/* Close enough.  This problem occurs because Sequence is not
 			   actually ordered due to fuzzy time comparison.  I'm pretty sure
 			   this is inherently a bad idea which causes problems all over the
@@ -428,7 +436,7 @@ SMFSource::append_event_beats (const Glib::Threads::Mutex::Lock&   lock,
 		} else {
 			/* Out of order by more than a tick. */
 			warning << string_compose(_("Skipping event with unordered beat time %1 < %2 (off by %3 beats, %4 ticks)"),
-			                          ev.time(), _last_ev_time_beats, difference, difference.to_double() / (double)ppqn())
+			                          ev.time(), _last_ev_time_beats, difference, difference)
 			        << endmsg;
 			return;
 		}
@@ -446,7 +454,8 @@ SMFSource::append_event_beats (const Glib::Threads::Mutex::Lock&   lock,
 		_model->append (ev, event_id);
 	}
 
-	_length_beats = max(_length_beats, time);
+	assert (!_length || (_length.time_domain() == Temporal::BeatTime));
+	_length  = timepos_t (max (_length.beats(), time));
 
 	const Temporal::Beats delta_time_beats = time - _last_ev_time_beats;
 	const uint32_t      delta_time_ticks = delta_time_beats.to_ticks(ppqn());
@@ -459,7 +468,7 @@ SMFSource::append_event_beats (const Glib::Threads::Mutex::Lock&   lock,
 
 /** Append an event with a timestamp in samples (samplepos_t) */
 void
-SMFSource::append_event_samples (const Glib::Threads::Mutex::Lock& lock,
+SMFSource::append_event_samples (const WriterLock& lock,
                                 const Evoral::Event<samplepos_t>&  ev,
                                 samplepos_t                        position)
 {
@@ -478,8 +487,11 @@ SMFSource::append_event_samples (const Glib::Threads::Mutex::Lock& lock,
 		return;
 	}
 
-	BeatsSamplesConverter converter(_session.tempo_map(), position);
-	const Temporal::Beats  ev_time_beats = converter.from(ev.time());
+	/* a distance measure that starts at @param position (audio time) and
+	   extends for ev.time() (audio time)
+	*/
+	const timecnt_t distance (timepos_t (ev.time()), timepos_t (position));
+	const Temporal::Beats  ev_time_beats = distance.beats ();
 	Evoral::event_id_t   event_id;
 
 	if (ev.id() < 0) {
@@ -490,26 +502,30 @@ SMFSource::append_event_samples (const Glib::Threads::Mutex::Lock& lock,
 
 	if (_model) {
 		const Evoral::Event<Temporal::Beats> beat_ev (ev.event_type(),
-		                                            ev_time_beats,
-		                                            ev.size(),
-		                                            const_cast<uint8_t*>(ev.buffer()));
+		                                              ev_time_beats,
+		                                              ev.size(),
+		                                              const_cast<uint8_t*>(ev.buffer()));
 		_model->append (beat_ev, event_id);
 	}
 
-	_length_beats = max(_length_beats, ev_time_beats);
+	assert (!_length || (_length.time_domain() == Temporal::BeatTime));
+	_length = timepos_t (max (_length.beats(), ev_time_beats));
 
-	const Temporal::Beats last_time_beats  = converter.from (_last_ev_time_samples);
-	const Temporal::Beats delta_time_beats = ev_time_beats - last_time_beats;
-	const uint32_t      delta_time_ticks = delta_time_beats.to_ticks(ppqn());
+	/* a distance measure that starts at @param _last_ev_time_samples (audio time) and
+	   extends for ev.time() (audio time)
+	*/
+	const timecnt_t       delta_distance (timepos_t (ev.time()), timepos_t (_last_ev_time_samples));
+	const Temporal::Beats delta_time_beats = delta_distance.beats ();
+	const uint32_t        delta_time_ticks = delta_time_beats.to_ticks(ppqn());
 
-	Evoral::SMF::append_event_delta(delta_time_ticks, ev.size(), ev.buffer(), event_id);
+	Evoral::SMF::append_event_delta (delta_time_ticks, ev.size(), ev.buffer(), event_id);
 	_last_ev_time_samples = ev.time();
 	_flags = Source::Flag (_flags & ~Empty);
 	_flags = Source::Flag (_flags & ~Missing);
 }
 
 XMLNode&
-SMFSource::get_state ()
+SMFSource::get_state () const
 {
 	XMLNode& node = MidiSource::get_state();
 	node.set_property (X_("origin"), _origin);
@@ -535,7 +551,7 @@ SMFSource::set_state (const XMLNode& node, int version)
 }
 
 void
-SMFSource::mark_streaming_midi_write_started (const Lock& lock, NoteMode mode)
+SMFSource::mark_streaming_midi_write_started (const WriterLock& lock, NoteMode mode)
 {
 	if (!_open && open_for_write()) {
 		error << string_compose (_("cannot open MIDI file %1 for write"), _path) << endmsg;
@@ -550,13 +566,13 @@ SMFSource::mark_streaming_midi_write_started (const Lock& lock, NoteMode mode)
 }
 
 void
-SMFSource::mark_streaming_write_completed (const Lock& lock)
+SMFSource::mark_streaming_write_completed (const WriterLock& lock)
 {
 	mark_midi_streaming_write_completed (lock, Evoral::Sequence<Temporal::Beats>::DeleteStuckNotes);
 }
 
 void
-SMFSource::mark_midi_streaming_write_completed (const Lock& lm, Evoral::Sequence<Temporal::Beats>::StuckNoteOption stuck_notes_option, Temporal::Beats when)
+SMFSource::mark_midi_streaming_write_completed (const WriterLock& lm, Evoral::Sequence<Temporal::Beats>::StuckNoteOption stuck_notes_option, Temporal::Beats when)
 {
 	MidiSource::mark_midi_streaming_write_completed (lm, stuck_notes_option, when);
 
@@ -624,29 +640,24 @@ static bool compare_eventlist (
 }
 
 void
-SMFSource::load_model (const Glib::Threads::Mutex::Lock& lock, bool force_reload)
+SMFSource::load_model (const WriterLock& lock, bool force_reload)
 {
-	if (_writing) {
-		return;
-	}
+	invalidate (lock);
+	load_model_unlocked (force_reload);
+	invalidate (lock);
+}
 
-	if (_model && !force_reload) {
-		return;
-	}
+void
+SMFSource::load_model_unlocked (bool force_reload)
+{
+	assert (!_writing);
 
 	if (!_model) {
-		boost::shared_ptr<SMFSource> smf = boost::dynamic_pointer_cast<SMFSource> ( shared_from_this () );
-		_model = boost::shared_ptr<MidiModel> (new MidiModel (smf));
+		_model = boost::shared_ptr<MidiModel> (new MidiModel (*this));
 	} else {
 		_model->clear();
 	}
 
-	invalidate(lock);
-
-	if (writable() && !_open) {
-		return;
-	}
-
 	_model->start_write();
 	Evoral::SMF::seek_to_start();
 
@@ -662,6 +673,11 @@ SMFSource::load_model (const Glib::Threads::Mutex::Lock& lock, bool force_reload
 	Evoral::event_id_t event_id;
 	bool have_event_id;
 
+	_num_channels     = 0;
+	_n_note_on_events = 0;
+	_has_pgm_change   = false;
+	_used_channels.reset ();
+
 	// TODO simplify event allocation
 	std::list< std::pair< Evoral::Event<Temporal::Beats>*, gint > > eventlist;
 
@@ -683,6 +699,23 @@ SMFSource::load_model (const Glib::Threads::Mutex::Lock& lock, bool force_reload
 				continue;
 			}
 
+			/* aggregate information about channels and pgm-changes */
+			uint8_t type = buf[0] & 0xf0;
+			uint8_t chan = buf[0] & 0x0f;
+			if (type >= 0x80 && type <= 0xE0) {
+				_used_channels.set(chan);
+				switch (type) {
+					case MIDI_CMD_NOTE_ON:
+						++_n_note_on_events;
+						break;
+					case MIDI_CMD_PGM_CHANGE:
+						_has_pgm_change = true;
+						break;
+					default:
+						break;
+				}
+			}
+
 			if (ret > 0) {
 				/* not a meta-event */
 
@@ -713,7 +746,8 @@ SMFSource::load_model (const Glib::Threads::Mutex::Lock& lock, bool force_reload
 				scratch_size = std::max(size, scratch_size);
 				size = scratch_size;
 
-				_length_beats = max(_length_beats, event_time);
+				assert (!_length || (_length.time_domain() == Temporal::BeatTime));
+				_length = max (_length, timepos_t (event_time));
 			}
 
 			/* event ID's must immediately precede the event they are for */
@@ -721,6 +755,8 @@ SMFSource::load_model (const Glib::Threads::Mutex::Lock& lock, bool force_reload
 		}
 	}
 
+	_num_channels = _used_channels.size();
+
 	eventlist.sort(compare_eventlist);
 
 	std::list< std::pair< Evoral::Event<Temporal::Beats>*, gint > >::iterator it;
@@ -733,15 +769,20 @@ SMFSource::load_model (const Glib::Threads::Mutex::Lock& lock, bool force_reload
         // _playback_buf->dump (cerr);
         // cerr << "----------------\n";
 
-	_model->end_write (Evoral::Sequence<Temporal::Beats>::ResolveStuckNotes, _length_beats);
+	_model->end_write (Evoral::Sequence<Temporal::Beats>::ResolveStuckNotes, _length.beats());
 	_model->set_edited (false);
-	invalidate(lock);
 
-	free(buf);
+	free (buf);
+}
+
+Evoral::SMF::UsedChannels
+SMFSource::used_midi_channels()
+{
+	return _used_channels;
 }
 
 void
-SMFSource::destroy_model (const Glib::Threads::Mutex::Lock& lock)
+SMFSource::destroy_model (const WriterLock& lock)
 {
 	//cerr << _name << " destroying model " << _model.get() << endl;
 	_model.reset();
@@ -749,9 +790,9 @@ SMFSource::destroy_model (const Glib::Threads::Mutex::Lock& lock)
 }
 
 void
-SMFSource::flush_midi (const Lock& lock)
+SMFSource::flush_midi (const WriterLock& lock)
 {
-	if (!writable() || _length_beats == 0.0) {
+	if (!writable() || _length.is_zero()) {
 		return;
 	}
 
@@ -772,7 +813,7 @@ SMFSource::set_path (const string& p)
 
 /** Ensure that this source has some file on disk, even if it's just a SMF header */
 void
-SMFSource::ensure_disk_file (const Lock& lock)
+SMFSource::ensure_disk_file (const WriterLock& lock)
 {
 	if (!writable()) {
 		return;
