@@ -35,6 +35,7 @@
 #include "pbd/file_utils.h"
 #include "pbd/pthread_utils.h"
 
+#include "ardour/debug.h"
 #include "ardour/filesystem_paths.h"
 #include "ardour/port_manager.h"
 #include "ardouralsautil/devicelist.h"
@@ -66,7 +67,7 @@ AlsaAudioBackend::AlsaAudioBackend (AudioEngine& e, AudioBackendInfo& info)
 	, _last_process_start (0)
 	, _input_audio_device ("")
 	, _output_audio_device ("")
-	, _midi_driver_option (get_standard_device_name (DeviceNone))
+	, _midi_driver_option (_("ALSA sequencer"))
 	, _samplerate (48000)
 	, _samples_per_period (1024)
 	, _periods_per_cycle (2)
@@ -345,8 +346,10 @@ AlsaAudioBackend::set_input_device_name (const std::string& d)
 		_input_audio_device_info.valid = false;
 		return 1;
 	}
+	_device_reservation.acquire_device (alsa_device.c_str ());
 	/* device will be busy once used, hence cache the parameters */
 	/* return */ get_alsa_device_parameters (alsa_device.c_str (), false, &_input_audio_device_info);
+	_device_reservation.release_device ();
 	return 0;
 }
 
@@ -1730,7 +1733,8 @@ AlsaAudioBackend::set_latency_range (PortEngine::PortHandle port_handle, bool fo
 {
 	BackendPortPtr port = boost::dynamic_pointer_cast<BackendPort> (port_handle);
 	if (!valid_port (port)) {
-		PBD::error << _("AlsaPort::set_latency_range (): invalid port.") << endmsg;
+		DEBUG_TRACE (PBD::DEBUG::BackendPorts, "AlsaPort::set_latency_range (): invalid port.");
+		return;
 	}
 	port->set_latency_range (latency_range, for_playback);
 }
@@ -1742,7 +1746,7 @@ AlsaAudioBackend::get_latency_range (PortEngine::PortHandle port_handle, bool fo
 	LatencyRange   r;
 
 	if (!valid_port (port)) {
-		PBD::error << _("AlsaPort::get_latency_range (): invalid port.") << endmsg;
+		DEBUG_TRACE (PBD::DEBUG::BackendPorts, "AlsaPort::get_latency_range (): invalid port.");
 		r.min = 0;
 		r.max = 0;
 		return r;
@@ -1790,7 +1794,6 @@ AlsaAudioBackend::get_buffer (PortEngine::PortHandle port_handle, pframes_t nfra
 {
 	BackendPortPtr port = boost::dynamic_pointer_cast<BackendPort> (port_handle);
 	assert (port);
-	assert (valid_port (port));
 	return port->get_buffer (nframes);
 }
 
@@ -2222,18 +2225,33 @@ AlsaAudioBackend::AudioSlave::update_latencies (uint32_t play, uint32_t capt)
 {
 	LatencyRange lr;
 	lr.min = lr.max = (capt);
+	bool changed = false;
 	for (std::vector<BackendPortPtr>::const_iterator it = inputs.begin (); it != inputs.end (); ++it) {
+		LatencyRange lx;
+		lx = (*it)->latency_range (false);
+		if (lr == lx) {
+			continue;
+		}
 		(*it)->set_latency_range (lr, false);
+		changed = true;
 	}
 
 	lr.min = lr.max = play;
 	for (std::vector<BackendPortPtr>::const_iterator it = outputs.begin (); it != outputs.end (); ++it) {
+		LatencyRange lx;
+		lx = (*it)->latency_range (true);
+		if (lr == lx) {
+			continue;
+		}
 		(*it)->set_latency_range (lr, true);
+		changed = true;
 	}
 #ifndef NDEBUG
-	printf ("ALSA SLAVE-device latency play=%d capt=%d\n", play, capt); // XXX DEBUG
+	printf ("ALSA SLAVE-device latency play=%d capt=%d changed:%d\n", play, capt, changed); // XXX DEBUG
 #endif
-	UpdateLatency (); /* EMIT SIGNAL */
+	if (changed) {
+		UpdateLatency (); /* EMIT SIGNAL */
+	}
 }
 
 /******************************************************************************/
@@ -2455,7 +2473,7 @@ AlsaDeviceReservation::acquire_device (const char* device_name)
 		return false;
 	}
 
-	/* wait to check if reservation suceeded. */
+	/* wait to check if reservation succeeded. */
 	int timeout = 500; // 5 sec
 	while (_device_reservation && !_reservation_succeeded && --timeout > 0) {
 		Glib::usleep (10000);
