@@ -28,6 +28,8 @@
 
 #include "pbd/signals.h"
 
+#include "temporal/timeline.h"
+
 #include "ardour/types.h"
 #include "ardour/presentation_info.h"
 
@@ -39,6 +41,7 @@ namespace ARDOUR {
 	class Session;
 	class SessionEvent;
 	class Stripable;
+	class Trigger;
 }
 
 class LIBCONTROLCP_API BasicUI {
@@ -57,7 +60,7 @@ class LIBCONTROLCP_API BasicUI {
 	/* transport control */
 
 	void loop_toggle ();
-	void loop_location (samplepos_t start, samplepos_t end);
+	void loop_location (Temporal::timepos_t const & start, Temporal::timepos_t const & end);
 	void access_action ( std::string action_path );
 	static PBD::Signal2<void,std::string,std::string> AccessAction;
 	void goto_zero ();
@@ -74,8 +77,8 @@ class LIBCONTROLCP_API BasicUI {
 	double transport_rolling () const;
 
 	void jump_by_seconds (double sec, ARDOUR::LocateTransportDisposition ltd = ARDOUR::RollIfAppropriate);
-	void jump_by_bars (double bars, ARDOUR::LocateTransportDisposition ltd = ARDOUR::RollIfAppropriate);
-	void jump_by_beats (double beats, ARDOUR::LocateTransportDisposition ltd = ARDOUR::RollIfAppropriate);
+	void jump_by_bars (int bars, ARDOUR::LocateTransportDisposition ltd = ARDOUR::RollIfAppropriate);
+	void jump_by_beats (int beats, ARDOUR::LocateTransportDisposition ltd = ARDOUR::RollIfAppropriate);
 
 	ARDOUR::samplepos_t transport_sample ();
 	void locate (ARDOUR::samplepos_t sample, ARDOUR::LocateTransportDisposition ltd);
@@ -97,6 +100,9 @@ class LIBCONTROLCP_API BasicUI {
 	void toggle_click();
 	void midi_panic();
 
+	void trigger_cue_row (int cue);
+	void trigger_stop_all (bool stop_now = false);
+
 	void toggle_monitor_mute();
 	void toggle_monitor_dim();
 	void toggle_monitor_mono();
@@ -106,7 +112,7 @@ class LIBCONTROLCP_API BasicUI {
 	void quick_snapshot_stay ();
 	void quick_snapshot_switch ();
 
-	void toggle_roll(bool roll_out_of_bounded_mode=true);  //this provides the same operation as the "spacebar", it's a lot smarter than "play".
+	void toggle_roll(bool with_abort, bool roll_out_of_bounded_mode);  //this provides the same operation as the "spacebar", it's a lot smarter than "play".
 
 	void stop_forget();
 
@@ -161,6 +167,12 @@ class LIBCONTROLCP_API BasicUI {
 	bool rewind_button_onoff() const;
 	bool loop_button_onoff() const;
 
+	void bang_trigger_at (int x, int y);
+	void unbang_trigger_at (int x, int y);
+
+	/* it would be nice to use TriggerPtr here but that implies including ardour/triggerbox.h */
+	boost::shared_ptr<ARDOUR::Trigger> find_trigger (int x, int y);
+
   protected:
 	BasicUI ();
 	ARDOUR::Session* session;
