@@ -146,6 +146,8 @@ GenericMidiControlProtocol::GenericMidiControlProtocol (Session& s)
 
 GenericMidiControlProtocol::~GenericMidiControlProtocol ()
 {
+	set_active (false);
+
 	if (_input_port) {
 		DEBUG_TRACE (DEBUG::GenericMidi, string_compose ("unregistering input port %1\n", boost::shared_ptr<ARDOUR::Port>(_input_port)->name()));
 		Glib::Threads::Mutex::Lock em (AudioEngine::instance()->process_lock());
@@ -606,7 +608,7 @@ GenericMidiControlProtocol::check_used_event (int pos, int control_number)
 }
 
 XMLNode&
-GenericMidiControlProtocol::get_state ()
+GenericMidiControlProtocol::get_state () const
 {
 	XMLNode& node (ControlProtocol::get_state());
 
@@ -634,15 +636,15 @@ GenericMidiControlProtocol::get_state ()
 	node.add_child_nocopy (*children);
 
 	Glib::Threads::Mutex::Lock lm2 (controllables_lock);
-	for (MIDIControllables::iterator i = controllables.begin(); i != controllables.end(); ++i) {
+	for (auto const & c : controllables) {
 
 		/* we don't care about bindings that come from a bindings map, because
 		   they will all be reset/recreated when we load the relevant bindings
 		   file.
 		*/
 
-		if ((*i)->get_controllable() && (*i)->learned()) {
-			children->add_child_nocopy ((*i)->get_state());
+		if (c->get_controllable() && c->learned()) {
+			children->add_child_nocopy (c->get_state());
 		}
 	}
 
@@ -1611,7 +1613,7 @@ GenericMidiControlProtocol::maybe_start_touch (boost::shared_ptr<Controllable> c
 {
 	boost::shared_ptr<AutomationControl> actl = boost::dynamic_pointer_cast<AutomationControl> (controllable);
 	if (actl) {
-		actl->start_touch (session->audible_sample ());
+		actl->start_touch (timepos_t (session->audible_sample ()));
 	}
 }
 
@@ -1647,7 +1649,7 @@ GenericMidiControlProtocol::midi_input_handler (Glib::IOCondition ioc, boost::we
 		return false;
 	}
 
-	DEBUG_TRACE (DEBUG::GenericMidi, string_compose ("something happend on  %1\n", boost::shared_ptr<MIDI::Port>(port)->name()));
+	DEBUG_TRACE (DEBUG::GenericMidi, string_compose ("something happened on  %1\n", boost::shared_ptr<MIDI::Port>(port)->name()));
 
 	if (ioc & ~IO_IN) {
 		return false;
@@ -1663,3 +1665,32 @@ GenericMidiControlProtocol::midi_input_handler (Glib::IOCondition ioc, boost::we
 
 	return true;
 }
+
+void
+GenericMidiControlProtocol::add_rid_to_selection (int rid)
+{
+	int id = rid + (_current_bank * _bank_size);
+	ControlProtocol::add_rid_to_selection (id);
+}
+
+void
+GenericMidiControlProtocol::set_rid_selection (int rid)
+{
+	int id = rid + (_current_bank * _bank_size);
+	ControlProtocol::set_rid_selection (id);
+}
+
+void
+GenericMidiControlProtocol::toggle_rid_selection (int rid)
+{
+	int id = rid + (_current_bank * _bank_size);
+	ControlProtocol::toggle_rid_selection (id);
+}
+
+void
+GenericMidiControlProtocol::remove_rid_from_selection (int rid)
+{
+	int id = rid + (_current_bank * _bank_size);
+	ControlProtocol::remove_rid_from_selection (id);
+}
+
