@@ -129,15 +129,15 @@ Strip::Strip (Surface& s, const std::string& name, int index, const map<Button::
 
 		// The main unit has 9 faders under the second display.
 		// Extenders have 8 faders.
-		if (s.mcp().device_info().has_master_fader()) {
+		if (s.number() == s.mcp().device_info().master_position()) {
 			_lcd2_label_pitch = 6;
 		}
 	}
 
 	for (map<Button::ID,StripButtonInfo>::const_iterator b = strip_buttons.begin(); b != strip_buttons.end(); ++b) {
-		Button* bb = dynamic_cast<Button*> (Button::factory (*_surface, b->first, b->second.base_id + index, b->second.name, *this));
+		DEBUG_RESULT_CAST (Button*, bb, dynamic_cast<Button*>, (Button::factory (*_surface, b->first, b->second.base_id + index, b->second.name, *this)));
 		DEBUG_TRACE (DEBUG::MackieControl, string_compose ("surface %1 strip %2 new button BID %3 id %4 from base %5\n",
-								   _surface->number(), index, Button::id_to_name (bb->bid()),
+		                                                   _surface->number(), index, Button::id_to_name (bb->bid()),
 								   bb->id(), b->second.base_id));
 	}
 }
@@ -180,7 +180,8 @@ Strip::add (Control & control)
 			break;
 		}
 	}
-}
+
+ }
 
 void
 Strip::set_stripable (boost::shared_ptr<Stripable> r, bool /*with_messages*/)
@@ -193,13 +194,22 @@ Strip::set_stripable (boost::shared_ptr<Stripable> r, bool /*with_messages*/)
 
 	stripable_connections.drop_connections ();
 
-	_solo->set_control (boost::shared_ptr<AutomationControl>());
-	_mute->set_control (boost::shared_ptr<AutomationControl>());
-	_select->set_control (boost::shared_ptr<AutomationControl>());
-	_recenable->set_control (boost::shared_ptr<AutomationControl>());
 	_fader->set_control (boost::shared_ptr<AutomationControl>());
 	_vpot->set_control (boost::shared_ptr<AutomationControl>());
 
+	if (_select) {
+		_select->set_control (boost::shared_ptr<AutomationControl>());
+	}
+	if (_solo) {
+		_solo->set_control (boost::shared_ptr<AutomationControl>());
+	}
+	if (_mute) {
+		_mute->set_control (boost::shared_ptr<AutomationControl>());
+	}
+	if (_recenable) {
+		_recenable->set_control (boost::shared_ptr<AutomationControl>());
+	}
+
 	_stripable = r;
 
 	reset_saved_values ();
@@ -213,8 +223,12 @@ Strip::set_stripable (boost::shared_ptr<Stripable> r, bool /*with_messages*/)
 	DEBUG_TRACE (DEBUG::MackieControl, string_compose ("Surface %1 strip %2 now mapping stripable %3\n",
 							   _surface->number(), _index, _stripable->name()));
 
-	_solo->set_control (_stripable->solo_control());
-	_mute->set_control (_stripable->mute_control());
+	if (_solo) {
+		_solo->set_control (_stripable->solo_control());
+	}
+	if (_mute) {
+		_mute->set_control (_stripable->mute_control());
+	}
 
 	_stripable->solo_control()->Changed.connect (stripable_connections, MISSING_INVALIDATOR, boost::bind (&Strip::notify_solo_changed, this), ui_context());
 	_stripable->mute_control()->Changed.connect(stripable_connections, MISSING_INVALIDATOR, boost::bind (&Strip::notify_mute_changed, this), ui_context());
@@ -235,7 +249,7 @@ Strip::set_stripable (boost::shared_ptr<Stripable> r, bool /*with_messages*/)
 
 	boost::shared_ptr<AutomationControl> rec_enable_control = _stripable->rec_enable_control ();
 
-	if (rec_enable_control) {
+	if (_recenable && rec_enable_control) {
 		_recenable->set_control (rec_enable_control);
 		rec_enable_control->Changed.connect (stripable_connections, MISSING_INVALIDATOR, boost::bind (&Strip::notify_record_enable_changed, this), ui_context());
 	}
@@ -271,7 +285,9 @@ Strip::set_stripable (boost::shared_ptr<Stripable> r, bool /*with_messages*/)
 		set_vpot_parameter (_pan_mode);
 	}
 
-	_fader->set_control (_stripable->gain_control());
+	if (_fader) {
+		_fader->set_control (_stripable->gain_control());
+	}
 
 	notify_all ();
 }
@@ -389,11 +405,15 @@ Strip::notify_processor_changed (bool force_update)
 void
 Strip::notify_property_changed (const PropertyChange& what_changed)
 {
+	if (!_stripable) {
+		return;
+	}
+
 	if (what_changed.contains (ARDOUR::Properties::name)) {
 		show_stripable_name ();
 	}
 
-	if (what_changed.contains (ARDOUR::Properties::selected)) {
+	if (_select && what_changed.contains (ARDOUR::Properties::selected)) {
 		_surface->write (_select->set_state (_stripable->is_selected()));
 	}
 }
@@ -401,7 +421,7 @@ Strip::notify_property_changed (const PropertyChange& what_changed)
 void
 Strip::update_selection_state ()
 {
-	if(_stripable) {
+	if(_select && _stripable) {
 		_surface->write (_select->set_state (_stripable->is_selected()));
 	}
 }
@@ -597,7 +617,7 @@ Strip::fader_touch_event (Button&, ButtonState bs)
 		boost::shared_ptr<AutomationControl> ac = _fader->control ();
 
 		_fader->set_in_use (true);
-		_fader->start_touch (_surface->mcp().transport_sample());
+		_fader->start_touch (timepos_t (_surface->mcp().transport_sample()));
 
 		if (ac) {
 			do_parameter_display (ac->desc(), ac->get_value());
@@ -606,7 +626,7 @@ Strip::fader_touch_event (Button&, ButtonState bs)
 	} else {
 
 		_fader->set_in_use (false);
-		_fader->stop_touch (_surface->mcp().transport_sample());
+		_fader->stop_touch (timepos_t (_surface->mcp().transport_sample()));
 
 	}
 }
@@ -682,7 +702,7 @@ Strip::handle_button (Button& button, ButtonState bs)
 }
 
 std::string
-Strip::format_paramater_for_display(
+Strip::format_parameter_for_display(
 		ARDOUR::ParameterDescriptor const& desc,
 		float val,
 		boost::shared_ptr<ARDOUR::Stripable> stripable_for_non_mixbus_azimuth_automation,
@@ -736,7 +756,7 @@ Strip::format_paramater_for_display(
 void
 Strip::do_parameter_display (ARDOUR::ParameterDescriptor const& desc, float val, bool screen_hold)
 {
-	pending_display[1] = format_paramater_for_display(desc, val, _stripable, screen_hold);
+	pending_display[1] = format_parameter_for_display(desc, val, _stripable, screen_hold);
 
 	if (screen_hold) {
 		/* we just queued up a parameter to be displayed.
@@ -749,10 +769,12 @@ Strip::do_parameter_display (ARDOUR::ParameterDescriptor const& desc, float val,
 void
 Strip::handle_fader_touch (Fader& fader, bool touch_on)
 {
+	timepos_t now (_surface->mcp().transport_sample());
+
 	if (touch_on) {
-		fader.start_touch (_surface->mcp().transport_sample());
+		fader.start_touch (now);
 	} else {
-		fader.stop_touch (_surface->mcp().transport_sample());
+		fader.stop_touch (now);
 	}
 }
 
