@@ -60,7 +60,7 @@ class Route;
 }
 
 /* this is mostly a placeholder because I suspect that at some
-   point we will want to add more members to accomodate
+   point we will want to add more members to accommodate
    certain types of requests to the OSC UI
 */
 
@@ -80,7 +80,7 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 
 	static OSC* instance() { return _instance; }
 
-	XMLNode& get_state ();
+	XMLNode& get_state () const;
 	int set_state (const XMLNode&, int version);
 
 	void stripable_selection_changed () {}
@@ -262,6 +262,8 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	std::string get_remote_port () { return remote_port; }
 	void set_remote_port (std::string pt) { remote_port = pt; }
 
+	CONTROL_PROTOCOL_THREADS_NEED_TEMPO_MAP_DECL();
+
   protected:
         void thread_init ();
 	void do_request (OSCUIRequest*);
@@ -484,6 +486,9 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 		return 0; \
 	}
 
+	PATH_CALLBACK1(trigger_cue_row,i,);
+	PATH_CALLBACK1(trigger_stop_all,i,);  //0 = "stop at end of bar"  1 = "stop now"
+
 	PATH_CALLBACK1(set_transport_speed,f,);
 	PATH_CALLBACK1(add_marker_name,s,&);
 	PATH_CALLBACK1(access_action,s,&);
@@ -627,6 +632,9 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	PATH_CALLBACK1_MSG(route_plugin_list,i);
 	PATH_CALLBACK2_MSG(route_plugin_descriptor,i,i);
 	PATH_CALLBACK2_MSG(route_plugin_reset,i,i);
+	PATH_CALLBACK2_MSG(trigger_bang,i,i);
+	PATH_CALLBACK2_MSG(trigger_unbang,i,i);
+	PATH_CALLBACK2_MSG(trigger_stop,i,i);  /* second arg is 'stop now' */
 
 	int strip_parse (const char *path, const char* types, lo_arg **argv, int argc, lo_message msg);
 	int master_parse (const char *path, const char* types, lo_arg **argv, int argc, lo_message msg);
@@ -637,6 +645,8 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	int _strip_select (boost::shared_ptr<ARDOUR::Stripable> s, lo_address addr);
 	int _strip_select2 (boost::shared_ptr<ARDOUR::Stripable> s, OSCSurface *sur, lo_address addr);
 
+	void loop_location (int start, int end);
+
 	int route_set_send_gain_dB (int rid, int sid, float val, lo_message msg);
 	int route_set_send_fader (int rid, int sid, float val, lo_message msg);
 	int route_set_send_enable (int rid, int sid, float val, lo_message msg);
@@ -648,6 +658,10 @@ class OSC : public ARDOUR::ControlProtocol, public AbstractUI<OSCUIRequest>
 	int route_plugin_descriptor(int ssid, int piid, lo_message msg);
 	int route_plugin_reset(int ssid, int piid, lo_message msg);
 
+	int trigger_bang(int rid, int stop_now, lo_message msg);
+	int trigger_unbang(int rid, int stop_now, lo_message msg);
+	int trigger_stop(int rid, int row_id, lo_message msg);
+
 	//banking functions
 	int set_bank (uint32_t bank_start, lo_message msg);
 	int _set_bank (uint32_t bank_start, lo_address addr);
