@@ -31,7 +31,8 @@
 #include "pbd/compose.h"
 #include "pbd/convert.h"
 #include "pbd/signals.h"
-#include "pbd/stacktrace.h"
+
+#include "temporal/tempo.h"
 
 #include "ardour/types.h"
 #include "ardour/dB.h"
@@ -203,7 +204,7 @@ WaveView::set_samples_per_pixel (double samples_per_pixel)
 		begin_change ();
 
 		_props->samples_per_pixel = samples_per_pixel;
-		_bounding_box_dirty = true;
+		set_bbox_dirty ();
 
 		end_change ();
 	}
@@ -370,55 +371,33 @@ WaveView::compute_tips (ARDOUR::PeakData const& peak, WaveView::LineTips& tips,
                         double const effective_height)
 {
 	/* remember: canvas (and cairo) coordinate space puts the origin at the upper left.
+	 *
+	 * So, a sample value of 1.0 (0dbFS) will be computed as:
+	 *     (1.0 - 1.0) * 0.5 * effective_height
+	 * which evaluates to 0, or the top of the image.
+	 *
+	 * A sample value of -1.0 will be computed as
+	 *     (1.0 + 1.0) * 0.5 * effective height
+	 * which evaluates to effective height, or the bottom of the image.
+	 */
 
-	   So, a sample value of 1.0 (0dbFS) will be computed as:
+	const double pmax = (1.0 - peak.max) * floor (0.5 * effective_height);
+	const double pmin = (1.0 - peak.min) * floor (0.5 * effective_height);
 
-	         (1.0 - 1.0) * 0.5 * effective_height
-
-	   which evaluates to 0, or the top of the image.
-
-	   A sample value of -1.0 will be computed as
-
-	        (1.0 + 1.0) * 0.5 * effective height
-
-           which evaluates to effective height, or the bottom of the image.
-	*/
-
-	const double pmax = (1.0 - peak.max) * 0.5 * effective_height;
-	const double pmin = (1.0 - peak.min) * 0.5 * effective_height;
-
-	/* remember that the bottom of the image (pmin) has larger y-coordinates
-	   than the top (pmax).
-	*/
-
-	double spread = (pmin - pmax) * 0.5;
-
-	/* find the nearest pixel to the nominal center. */
-	const double center = round (pmin - spread);
-
-	if (spread < 1.0) {
-		/* minimum distance between line ends is 1 pixel, and we want it "centered" on a pixel,
-		   as per cairo single-pixel line issues.
-
-		   NOTE: the caller will not draw a line between these two points if the spread is
-		   less than 2 pixels. So only the tips.top value matters, which is where we will
-		   draw a single pixel as part of the outline.
-		 */
-		tips.top = center;
-		tips.bot = center + 1.0;
+	if (pmax * pmin < 0) {
+		/* signal crosses zero, round away from 0 */
+		tips.top = ceil (pmax);
+		tips.bot = floor (pmin);
 	} else {
-		/* round spread above and below center to an integer number of pixels */
-		spread = round (spread);
-		/* top and bottom are located equally either side of the center */
-		tips.top = center - spread;
-		tips.bot = center + spread;
+		tips.top = rint (pmax);
+		tips.bot = rint (pmin);
 	}
 
-	tips.top = min (effective_height, max (0.0, tips.top));
-	tips.bot = min (effective_height, max (0.0, tips.bot));
+	if (tips.top > tips.bot) {
+		tips.top = tips.bot = rint (0.5 * (tips.top + tips.bot));
+	}
 }
 
-
 Coord
 WaveView::y_extent (double s, Shape const shape, double const height)
 {
@@ -487,7 +466,7 @@ WaveView::draw_image (Cairo::RefPtr<Cairo::ImageSurface>& image, PeakData* peaks
 	Cairo::RefPtr<Cairo::Context> outline_context = Cairo::Context::create (images.outline);
 	Cairo::RefPtr<Cairo::Context> clip_context = Cairo::Context::create (images.clip);
 	Cairo::RefPtr<Cairo::Context> zero_context = Cairo::Context::create (images.zero);
-	wave_context->set_antialias (Cairo::ANTIALIAS_NONE);
+
 	outline_context->set_antialias (Cairo::ANTIALIAS_NONE);
 	clip_context->set_antialias (Cairo::ANTIALIAS_NONE);
 	zero_context->set_antialias (Cairo::ANTIALIAS_NONE);
@@ -548,6 +527,7 @@ WaveView::draw_image (Cairo::RefPtr<Cairo::ImageSurface>& image, PeakData* peaks
 		}
 
 	} else {
+		const int y_span = 2 * floor ((height - 1) * .5);
 
 		if (logscaled) {
 			for (int i = 0; i < n_peaks; ++i) {
@@ -578,7 +558,7 @@ WaveView::draw_image (Cairo::RefPtr<Cairo::ImageSurface>& image, PeakData* peaks
 					p.min = 0.0;
 				}
 
-				compute_tips (p, tips[i], height);
+				compute_tips (p, tips[i], y_span);
 				tips[i].spread = tips[i].bot - tips[i].top;
 			}
 
@@ -591,7 +571,7 @@ WaveView::draw_image (Cairo::RefPtr<Cairo::ImageSurface>& image, PeakData* peaks
 					tips[i].clip_min = true;
 				}
 
-				compute_tips (peaks[i], tips[i], height);
+				compute_tips (peaks[i], tips[i], y_span);
 				tips[i].spread = tips[i].bot - tips[i].top;
 			}
 
@@ -612,9 +592,12 @@ WaveView::draw_image (Cairo::RefPtr<Cairo::ImageSurface>& image, PeakData* peaks
 	/* ensure single-pixel lines */
 
 	wave_context->set_line_width (1.0);
+	wave_context->set_line_cap (Cairo::LINE_CAP_BUTT);
+	wave_context->set_line_join (Cairo::LINE_JOIN_ROUND);
 	wave_context->translate (0.5, 0.5);
 
 	outline_context->set_line_width (1.0);
+	outline_context->set_line_cap (Cairo::LINE_CAP_ROUND);
 	outline_context->translate (0.5, 0.5);
 
 	clip_context->set_line_width (1.0);
@@ -673,8 +656,7 @@ WaveView::draw_image (Cairo::RefPtr<Cairo::ImageSurface>& image, PeakData* peaks
 				clip_context->rel_line_to (0, min (clip_height, ceil(tips[i].spread + .5)));
 			} else {
 				outline_context->move_to (i, tips[i].top);
-				/* normal upper terminal dot */
-				outline_context->rel_line_to (0, -1.0);
+				outline_context->line_to (i, tips[i].top);
 			}
 		}
 
@@ -683,30 +665,26 @@ WaveView::draw_image (Cairo::RefPtr<Cairo::ImageSurface>& image, PeakData* peaks
 		outline_context->stroke ();
 
 	} else {
-		const int height_zero = floor(height * .5);
+		const int height_zero = floor ((height - 1) * .5);
 
 		for (int i = 0; i < n_peaks; ++i) {
 
-			/* waveform line */
-
-			if (tips[i].spread >= 2.0) {
+			bool connected_segment = false;
+			/* http://lac.linuxaudio.org/2013/papers/36.pdf Fig3 */
+			if (i + 1 == n_peaks) {
+				wave_context->move_to (i, tips[i].top);
+				wave_context->line_to (i, tips[i].bot);
+			} else if (tips[i].top >= tips[i + 1].bot) {
+				connected_segment = true;
+				wave_context->move_to (i - .5, tips[i].bot);
+				wave_context->line_to (i + .5, tips[i + 1].bot);
+			} else if (tips[i].bot <= tips[i + 1].top) {
+				connected_segment = true;
+				wave_context->move_to (i - .5, tips[i].top);
+				wave_context->line_to (i + .5, tips[i + 1].top);
+			} else {
 				wave_context->move_to (i, tips[i].top);
 				wave_context->line_to (i, tips[i].bot);
-			}
-
-			/* draw square waves and other discontiguous points clearly */
-			if (i > 0) {
-				if (tips[i-1].top + 2 < tips[i].top) {
-					wave_context->move_to (i-1, tips[i-1].top);
-					wave_context->line_to (i-1, (tips[i].bot + tips[i-1].top)/2);
-					wave_context->move_to (i, (tips[i].bot + tips[i-1].top)/2);
-					wave_context->line_to (i, tips[i].top);
-				} else if (tips[i-1].bot > tips[i].bot + 2) {
-					wave_context->move_to (i-1, tips[i-1].bot);
-					wave_context->line_to (i-1, (tips[i].top + tips[i-1].bot)/2);
-					wave_context->move_to (i, (tips[i].top + tips[i-1].bot)/2);
-					wave_context->line_to (i, tips[i].bot);
-				}
 			}
 
 			/* zero line, show only if there is enough spread
@@ -718,58 +696,32 @@ WaveView::draw_image (Cairo::RefPtr<Cairo::ImageSurface>& image, PeakData* peaks
 				zero_context->rel_line_to (1.0, 0);
 			}
 
-			if (tips[i].spread > 1.0) {
-				bool clipped = false;
-				/* outline/clip indicators */
-				if (_global_show_waveform_clipping && tips[i].clip_max) {
-					clip_context->move_to (i, tips[i].top);
-					/* clip-indicating upper terminal line */
-					clip_context->rel_line_to (0, min (clip_height, ceil(tips[i].spread + 0.5)));
-					clipped = true;
-				}
+			bool clipped = false;
+			/* outline/clip indicators */
+			if (_global_show_waveform_clipping && tips[i].clip_max) {
+				clip_context->move_to (i, tips[i].top);
+				/* clip-indicating upper terminal line */
+				clip_context->rel_line_to (0, min (clip_height, ceil(tips[i].spread + 0.5)));
+				clipped = true;
+			}
 
-				if (_global_show_waveform_clipping && tips[i].clip_min) {
-					clip_context->move_to (i, tips[i].bot);
-					/* clip-indicating lower terminal line */
-					clip_context->rel_line_to (0, - min (clip_height, ceil(tips[i].spread + 0.5)));
-					clipped = true;
-				}
+			if (_global_show_waveform_clipping && tips[i].clip_min) {
+				clip_context->move_to (i, tips[i].bot + 1);
+				/* clip-indicating lower terminal line */
+				clip_context->rel_line_to (0, - min (clip_height, ceil(tips[i].spread + 0.5)));
+				clipped = true;
+			}
 
-				if (!clipped && tips[i].spread > 2.0) {
-					/* only draw the outline if the spread
-					   implies 3 or more pixels (so that we see 1
-					   white pixel in the middle).
-					*/
-					outline_context->move_to (i, tips[i].bot);
-					/* normal lower terminal dot; line moves up */
-					outline_context->rel_line_to (0, -1.0);
+			if (!connected_segment && !clipped && tips[i].spread > 2.0) {
+				/* only draw the outline if the spread
+				 * implies 3 or more pixels (so that we see 1
+				 * white pixel in the middle).
+				 */
+				outline_context->move_to (i, tips[i].bot);
+				outline_context->line_to (i, tips[i].bot);
 
-					outline_context->move_to (i, tips[i].top);
-					/* normal upper terminal dot, line moves down */
-					outline_context->rel_line_to (0, 1.0);
-				}
-			} else {
-				bool clipped = false;
-				/* outline/clip indicator */
-				if (_global_show_waveform_clipping && (tips[i].clip_max || tips[i].clip_min)) {
-					clip_context->move_to (i, tips[i].top);
-					/* clip-indicating upper / lower terminal line */
-					clip_context->rel_line_to (0, 1.0);
-					clipped = true;
-				}
-
-				if (!clipped) {
-					/* special case where only 1 pixel of
-					 * the waveform line is drawn (and
-					 * nothing else).
-					 *
-					 * we draw a 1px "line", pretending
-					 * that the span is 1.0 (whether it is
-					 * zero or 1.0)
-					 */
-					wave_context->move_to (i, tips[i].top);
-					wave_context->rel_line_to (0, 1.0);
-				}
+				outline_context->move_to (i, tips[i].top);
+				outline_context->line_to (i, tips[i].top);
 			}
 		}
 
@@ -934,6 +886,8 @@ WaveView::process_draw_request (boost::shared_ptr<WaveViewDrawRequest> req)
 		return;
 	}
 
+	(void) Temporal::TempoMap::fetch();
+
 	WaveViewProperties const& props = req->image->props;
 
 	const int n_peaks = props.get_width_pixels ();
@@ -1016,7 +970,21 @@ WaveView::render (Rect const & area, Cairo::RefPtr<Cairo::Context> context) cons
 	Rect self;
 
 	if (!get_item_and_draw_rect_in_window_coords (area, self, draw)) {
-		assert(true);
+		assert(false);
+		return;
+	}
+
+	if (_props->height < 1) {
+			if (_props->channel % 2) {
+				return;
+			}
+			context->rectangle (draw.x0, draw.y0, draw.width (), draw.height ());
+			if (1 == (_props->channel % 3)) {
+				set_source_rgba (context, _props->zero_color);
+			} else {
+				set_source_rgba (context, _props->fill_color);
+			}
+			context->fill ();
 		return;
 	}
 
@@ -1181,7 +1149,7 @@ WaveView::compute_bounding_box () const
 		_bounding_box = Rect ();
 	}
 
-	_bounding_box_dirty = false;
+	set_bbox_clean ();
 }
 
 void
@@ -1193,7 +1161,7 @@ WaveView::set_height (Distance height)
 		_props->height = height;
 		_draw_image_in_gui_thread = true;
 
-		_bounding_box_dirty = true;
+		set_bbox_dirty ();
 		end_change ();
 	}
 }
@@ -1205,7 +1173,7 @@ WaveView::set_channel (int channel)
 		begin_change ();
 		_props->channel = channel;
 		reset_cache_group ();
-		_bounding_box_dirty = true;
+		set_bbox_dirty ();
 		end_change ();
 	}
 }
@@ -1324,10 +1292,16 @@ WaveView::set_global_logscaled (bool yn)
 	}
 }
 
+void
+WaveView::clear_cache ()
+{
+	WaveViewCache::get_instance()->clear_cache ();
+}
+
 samplecnt_t
 WaveView::region_length() const
 {
-	return _region->length() - (_props->region_start - _region->start());
+	return _region->length_samples() - (_props->region_start - _region->start_sample());
 }
 
 samplepos_t
@@ -1349,7 +1323,7 @@ WaveView::set_region_start (sampleoffset_t start)
 
 	begin_change ();
 	_props->region_start = start;
-	_bounding_box_dirty = true;
+	set_bbox_dirty ();
 	end_change ();
 }
 
@@ -1364,9 +1338,9 @@ WaveView::region_resized ()
 	}
 
 	begin_change ();
-	_props->region_start = _region->start();
-	_props->region_end = _region->start() + _region->length();
-	_bounding_box_dirty = true;
+	_props->region_start = _region->start_sample();
+	_props->region_end = _region->start_sample() + _region->length_samples();
+	set_bbox_dirty ();
 	end_change ();
 }
 
