@@ -46,7 +46,7 @@ while [ $# -gt 0 ] ; do
 			PROGRAM_NAME=Mixbus
 			PROGRAM_KEY=Mixbus
 			PRODUCT_NAME=Mixbus
-			MANUAL_NAME="mixbus${major_version}-live-manual"
+			MANUAL_NAME="mixbus-live-manual"
 			shift ;;
 		--mixbus32c)
 			MIXBUS=1
@@ -57,7 +57,7 @@ while [ $# -gt 0 ] ; do
 			PROGRAM_KEY=Mixbus32C
 			PROGRAM_NAME=Mixbus32C-${PROGRAM_VERSION}
 			PROGRAM_VERSION=""
-			MANUAL_NAME="mixbus32c-${major_version}-live-manual"
+			MANUAL_NAME="mixbus32c-live-manual"
 			shift ;;
 		--chanstrip) HARRISONCHANNELSTRIP=$2 ; shift; shift ;;
 	esac
@@ -148,7 +148,7 @@ cp build/libs/audiographer/audiographer-*.dll $DESTDIR/bin/
 cp build/libs/fst/ardour-vst-scanner.exe $DESTDIR/bin/ || true
 cp build/libs/fst/ardour-vst3-scanner.exe $DESTDIR/bin/ || true
 cp build/session_utils/*-*.exe $DESTDIR/bin/ || true
-cp build/luasession/ardour6-lua.exe $DESTDIR/bin/ || true
+cp build/luasession/ardour*-lua.exe $DESTDIR/bin/ || true
 cp `ls -t build/gtk2_ardour/ardour-*.exe | head -n1` $DESTDIR/bin/${PRODUCT_EXE}
 
 mkdir -p $DESTDIR/lib/gtk-2.0/engines
@@ -159,7 +159,15 @@ cp $PREFIX/bin/*.yes $DESTDIR/bin/ || true
 cp $PREFIX/lib/*.dll $DESTDIR/bin/
 # special case libportaudio (wasapi), old stack has no wasapi and hence no .xp
 cp $PREFIX/bin/libportaudio-2.xp $DESTDIR/bin/ || cp $PREFIX/bin/libportaudio-2.dll $DESTDIR/bin/libportaudio-2.xp
+
+# prefer system-wide DLL
 rm -rf $DESTDIR/bin/libjack*.dll
+# Also for these (even though M$ recommends to bundle these [1],
+# there is no single set that works on all target systems, particularly
+# since some plugins also rely on it.
+# [1] https://docs.microsoft.com/en-us/windows/win32/debug/calling-the-dbghelp-library
+rm -rf $DESTDIR/bin/dbghelp*.dll
+rm -rf $DESTDIR/bin/dbgcore*.dll
 
 cp `find build/libs/surfaces/ -iname "*.dll"` $ALIBDIR/surfaces/
 cp `find build/libs/backends/ -iname "*.dll"` $ALIBDIR/backends/
@@ -300,7 +308,7 @@ if test x$WITH_GRATIS_X42_LV2 != x ; then
 
 	echo "Adding gratis x42 Plugins"
 
-	for proj in x42-midifilter x42-stereoroute setBfree x42-avldrums x42-limiter x42-tuner; do
+	for proj in x42-autotune x42-midifilter x42-stereoroute setBfree x42-avldrums x42-limiter x42-tuner; do
 		X42_VERSION=$(curl -s -S http://x42-plugins.com/x42/win/${proj}.latest.txt)
 		rsync -a -q --partial \
 			rsync://x42-plugins.com/x42/win/${proj}-lv2-${WARCH}-${X42_VERSION}.zip \
@@ -342,15 +350,39 @@ if test -n "$MIXBUS"; then
 
 	cp "${SRCCACHE}/harrison_vamp.${WARCH}.dll" \
 		"$ALIBDIR/vamp/harrison_vamp.dll"
+
+	# Mixbus Bundled Media Content
+	curl -s -S --fail -#  \
+		-z "${SRCCACHE}/MixbusBundledMedia.zip" \
+		-o "${SRCCACHE}/MixbusBundledMedia.zip" \
+		"http://builder.harrisonconsoles.com/pub/share/MixbusBundledMedia.zip"
+
+	if test -f "${SRCCACHE}/MixbusBundledMedia.zip"; then
+		echo "Adding Mixbus Bundled Content"
+		rm -f $DESTDIR/share/${LOWERCASE_DIRNAME}/media/*.*
+		unzip -q -o -d "$DESTDIR/share/${LOWERCASE_DIRNAME}/media/" "${SRCCACHE}/MixbusBundledMedia.zip"
+	fi
+else
+        echo "Fetching Ardour bundled media"
+	curl -s -S --fail -#  \
+		-z "${SRCCACHE}/ArdourBundledMedia.zip" \
+		-o "${SRCCACHE}/ArdourBundledMedia.zip" \
+		"http://stuff.ardour.org/loops/ArdourBundledMedia.zip"
+
+	if test -f "${SRCCACHE}/ArdourBundledMedia.zip"; then
+		echo "Adding Ardour Bundled Content"
+		rm -f $DESTDIR/share/${LOWERCASE_DIRNAME}/media/*.*
+		unzip -q -o -d "$DESTDIR/share/${LOWERCASE_DIRNAME}/media/" "${SRCCACHE}/ArdourBundledMedia.zip"
+	fi
 fi
 
 ################################################################################
 
 if test x$DEMO_SESSION_URL != x ; then
-	mkdir -p $Shared/sessions
+	mkdir -p $DESTDIR/share/${LOWERCASE_DIRNAME}/sessions
 	DEMO_SESSIONS=$(curl -s -S --fail $DEMO_SESSION_URL/index.txt)
 	for demo in $DEMO_SESSIONS; do
-		curl -s -S --fail -# -o $Shared/sessions/$demo $DEMO_SESSION_URL/$demo
+		curl -s -S --fail -# -o $DESTDIR/share/${LOWERCASE_DIRNAME}/sessions/$demo $DEMO_SESSION_URL/$demo
 	done
 fi
 
@@ -394,6 +426,7 @@ RequestExecutionLevel admin
 InstallDir "\$${PGF}\\${PRODUCT_ID}"
 InstallDirRegKey HKLM "Software\\${PRODUCT_NAME}\\${PRODUCT_ID}\\$WARCH" "Install_Dir"
 !define MUI_ICON "share\\${PRODUCT_ICON}"
+!define MUI_UNICON "share\\${PRODUCT_ICON}"
 
 EOF
 
@@ -401,10 +434,10 @@ if test -n "$MIXBUS"; then
 
 # TODO: proper welcome/finish text.
 	cat >> $NSISFILE << EOF
-!define MUI_FINISHPAGE_TITLE "Welcome to Harrison Mixbus"
-!define MUI_FINISHPAGE_TEXT "Thanks for your purchase of Mixbus!\$\\r\$\\nYou will find the Mixbus application in the Start Menu (or the All Apps panel for Windows 8) \$\\r\$\\nClick the link below to view the Mixbus manual, and learn ways to get involved with the Mixbus community."
-!define MUI_FINISHPAGE_LINK "Mixbus Manual"
-!define MUI_FINISHPAGE_LINK_LOCATION "http://www.harrisonconsoles.com/mixbus/${MANUAL_NAME}/"
+!define MUI_FINISHPAGE_TITLE "Welcome to Harrison ${PROGRAM_NAME}"
+!define MUI_FINISHPAGE_TEXT "Thanks for your purchase of ${PROGRAM_NAME}!\$\\r\$\\nYou will find the ${PROGRAM_NAME} application in the Start Menu (or the All Apps panel for Windows 8) \$\\r\$\\nClick the link below to view the ${PROGRAM_NAME} manual, and learn ways to get involved with the Mixbus community."
+!define MUI_FINISHPAGE_LINK "${PROGRAM_NAME} Manual"
+!define MUI_FINISHPAGE_LINK_LOCATION "https://rsrc.harrisonconsoles.com/mixbus/${MANUAL_NAME}/"
 !define MUI_FINISHPAGE_NOREBOOTSUPPORT
 EOF
 
@@ -477,7 +510,7 @@ SectionEnd
 EOF
 else
 	cat >> $NSISFILE << EOF
-Section "Harrison XT plugins and a-/ACE plugin GUIs" SecXT
+Section "Harrison XT plugins and ACE plugin GUIs" SecXT
   SetOutPath \$INSTDIR\\lib\\${LOWERCASE_DIRNAME}\\LV2
   File LV2\\.harrison_version.txt
   File /r LV2\\*.lv2
